-- SAMPLE CODE for the Advanced CTE Presentation
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG;


use [cte_demo];  -- download the cte_demo database from http://stevestedman.com/?p=2839
go


--For more info on recursive CTEs see http://stevestedman.com/?p=2914

--Look at all Departments
SELECT * 
  FROM Departments;



--Find the top level
SELECT * 
  FROM Departments
 WHERE parent IS NULL;



--Building the anchor query
SELECT id, department, parent, 1 AS lvl 
  FROM Departments
 WHERE parent IS NULL;



--Turn into a CTE
;WITH DepartmentCTE AS
(
	SELECT id, department, parent, 1 AS lvl 
	  FROM Departments
	 WHERE parent IS NULL
) 
SELECT * FROM DepartmentCTE;


-- now just those that are not top level
-- the recursive query

SELECT id, department, parent, 2 AS lvl 
  FROM Departments AS d
 WHERE parent IS NOT NULL;


--Add all second level departments
;WITH DepartmentCTE AS
(
	SELECT id, Department, parent, 1 AS lvl 
	  FROM Departments
	 WHERE parent IS NULL

 	UNION ALL 

	SELECT id, Department, parent, 2 AS lvl 
	  FROM Departments AS d
	 WHERE parent IS NOT NULL
) 
SELECT * 
  FROM DepartmentCTE;





--Add recursion
;WITH DepartmentCTE AS
(
	SELECT id, Department, parent, 1 AS lvl 
  	  FROM Departments
	 WHERE parent IS NULL

	 UNION ALL 

	SELECT d.id, d.Department, d.parent, dCTE.lvl + 1 AS lvl 
	  FROM Departments AS d 
	 INNER JOIN DepartmentCTE AS dCTE ON d.parent = dCTE.id
) 
SELECT * 
  FROM DepartmentCTE;




-- Non CTE to get department levels
SELECT d1.department as 'Top_level',
       d2.department as 'Child_level'
  FROM Departments d1
 RIGHT JOIN Departments d2 on d1.id = d2.parent;




SELECT d2.id, 
       d2.Department, 
	   d2.Parent,
       CASE WHEN d2.Parent is NULL THEN 0 ELSE 1 END AS lvl
  FROM Departments d1
 RIGHT JOIN Departments d2 ON d1.id = d2.parent;





EXEC FillMoreDepartments;


SELECT id, Department, Parent
  FROM Departments;




-- recursion CTE to list departments (same as earlier)
;WITH DepartmentCTE AS
(
	SELECT id, Department, parent, 1 AS lvl 
  	  FROM Departments
	 WHERE parent IS NULL

	 UNION ALL 

	SELECT d.id, d.Department, d.parent, dCTE.lvl + 1 AS lvl 
	  FROM Departments AS d 
	 INNER JOIN DepartmentCTE AS dCTE ON d.parent = dCTE.id
) 
SELECT * 
  FROM DepartmentCTE;





use [cte_demo];  
GO
------------------------------------------------
-- RECURSIVE CTE COMPARED TO RECURSIVE FUNCTIONS
------------------------------------------------

/*sum of parts
 1: 1
 2: 1+2 = 3
 3: 1+2+3 = 6
 4: 1+2+3+4 = 10
 .
 .
 .
 10: 1+2+3+4+5+6+7+8+9+10 = 55

 For more info on recursive CTEs see http://stevestedman.com/?p=2914
*/




-- with NON CTE way to do recursion.
GO
CREATE FUNCTION dbo.SumParts (@Num INTEGER)
RETURNS INTEGER
AS
BEGIN
    DECLARE @ResultVar AS INTEGER = 0;
    IF (@Num > 0)
	BEGIN
        SET @ResultVar = @Num + dbo.SumParts(@Num - 1);
	END
    RETURN @ResultVar;
END
GO

SELECT dbo.SumParts(10);

SELECT dbo.SumParts(3);

SELECT dbo.SumParts(100);  -- this will throw an error






-- build sum of parts as a CTE.

-- Anchor Query
SELECT 1 AS CountNumber, 1 AS GrandTotal;





 
-- Next, the anchor query is placed into a CTE
;WITH SumPartsCTE AS
( 
	SELECT 1 AS CountNumber, 1 AS GrandTotal
)
SELECT * 
  FROM SumPartsCTE;




 

-- Recursive Query
SELECT CountNumber + 1,
       GrandTotal + CountNumber + 1






--Next, add a UNION ALL statement into the existing CTE and add the recursive part of the query, 
--which should run fine, but does it?
;WITH SumPartsCTE AS
(
	SELECT 1 AS CountNumber, 1 AS GrandTotal

    UNION ALL

	SELECT CountNumber + 1,
           GrandTotal + CountNumber + 1
      FROM SumPartsCTE
)
SELECT * 
  FROM SumPartsCTE;
--Maximum recursion exhausted error.




-- Add a where clause that will terminate the recursion.

;WITH SumPartsCTE AS
(
	SELECT 1 AS CountNumber, 1 AS GrandTotal

     UNION ALL

    SELECT CountNumber + 1,
           GrandTotal + CountNumber + 1
      FROM SumPartsCTE
     WHERE CountNumber < 10
)
SELECT * 
  FROM SumPartsCTE;




;WITH SumPartsCTE AS
(
	SELECT 1 AS CountNumber, 1 AS GrandTotal

     UNION ALL

    SELECT CountNumber + 1,
           GrandTotal + CountNumber + 1
      FROM SumPartsCTE
     WHERE CountNumber < 10
)
SELECT MAX(GrandTotal) as SumOfParts
  FROM SumPartsCTE;









 -- But what if we want to go futher than 100 on the recursion
;WITH SumPartsCTE AS
(
	SELECT 1 AS CountNumber, 1 AS GrandTotal

     UNION ALL

    SELECT CountNumber + 1,
           GrandTotal + CountNumber + 1
      FROM SumPartsCTE
     WHERE CountNumber < 150
)
SELECT MAX(GrandTotal) as SumOfParts
  FROM SumPartsCTE



--OPTION (MAXRECURSION 200);