-- SAMPLE CODE for the Advanced CTE Presentation
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG;

use [cte_demo];
GO






-- Delete

-- simple SELECT to find customers name Williams
SELECT *
  FROM Customer 
 WHERE LastName like 'Williams';


-- simple CTE to do the same
;WITH CustomerCTE AS
( 
	SELECT *
      FROM Customer 
     WHERE LastName like 'Williams'
) 
SELECT * FROM CustomerCTE;


--NOTE above queries produce 7 results




BEGIN TRANSACTION;

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Williams';


	;WITH CustomerCTE AS
	( 
		SELECT *
			FROM Customer 
			WHERE LastName like 'Williams'
	) 
	DELETE FROM CustomerCTE;


	SELECT * FROM Customer WHERE LastName like 'Williams';
	SELECT COUNT(1) FROM Customer;

ROLLBACK TRANSACTION;






-- deleting from multiple base tables
BEGIN TRANSACTION;

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Williams';


	;WITH CustomerCTE AS
	( 
		SELECT c.*
			FROM Customer AS c
			INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
			WHERE c.LastName like 'Williams'
	) 
	DELETE FROM CustomerCTE;


	SELECT * FROM Customer WHERE LastName like 'Williams';
	SELECT COUNT(1) FROM Customer;

ROLLBACK TRANSACTION;
   



------------------------------------------
-- INSERTING TO A CTE
------------------------------------------




-- simple SELECT to find specific customers
SELECT *
  FROM Customer 
 WHERE LastName like 'Stedman';


-- simple CTE to do the same
;WITH CustomerCTE AS
( 
	SELECT *
      FROM Customer 
     WHERE LastName like 'Stedman'
) 
SELECT * FROM CustomerCTE;







BEGIN TRANSACTION;

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Stedman';

	;WITH CustomerCTE AS
	( 
		SELECT *
			FROM Customer 
			WHERE LastName like 'Stedman'
	) 
	INSERT INTO CustomerCTE
            (CustomerID, FirstName, LastName)
     VALUES (99999, 'Steve', 'Stedman');

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Stedman';

ROLLBACK TRANSACTION;




-- selecting multiple Base Tables will this work?

BEGIN TRANSACTION;

	SELECT COUNT(1) AS NumCustomers FROM Customer;
	SELECT COUNT(1) AS NumInvoices FROM SalesInvoice;
	SELECT * FROM Customer WHERE LastName like 'Stedman';

	;WITH CustomerCTE AS
	( 
		SELECT c.*
		  FROM Customer AS c
		 INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
		 WHERE c.LastName like 'Stedman'
	) 
	INSERT INTO CustomerCTE
            (CustomerID, FirstName, LastName)
     VALUES (99999, 'Steve', 'Stedman');

	SELECT COUNT(1) AS NumCustomers FROM Customer;
	SELECT COUNT(1) AS NumInvoices FROM SalesInvoice;
	SELECT * FROM Customer WHERE LastName like 'Stedman';

ROLLBACK TRANSACTION;







-- INSERT to multiple base tables

;WITH CustomerCTE AS
( 
	SELECT c.*, si.comment
		FROM Customer AS c
		INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
		WHERE c.LastName like 'Stedman'
) 
INSERT INTO CustomerCTE
        (CustomerID, FirstName, LastName, comment)
    VALUES (99999, 'Steve', 'Stedman', 'Test Comment');
















-- Update

-- simple SELECT to find specific customers
SELECT *
  FROM Customer 
 WHERE LastName like 'Williams';


-- simple CTE to do the same
;WITH CustomerCTE AS
( 
	SELECT *
      FROM Customer 
     WHERE LastName like 'Williams'
) 
SELECT * FROM CustomerCTE;





BEGIN TRANSACTION;

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Williams';

	;WITH CustomerCTE AS
	( 
		SELECT c.*
		  FROM Customer AS c
		 WHERE c.LastName like 'Williams'
	) 
	UPDATE CustomerCTE
	   SET LastName = 'Willie';

	SELECT COUNT(1) FROM Customer;
	SELECT * FROM Customer WHERE LastName like 'Williams';
	SELECT * FROM Customer WHERE LastName like 'Willie';

ROLLBACK TRANSACTION;




-- updating with multiple Base Tables will this work?

BEGIN TRANSACTION;

	SELECT c.*, si.Comment 
	  FROM Customer AS c
	 INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
	 WHERE c.LastName like 'Williams';

	;WITH CustomerCTE AS
	( 
		SELECT c.*, si.Comment 
		  FROM Customer AS c
		 INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
		 WHERE c.LastName like 'Williams'
	) 
	UPDATE CustomerCTE
	   SET Comment = 'Some Comment Goes Here';

	SELECT c.*, si.Comment 
	  FROM Customer AS c
	 INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
	 WHERE c.LastName like 'Williams';

ROLLBACK TRANSACTION;









-- updating multiple base tables
;WITH CustomerCTE AS
( 
	SELECT c.*, si.Comment 
		FROM Customer AS c
		INNER JOIN SalesInvoice AS si ON si.CustomerID = c.CustomerID
		WHERE c.LastName like 'Williams'
) 
UPDATE CustomerCTE
    SET Comment = 'Some Comment Goes Here',
        CompanyName = 'Willies Toys';
	   

