-- SAMPLE CODE for the Advanced CTE Presentation
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG;


use [cte_demo];  -- download the cte_demo database from http://stevestedman.com/?p=2839
GO

-- The old way of doing it.
CREATE TABLE Numbers
(
	Number INTEGER Identity
);
GO

SET NOCOUNT ON;
INSERT INTO Numbers DEFAULT VALUES;
GO 1000


SELECT * FROM NUMBERS;
GO

DROP TABLE Numbers;




-- Numbers as a CTE

;WITH Numbers (N) AS 
(
  SELECT 1 
   UNION ALL 
  SELECT 1 + N FROM Numbers 
   WHERE N < 1000
)
SELECT * 
  FROM Numbers
OPTION (MAXRECURSION 1000);




------------------------------------------------------
-- Common use case 2 - Finding holes
------------------------------------------------------


--Take a look at the SalesInvoice table
SELECT TOP 10 *
  FROM SalesInvoice;






-- not lets look at the hour of day that orders have been placed

SELECT datepart(HOUR, OrderDate) as HourOfDay,
       COUNT(1) as numOrders
  FROM SalesInvoice
 GROUP BY datepart(HOUR, OrderDate)
 ORDER BY numOrders ASC;


-- smaller set of data
SELECT datepart(HOUR, OrderDate) as HourOfDay,
       COUNT(1) as numOrders
  FROM SalesInvoice
 WHERE OrderDate < '02/01/2006'
 GROUP BY datepart(HOUR, OrderDate)
 ORDER BY numOrders ASC;





-- the numbers table alternative
;WITH Numbers (N) AS
(
  SELECT 0
   UNION ALL 
  SELECT 1 + N FROM Numbers 
   WHERE N < 23
) 
SELECT N 
  FROM Numbers ;

-- why do we use N < 23 instead of N < 24 for 24 hours in the day?





-- Multiple CTEs to find the hours of the day MISSING
;WITH Numbers (N) AS
(
  SELECT 0
   UNION ALL 
  SELECT 1 + N FROM Numbers 
   WHERE N < 23
)
, 
OrderHours (HourOfDay, TheCount) AS
(
  SELECT DATEPART(HOUR, OrderDate) as HourOfDay,
         COUNT(1) AS TheCount
    FROM SalesInvoice
   WHERE OrderDate < '02/01/2006'
   GROUP BY DATEPART(HOUR, OrderDate)
)
SELECT n.N AS HourOfDay, ISNULL(oh.TheCount, 0) OrderCount
  FROM Numbers n
  LEFT JOIN OrderHours oh ON n.N = oh.HourOfDay
  --WHERE oh.TheCount is NULL
 ORDER BY TheCount ASC;









------------------------------------------------------
-- Common use case 2 - Scrubbing Duplicates
------------------------------------------------------


-- Look at Customer Table
SELECT *
  FROM Customer;





-- Find duplicate names
SELECT LastName, FirstName, COUNT(*) AS NumDups
  FROM Customer
 GROUP BY LastName, FirstName
 ORDER BY NumDups DESC;






-- 2nd Window: Look at Customer Table. We will keep CustID 9 as the lowest duplicateID
SELECT *
  FROM Customer
 WHERE LastName = 'Adams' 
   AND FirstName = 'Linda';





-- GOAL: Keep record with the lowest ID
-- Find Dupicates and thier order number

SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
FROM Customer
ORDER BY LastName, FirstName;





-- ERROR: We want to see numbers that are not the lowest (Does not work)

SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
FROM Customer
WHERE DupNum > 1
ORDER BY LastName, FirstName

-- what about a derived table query

SELECT * 
  FROM (
        SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
          FROM Customer
	   ) AS t
WHERE DupNum > 1
ORDER BY LastName, FirstName;


-- No try DELETE using a derived table
DELETE FROM (
             SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
               FROM Customer
	        ) AS t
WHERE DupNum > 1;






--Put in a CTE
WITH CustomerCTE AS
(
	SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
	  FROM Customer
)
SELECT * 
FROM CustomerCTE
WHERE DupNum > 1;




BEGIN TRANSACTION;

	SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
		  FROM Customer;

	--Change SELECT * to DELETE
	WITH CustomerCTE AS
	(
		SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
		  FROM Customer
	)
	DELETE 
	  FROM CustomerCTE
	 WHERE DupNum > 1;



	SELECT *, ROW_NUMBER() OVER(PARTITION BY LastName, FirstName ORDER BY CustomerID) AS DupNum 
		  FROM Customer;


	SELECT LastName, FirstName, COUNT(*) AS NumDups
	  FROM Customer
	 GROUP BY LastName, FirstName
	 ORDER BY NumDups DESC;

ROLLBACK TRANSACTION;