-- SAMPLE CODE for the Advanced CTE Presentation
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt

RAISERROR ('Dont run the whole script, just run sections at a time', 20, 1)  WITH LOG

USE [cte_demo];
GO


-- Calculating the Fibonacci sequence 
-- http://stevestedman.com/?p=1142

-- Definition of FIBONACCI NUMBER. : an integer in the infinite sequence 1, 1, 2, 3, 5, 8, 13,  
--  of which the first two terms are 1 and 1 and each succeeding term is the sum of the two immediately preceding. 

-- Build the anchor query
;WITH Fibonacci (PrevN, N) AS 
( 
	SELECT 0, 1
) 
SELECT * 
  FROM Fibonacci;



-- add the recursive query
;WITH Fibonacci (PrevN, N) AS 
(
  SELECT 0, 1
   UNION ALL 
  SELECT N, PrevN + N 
    FROM Fibonacci 
   WHERE N < 1000000000
) 
SELECT * 
  FROM Fibonacci;


 
 -- select just the column we want
;WITH Fibonacci (PrevN, N) AS 
( SELECT 0, 1
   UNION ALL 
  SELECT N, PrevN + N 
    FROM Fibonacci 
   WHERE N < 1000000000
) 
 SELECT N as Fibo
   FROM Fibonacci ;



---- Fibonacci with output as a csv
---- http://stevestedman.com/?p=320
--WITH Fibonacci (PrevN, N) AS 
--( SELECT 0, 1
--   UNION ALL 
--  SELECT N, PrevN + N 
--    FROM Fibonacci 
--   WHERE N < 1000000000
--) 
-- SELECT Substring((SELECT CAST(', ' as varchar(max)) + 
--		CAST(N as varchar(max))
--   FROM Fibonacci 
--  FOR XML PATH('')),3,10000000) AS list;
  
  









-- Calculating Factorials with a CTE : Step 1 Build the Anchor Query
-- http://stevestedman.com/?p=1166

-- Factorial is the product of all positive integers less than or equal to n. For example,
--  5! = 5 x 4 x 3 x 2 x 1 = 120


-- Not yet recursive, only calculates 1!
--   A start, but not very interesting
;WITH Factorial (N, Factorial) AS 
(
  SELECT 1, 1
) 
 SELECT N, Factorial
   FROM Factorial;

 

  
-- Calculating Factorials with a CTE : Step 2 Build the recursive query
-- http://stevestedman.com/?p=1166
-- This works great for 5!
;WITH Factorial (N, Factorial) AS 
( 
  SELECT 1, 1
   UNION ALL -- here is where it gets recursive
  SELECT N + 1, (N + 1) * Factorial
    FROM Factorial -- reference back to the CTE
   WHERE N < 5 -- abort when we get to 5!
) 
 SELECT N, Factorial
   FROM Factorial;
 
 
  
  
-- Calculating Factorials with a CTE : Step 3
-- http://stevestedman.com/?p=1166
-- this one throws an error of "Arithmetic overflow error converting expression to data type int."
;WITH Factorial (N, Factorial) AS 
( 
  SELECT 1, 1
   UNION ALL -- here is where it gets recursive
  SELECT N + 1, (N + 1) * Factorial
    FROM Factorial -- reference back to the CTE
   WHERE N < 20 -- abort when we get to 20!
) 
 SELECT N, Factorial
   FROM Factorial;
   
   
   
   
   
   
   
-- Calculating Factorials with a CTE  : Step 4  - GO BIG
-- http://stevestedman.com/?p=1166
;WITH Factorial (N, Factorial) AS 
(
  SELECT 1, CAST(1 as BIGINT) -- Cast to BIGINT to avoid overflow
   UNION ALL -- here is where it gets recursive
  SELECT N + 1, (N + 1) * Factorial
    FROM Factorial -- reference back to the CTE
   WHERE N < 20 -- abort when we get to 20!
) 
 SELECT N, Factorial
   FROM Factorial;
   
   
   
   
   
   
   
   
 
 
 
-- Calculating Factorials with a CTE  : Step 5  - GO REALLY BIG
-- http://stevestedman.com/?p=1166
;WITH Factorial (N, Factorial) AS 
(
  SELECT 1, CAST(1 as NUMERIC(38,0)) -- Cast to NUMERIC to avoid overflow
   UNION ALL -- here is where it gets recursive
  SELECT N + 1, (N + 1) * Factorial
    FROM Factorial -- reference back to the CTE
   WHERE N < 33 -- abort when we get to 33!
) 
 SELECT N, Factorial
   FROM Factorial; 
 
 
 
 --previous query, try 34...  What now?
 
 
 















------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
-- Additional Samples
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------














 
 
 
 
 
 
 
 
 
 
 
 
 --Insert Statements from a CTE, but not inside the CTE
 -- http://stevestedman.com/?p=1153
DECLARE @NumTableVar TABLE(n int);

;WITH Numbers (N) AS 
( SELECT 1 
   UNION ALL 
  SELECT 1 + N FROM Numbers 
   WHERE N < 1000
) 
INSERT INTO @NumTableVar(n)
 SELECT N 
   FROM Numbers 
 OPTION (MAXRECURSION 1001);
  
 select * from @NumTableVar;
 
 
 
 
 
 
 

--proving that the CTE is run multiple times.
-- Remember Ctrl+M to turn on Execution Plan
;WITH deptCTE(id, department, parent) AS 
(	SELECT id, department, parent
	  FROM Departments) 
SELECT q1.department, q2.department as subDepartment
  FROM deptCTE q1
 INNER JOIN deptCTE q2 on q1.id = q2.parent
 WHERE q1.parent is null; 

 
 
 
 
 
 
 
 ---CTE for Subquery Re-use  compared to a view.
;WITH deptCTE(id, department, parent) AS 
(	SELECT id, department, parent
	  FROM Departments) 
SELECT q1.department, q2.department as subDepartment
  FROM deptCTE q1
 INNER JOIN deptCTE q2 on q1.id = q2.parent
 WHERE q1.parent is null; 
 
GO
CREATE VIEW [dbo].[DeptView]
AS 
 SELECT id, department, parent
	  FROM Departments;
GO

SELECT q1.department, q2.department as subDepartment
  FROM DeptView q1
 INNER JOIN DeptView q2 on q1.id = q2.parent
 WHERE q1.parent is null; 
 
GO	  
DROP VIEW [dbo].[DeptView];
 

-- YES you can use a CTE in a view. 
GO
CREATE VIEW [dbo].[DeptCTEView]
AS 
WITH deptCTE(id, department, parent) AS 
(	SELECT id, department, parent
	  FROM Departments) 
SELECT q1.department, q2.department as subDepartment
  FROM deptCTE q1
 INNER JOIN deptCTE q2 on q1.id = q2.parent
 WHERE q1.parent is null; 

GO
SELECT * 
  FROM [dbo].[DeptCTEView];

GO
DROP VIEW [dbo].[DeptCTEView];


 

 --CTE in a function to split a string
 --http://stevestedman.com/?p=1619
USE [cte_demo];

GO
CREATE FUNCTION dbo.Split (@sep char(1), @s varchar(8000))
RETURNS table
AS
RETURN (
    WITH splitter_cte AS (
      SELECT CHARINDEX(@sep, @s) as pos, 0 as lastPos
      UNION ALL
      SELECT CHARINDEX(@sep, @s, pos + 1), pos 
      FROM splitter_cte
      WHERE pos > 0
    )
    SELECT SUBSTRING(@s, lastPos + 1, case when pos = 0 then 80000 else pos - lastPos -1 end) as chunk
    FROM splitter_cte
  )
GO


SELECT * 
  FROM dbo.Split(' ', 'the quick brown dog jumps over the lazy fox')
OPTION(MAXRECURSION 0);



declare @InputString as varchar(8000);
set @InputString = 'Anyone can perform a SQL Server upgrade, but the risk of failure is much greater if you dont plan it correctly.  By the end of this chapter you will understand how to plan for the things that could go wrong during the upgrade, and options you can take to mitigate those risks.  
This chapter will cover the details that you will need to know when upgrading a stand alone, non High Availability (HA), SQL Server from one version to a newer version, with minimal downtime.  I use the term minimal downtime as the goal, as it is very expensive to get to the point of zero downtime with a SQL Server upgrade without an HA solution.  For the purpose of this upgrade process we will be considering a single SQL Server being upgraded, although it may end up on different hardware or it may be the same hardware.
Upgrading may involve upgrading just the hardware for better performance or more capacity and not the SQL Server version.  The specific upgrade path may depend on your business needs and the resources or budget that you have available.
Most of the summaries and stories in this chapter are based on my experiences using SQL Server over the last 21 years.  I will refer to several examples in this chapter of things that went wrong, and then at the end follow up with a case study of an upgrade that went very well.  Considering the things that can go wrong will help you think about ways to prevent them, or reduce their likelihood of happening.'
set @InputString = replace( @InputString, '.', ' ');
set @InputString = replace( @InputString, ',', ' ');
set @InputString = replace( @InputString, ':', ' ');
set @InputString = replace( @InputString, char(13), ' ');
set @InputString = replace( @InputString, char(10), ' ');
set @InputString = replace( @InputString, '  ', ' ');
set @InputString = replace( @InputString, '  ', ' ');
set @InputString = replace( @InputString, '  ', ' ');
set @InputString = replace( @InputString, '  ', ' ');


SELECT * 
  FROM dbo.Split(' ', @InputString)
OPTION(MAXRECURSION 0);

SELECT count(1) as NumUsages, CAST(chunk as varchar(100))
  FROM dbo.Split(' ', @InputString)
  GROUP BY chunk
  order by NumUsages desc
OPTION(MAXRECURSION 0);





GO
CREATE FUNCTION dbo.SplitWithLineNumber (@sep char(1), @s varchar(max))
RETURNS table
AS
RETURN (
 WITH splitter_cte AS (
 SELECT CHARINDEX(@sep, @s) as pos,
        CAST(0 as bigint) as lastPos,
        CAST(0 as int) as LineNumber
 UNION ALL
 SELECT CHARINDEX(@sep, @s, pos + 1),
        CAST(pos as bigint),
        LineNumber + 1 as LineNumber
 FROM splitter_cte
 WHERE pos > 0
 )
 SELECT LineNumber,
        SUBSTRING(@s,
                 lastPos + 1,
                 case when pos = 0 then 2147483647
                      else pos - lastPos -1 end) as chunk
 FROM splitter_cte
 );


GO
CREATE FUNCTION [dbo].[varcharDiff](@s1 varchar(max), @s2 varchar(max))
RETURNS table
AS
RETURN (
	WITH FirstStringAsTable as
	(
	SELECT leftDiff.lineNumber, leftDiff.chunk AS leftChunk, rightDiff.chunk AS rightChunk
	  FROM dbo.SplitWithLineNumber(char(10), @s1) AS leftDiff
	 LEFT JOIN dbo.SplitWithLineNumber(char(10), @s2) AS rightDiff ON leftDiff.lineNumber = rightDiff.lineNumber,
	)
	SELECT * 
	FROM FirstStringAsTable
	WHERE leftChunk <> rightChunk
  )
GO



declare @InputString as varchar(8000);
set @InputString = 'Anyone can perform a SQL Server upgrade, but the risk of failure is much 
greater if you dont plan it correctly.  By the end of this chapter you will understand how 
to plan for the things that could go wrong during the upgrade, and options you can take to 
mitigate those risks.  
This chapter will cover the details that you will need to know when upgrading a stand alone, 
non High Availability (HA), SQL Server from one version to a newer version, with minimal 
downtime.  I use the term minimal downtime as the goal, as it is very expensive to get to the
point of zero downtime with a SQL Server upgrade without an HA solution.  For the purpose of
this upgrade process we will be considering a single SQL Server being upgraded, although it
may end up on different hardware or it may be the same hardware.
Upgrading may involve upgrading just the hardware for better performance or more capacity and
not the SQL Server version.  The specific upgrade path may depend on your business needs and 
the resources or budget that you have available.
Most of the summaries and stories in this chapter are based on my experiences using SQL Server 
over the last 21 years.  I will refer to several examples in this chapter of things that went 
wrong, and then at the end follow up with a case study of an upgrade that went very well.  
Considering the things that can go wrong will help you think about ways to prevent them, or 
reduce their likelihood of happening.'




declare @InputString2 as varchar(8000);
set @InputString2 = 'Anyone can perform a SQL Server upgrade, but the risk of failure is much 
greater if you dont plan it correctly.  By the end of this chapter you will understand how 
to plan for the things that could go wrong during the upgrade, and options you can take to 
mitigate those risks.  
This chapter will cover the details that you need to know when upgrading a stand alone, 
non High Availability (HA), SQL Server from one version to a newer version, with minimal 
downtime.  I use the term minimalistic downtime as the goal, as it is very expensive to get to the
point of zero downtime with a SQL Server upgrade without an HA solution.  For the purpose of
this upgrade process we will be considering a single SQL Server being upgraded, although it
may end up on different hardware or it may be the same hardware.
Upgrading may involve upgrading just the hardware for better performance or more capacity and
not the SQL Server version.  The specific upgrade path may depend on your business needs and 
the resources or budget that you have.
Most of the summaries and stories in this chapter are based on my experiences using SQL Server 
over the last 21 years.  I will refer to several examples in this chapter of things that went 
wrong, and then at the end follow up with a case study of an upgrade that went very well.  
Considering the things that can go wrong will help you think about ways to prevent them, or 
reduce their likelihood of happening.'



select * 
from [dbo].[varcharDiff](@InputString, @InputString2);














--multiple anchor queries
USE [cte_demo];

-- first start with just the cycle department
;WITH DepartmentCTE AS
( SELECT id, Department, parent, 0 as Level 
    FROM Departments
   WHERE id = 2 -- 2 for cycle
   UNION ALL
  SELECT d.id, d.Department, d.parent,
         DepartmentCTE.Level + 1 as Level 
    FROM Departments d
   INNER JOIN DepartmentCTE
      ON DepartmentCTE.id = d.parent) 
SELECT * 
  FROM DepartmentCTE
 ORDER BY parent;

 -- then add in the fitness department
 ;WITH DepartmentCTE AS
( SELECT id, Department, parent, 0 as Level 
    FROM Departments
   WHERE id = 2
   UNION ALL
SELECT id, Department, parent, 0 as Level 
  FROM Departments
 WHERE id = 4
   UNION ALL
  SELECT d.id, d.Department, d.parent,
         DepartmentCTE.Level + 1 as Level 
    FROM Departments d
   INNER JOIN DepartmentCTE
      ON DepartmentCTE.id = d.parent) 
SELECT * 
  FROM DepartmentCTE
 ORDER BY parent;


 -- single anchor alternative
 ;WITH DepartmentCTE AS
( SELECT id, Department, parent, 0 as Level 
    FROM Departments
   WHERE id in (2, 4)
   UNION ALL
  SELECT d.id, d.Department, d.parent,
         DepartmentCTE.Level + 1 as Level 
    FROM Departments d
   INNER JOIN DepartmentCTE
      ON DepartmentCTE.id = d.parent) 
SELECT * 
  FROM DepartmentCTE
 ORDER BY parent;







SELECT * FROM Royalty;

-- Trace Prince William's maternal side
WITH RoyalTreeCTE 
AS
(
	SELECT * FROM Royalty WHERE id = 17
	UNION ALL
	SELECT r.* 
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.mother = r.id
)
SELECT * FROM RoyalTreeCTE;


-- Trace Prince William's paternal side

WITH RoyalTreeCTE 
AS
(
	SELECT * FROM Royalty WHERE id = 17
	UNION ALL
	SELECT r.* 
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.father = r.id
)
SELECT * FROM RoyalTreeCTE;


-- trace both sides.
WITH RoyalTreeCTE 
AS
(
	SELECT * 
	 FROM Royalty WHERE id = 17
	UNION ALL
	SELECT r.* 
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.father = r.id
	UNION ALL
	SELECT r.* 
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.mother = r.id
)
SELECT * FROM RoyalTreeCTE;


WITH RoyalTreeCTE (id, name, mother, father, relationship)
AS
(
	SELECT *, 'source' as relationship
	 FROM Royalty WHERE id = 17
	UNION ALL
	SELECT r.*, 'father' as relationship
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.father = r.id
	UNION ALL
	SELECT r.*, 'mother' as relationship
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.mother = r.id
)
SELECT * FROM RoyalTreeCTE;











WITH RoyalTreeCTE AS
(  SELECT *, 0 as level,
          CAST(name as varchar(4096)) as TreePath
     FROM Royalty WHERE id = 17
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(rt.TreePath + '-->father:' +
               r.name as varchar(4096)) as TreePath
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.father = r.id
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(rt.TreePath + '-->mother:' +
               r.name as varchar(4096)) as TreePath
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.mother = r.id)
SELECT TreePath
  FROM RoyalTreeCTE
 WHERE level < 3
 ORDER BY TreePath;


WITH RoyalTreeCTE AS
(  SELECT *, 0 as level,
          CAST(name as varchar(4096)) as TreePath,
		  CAST('Root' as varchar(10)) as Relationship
     FROM Royalty WHERE id = 17
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(rt.TreePath + '-->father:' +
               r.name as varchar(4096)) as TreePath,
		  CAST('Father' as varchar(10)) as Relationship
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.father = r.id
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(rt.TreePath + '-->mother:' +
               r.name as varchar(4096)) as TreePath,
		  CAST('Mother' as varchar(10)) as Relationship
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.mother = r.id)
SELECT REPLICATE('.      ', Level) + Relationship + ': ' + name
  FROM RoyalTreeCTE
 ORDER BY TreePath;





WITH RoyalTreeCTE AS
(  SELECT *, 0 as level,
          CAST('b' as varchar(4096)) as Sorter,
		  CAST('Top' as varchar(10)) as Relationship
     FROM Royalty WHERE id = 17
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(Sorter + 'a' as varchar(4096)) as Sorter,
		  CAST('Father' as varchar(10)) as Relationship
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.father = r.id
    UNION ALL
   SELECT r.*, level + 1 as level,
          CAST(Sorter + 'c' as varchar(4096)) as Sorter,
		  CAST('Mother' as varchar(10)) as Relationship
     FROM Royalty r
    INNER JOIN RoyalTreeCTE rt on rt.mother = r.id)
SELECT sorter, name--, REPLICATE('.           ', Level) + Relationship + ': ' + name
  FROM RoyalTreeCTE
 ORDER BY Sorter + 'bbbbbbbbbbbbb';







WITH RoyalTreeCTE (id, name, mother, father, level, TreePath)
AS
(
	SELECT *, 0 as level,
           CAST(name as varchar(4096)) as TreePath
	 FROM Royalty WHERE id = 17
	UNION ALL
	SELECT r.*, level + 1 as level,
           CAST(rt.TreePath + '-->father:' + CAST(r.name as varchar(4096)) as varchar(4096)) as TreePath
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.father = r.id
	UNION ALL
	SELECT r.*, level + 1 as level,
           CAST(rt.TreePath + '-->mother:' + CAST(r.name as varchar(4096)) as varchar(4096)) as TreePath
	  FROM Royalty r
	 INNER JOIN RoyalTreeCTE rt on rt.mother = r.id
)
--SELECT TreePath FROM RoyalTreeCTE;
SELECT TreePath FROM RoyalTreeCTE where level < 2
ORDER BY TreePath;
 





-- compared to a recursive function
GO
DROP FUNCTION dbo.SumParts;
GO
CREATE FUNCTION dbo.SumParts (@Num integer)
RETURNS integer
AS
BEGIN
	DECLARE @ResultVar as integer;
	set @ResultVar = 0;
	IF (@Num > 0)
		SET @ResultVar = @Num + dbo.SumParts(@Num - 1);
	RETURN @ResultVar;
END
GO
select dbo.SumParts(10);

GO

DECLARE @Num as INTEGER;
SET @Num = 10;

;WITH SumPartsCTE AS
(
	SELECT @Num as Total, @Num as CountDown
	UNION ALL
	SELECT Total + CountDown - 1, CountDown - 1 as CountDown
	  FROM SumPartsCTE
	 WHERE CountDown > 0
)
SELECT max(Total) FROM SumPartsCTE;



GO



 --The department id field will show up as department.
WITH departmentsCTE(department, parent, id) AS
( SELECT id, department, parent
    FROM Departments) 
SELECT * 
  FROM departmentsCTE;















-- filling up a test table.
USE [cte_demo]
GO
DROP TABLE [dbo].[Users];
--create a table Users to fill from a CTE
CREATE TABLE [dbo].[Users](
	[id] [int] IDENTITY(1,1) NOT NULL,
	[FirstName] [varchar](50) NULL,
	[MiddleName] [varchar](50) NULL,
	[LastName] [varchar](50) NULL,
	[LoginName] [varchar](100) NULL
) ON [PRIMARY];

CREATE CLUSTERED INDEX [ClusteredUsers] 
    ON [dbo].[Users] ( [id] ASC );

CREATE NONCLUSTERED INDEX [UsersLoginName] 
    ON [dbo].[Users] ( [LoginName] ASC );


GO

set nocount on;
GO
INSERT INTO [dbo].[Users]
           ([FirstName], [MiddleName], [LastName], [LoginName])
     VALUES
           ('Steve', 'Tester', 'Stedman', 'SteveTesterStedman');
GO 10000000


--or fill it up with more varying data.


;WITH Fnames (Name) AS
(SELECT 'John' 
  UNION 
 SELECT 'Mary' 
  UNION 
 SELECT 'Bill'),
Mnames (Name) AS
(SELECT 'A' 
  UNION 
 SELECT 'Ed' 
  UNION 
 SELECT 'Helen') ,
Lnames (Name) AS
(SELECT 'Smith' 
  UNION 
 SELECT 'Gibb' 
  UNION 
 SELECT 'Jones')
SELECT F.Name AS FirstName,
       M.Name AS MiddleName,
       L.Name AS LastName
  FROM Fnames AS F
 CROSS JOIN Mnames AS M
 CROSS JOIN Lnames AS L;




 ;WITH Fnames (Name) AS
(SELECT 'John' UNION SELECT 'Mary' UNION SELECT 'Bill'),
Mnames (Name) AS
(SELECT 'A' UNION SELECT 'Ed' UNION SELECT 'Helen') ,
Lnames (Name) AS
(SELECT 'Smith' UNION SELECT 'Gibb' UNION SELECT 'Jones'),
UserNameBase (Name) AS
(SELECT 'Ace' UNION SELECT 'Stallion' UNION SELECT 'Princess')
SELECT F.Name AS FirstName, M.Name AS MiddleName,
       L.Name AS LastName, U.Name + L.Name as UserName
  FROM Fnames AS F
 CROSS JOIN Mnames AS M
 CROSS JOIN Lnames AS L
 CROSS JOIN UserNameBase AS U;




 ;WITH Fnames (Name) AS
(SELECT 'John' UNION SELECT 'Mary' UNION SELECT 'Bill'),
Mnames (Name) AS
(SELECT 'A' UNION SELECT 'Ed' UNION SELECT 'Helen') ,
Lnames (Name) AS
(SELECT 'Smith' UNION SELECT 'Gibb' UNION SELECT 'Jones'),
UserNameBase (Name) AS
(SELECT 'Ace' UNION SELECT 'Stallion' UNION 
 SELECT 'Princess'),
Uniqueifier(Name) AS
(SELECT 'A' UNION SELECT 'B' UNION SELECT 'C')
INSERT INTO [dbo].[Users]
           ([FirstName], [MiddleName], [LastName], [LoginName])
SELECT F.Name AS FirstName, M.Name AS MiddleName,
       L.Name AS LastName, 
       Uniq.Name + U.Name + L.Name + Uniq2.Name as UserName
  FROM Fnames AS F
 CROSS JOIN Mnames AS M
 CROSS JOIN Lnames AS L
 CROSS JOIN UserNameBase AS U
 CROSS JOIN Uniqueifier AS Uniq
 CROSS JOIN Uniqueifier AS Uniq2;

 SELECT * FROM Users;




 ;WITH Fnames (Name) AS
(SELECT 'John' UNION ALL SELECT 'Mary' UNION ALL SELECT 'Bill' UNION ALL SELECT 'Fred' UNION ALL SELECT 'Harry' UNION ALL SELECT 'Peter' UNION ALL SELECT 'Donna' UNION ALL SELECT 'Karen' UNION ALL SELECT 'Jack' UNION ALL SELECT 'Dee' UNION ALL SELECT 'Mary' UNION ALL SELECT 'Bill2' UNION ALL SELECT 'Bill3' UNION ALL SELECT 'Bill4' UNION ALL SELECT 'Bill5'),
Mnames (Name) AS
(SELECT 'A' UNION ALL SELECT 'Ed' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen' UNION ALL SELECT 'Helen') ,
Lnames (Name) AS
(SELECT 'Smith' UNION ALL SELECT 'Gibb' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones' UNION ALL SELECT 'Jones'),
UserNameBase (Name) AS
(SELECT 'Ace' UNION ALL SELECT 'Stallion' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL 
 SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL 
 SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess' UNION ALL SELECT 'Princess'),
Uniqueifier(Name) AS
(SELECT 'A' UNION ALL SELECT 'B' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL 
 SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL 
 SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C' UNION ALL SELECT 'C')
INSERT INTO [dbo].[Users]
           ([FirstName], [MiddleName], [LastName], [LoginName])
SELECT F.Name AS FirstName, M.Name AS MiddleName,
       L.Name AS LastName, 
       Uniq.Name + U.Name + L.Name + Uniq2.Name as UserName
  FROM Fnames AS F
 CROSS JOIN Mnames AS M
 CROSS JOIN Lnames AS L
 CROSS JOIN UserNameBase AS U
 CROSS JOIN Uniqueifier AS Uniq
 CROSS JOIN Uniqueifier AS Uniq2;





 --THE END---------------------------------------------------


