-- SAMPLE CODE for the Common Table Expressions
-- Created by Steve Stedman
--    http://SteveStedman.com
--    twitter:   @SqlEmt


USE [Master];
SET STATISTICS IO OFF;
SET NOCOUNT ON;

IF EXISTS(SELECT name 
            FROM sys.databases 
		   WHERE name = 'cte_demo')
BEGIN
	ALTER DATABASE [cte_demo] 
	  SET SINGLE_USER WITH ROLLBACK IMMEDIATE;
	DROP DATABASE [cte_demo];
END
CREATE DATABASE [cte_demo];
GO

USE [cte_demo];


--create a table to use for CTE query demo
CREATE TABLE [Departments] (
	id int,  --would normally be an INT IDENTITY
	department VARCHAR (200),
	parent int,
	archived bit default 0
	);

GO
CREATE PROCEDURE ReloadDepartments
AS 
BEGIN
	SET NOCOUNT ON;
	TRUNCATE TABLE [dbo].[Departments];	
	-- insert top level departments
	insert into [Departments] (id, department, parent) 
	     values (1, 'Camping', null);
	insert into [Departments] (id, department, parent) 
	     values (2, 'Cycle', null);
	insert into [Departments] (id, department, parent) 
	     values (3, 'Snowsports', null);
	insert into [Departments] (id, department, parent) 
	     values (4, 'Fitness', null);
      
	-- now some sub-departments for camping
	insert into [Departments] (id, department, parent) 
	     values (5, 'Tents', 1);
	insert into [Departments] (id, department, parent) 
	     values (6, 'Backpacks', 1);
	insert into [Departments] (id, department, parent) 
	     values (7, 'Sleeping Bags', 1);
	insert into [Departments] (id, department, parent) 
	     values (8, 'Cooking', 1);
       
	-- now some sub-departments for cycle
	insert into [Departments] (id, department, parent) 
	     values (9, 'Bikes', 2);
	insert into [Departments] (id, department, parent) 
	     values (10, 'Helmets', 2);
	insert into [Departments] (id, department, parent) 
	     values (11, 'Locks', 2);

	-- now some sub-departments for snowsports
	insert into [Departments] (id, department, parent) 
	     values (12, 'Ski', 3);
	insert into [Departments] (id, department, parent) 
	     values (13, 'Snowboard', 3);
	insert into [Departments] (id, department, parent, archived) 
	     values (14, 'Snowshoe', 3, 1);
       
	-- now some sub-departments for fitness
	insert into [Departments] (id, department, parent) 
	     values (15, 'Running', 4);
	insert into [Departments] (id, department, parent) 
	     values (16, 'Swimming', 4);
	insert into [Departments] (id, department, parent, archived) 
	     values (17, 'Yoga', 4, 1);
END
GO 
CREATE PROCEDURE FillMoreDepartments
AS
BEGIN
	SET NOCOUNT ON;
	delete from Departments where id > 17; 
	 --now add 2 more levels of depth
	insert into [Departments] (id, department, parent) 
	     values (18, '1 Person', 5);
	insert into [Departments] (id, department, parent) 
	     values (19, '2 Person', 5);
	insert into [Departments] (id, department, parent) 
	     values (20, '3 Person', 5);
	insert into [Departments] (id, department, parent) 
	     values (21, '4 Person', 5);

	insert into [Departments] (id, department, parent) 
	     values (22, 'Backpacking', 19);
	insert into [Departments] (id, department, parent) 
	     values (23, 'Family Camping', 19);
	insert into [Departments] (id, department, parent) 
	     values (24, 'Mountaineering', 19);

	insert into [Departments] (id, department, parent) 
	     values (25, 'Ultra-lightweight', 24);
	insert into [Departments] (id, department, parent) 
	     values (26, 'Lightweight', 24);
	insert into [Departments] (id, department, parent) 
	     values (27, 'Standard', 24);

	insert into [Departments] (id, department, parent) 
	     values (28, 'Gifts', null);
	insert into [Departments] (id, department, parent) 
	     values (29, 'Clearance', null);

END
GO
-- multipe recursive queries
--drop table [Royalty];
CREATE TABLE [Royalty] (
	id int,  --would normally be an INT IDENTITY
    name VARCHAR (200),
	mother int,
	father int,
	);
	
insert into [Royalty] (id, name, mother, father) 
	     values (1, 'George V, King of England', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (2, 'Mary, Princess of Teck', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (3, 'George VI Windsor, King of England', 2, 1);
insert into [Royalty] (id, name, mother, father) 
	     values (4, 'Claude George Bowes-Lyon', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (5, 'Nina Cecilia Cavendish-Bentinck', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (6, 'Elizabeth Angela Marguerite Bowes-Lyon', 5, 4);
insert into [Royalty] (id, name, mother, father) 
	     values (7, 'William George I of the Hellenes', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (8, 'Olga Konstantinovna Romanova', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (9, 'Louis Alexander von Battenburg', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (10, 'Victoria von Hessen und bei Rhein', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (11, 'Andreas, Prince of Greece', 8, 7);
insert into [Royalty] (id, name, mother, father) 
	     values (12, 'Alice, Princess of Battenbugr', 10, 9);
insert into [Royalty] (id, name, mother, father) 
	     values (13, 'Phillip Mountbatten, Duke of Edinburgh', 12, 11);
insert into [Royalty] (id, name, mother, father) 
	     values (14, 'Elizabeth II Windsor, Queene of England', 6, 3);
insert into [Royalty] (id, name, mother, father) 
	     values (15, 'Charles Philip Arthur Windsor', 14, 13);
insert into [Royalty] (id, name, mother, father) 
	     values (16, 'Diana Frances (Lady) Spencer', 18, 19);
insert into [Royalty] (id, name, mother, father) 
	     values (17, 'William Arthur Phillip Windsor', 16, 15);
insert into [Royalty] (id, name, mother, father) 
	     values (18, 'Frances Ruth Burke Roche', null, null);
insert into [Royalty] (id, name, mother, father) 
	     values (19, 'Edward John Spencer', null, null);

SET NOCOUNT OFF;

USE [cte_demo];

EXECUTE ReloadDepartments;


	     
GO
CREATE TABLE SalesInvoice
(
	InvoiceID INT PRIMARY KEY,
	OrderDate DATETIME NOT NULL,
	PaidDate DATETIME NOT NULL,
	CustomerID INT NOT NULL,
	Comment NTEXT
);

CREATE TABLE Customer
(	CustomerID INT PRIMARY KEY,
	CustomerType NVARCHAR(25),
	FirstName NVArCHAR(30),
	LastName NVARCHAR(40),
	CompanyName NVARCHAR(100)
);
GO
TRUNCATE TABLE Customer;
GO
	SET NOCOUNT ON;
INSERT INTO Customer VALUES (1,'Consumer','Mark','Williams',NULL)
INSERT INTO Customer VALUES (2,'Consumer','Lee','Young',NULL)
INSERT INTO Customer VALUES (3,'Consumer','Patricia','Martin',NULL)
INSERT INTO Customer VALUES (4,'Consumer','Mary','Lopez',NULL)
INSERT INTO Customer VALUES (5,'Consumer',NULL,NULL,'MoreTechnology.com')
INSERT INTO Customer VALUES (6,'Consumer','Ruth','Clark',NULL)
INSERT INTO Customer VALUES (7,'Consumer','Tessa','Wright',NULL)
INSERT INTO Customer VALUES (8,'Consumer','Jennifer','Garcia',NULL)
INSERT INTO Customer VALUES (9,'Consumer','Linda','Adams',NULL)
INSERT INTO Customer VALUES (10,'Consumer',' Robert','Wilson',NULL)
INSERT INTO Customer VALUES (11,'Consumer','Kimberly','Taylor',NULL)
INSERT INTO Customer VALUES (12,'Consumer','Helen','Hernandez',NULL)
INSERT INTO Customer VALUES (13,'Consumer','Linda','Hernandez',NULL)
INSERT INTO Customer VALUES (14,'Consumer','Elizabeth','White',NULL)
INSERT INTO Customer VALUES (15,'Consumer','Anthony','Scott',NULL)
INSERT INTO Customer VALUES (16,'Consumer','Paul','Perez',NULL)
INSERT INTO Customer VALUES (17,'Consumer','Sanjay','Mitchell',NULL)
INSERT INTO Customer VALUES (18,'Consumer','Donald','Walker',NULL)
INSERT INTO Customer VALUES (19,'Consumer','Patricia','Rodriguez',NULL)
INSERT INTO Customer VALUES (20,'Consumer','Elizabeth','Wilson',NULL)
INSERT INTO Customer VALUES (21,'Consumer','Carol','Thompson',NULL)
INSERT INTO Customer VALUES (22,'Consumer','George','Hall',NULL)
INSERT INTO Customer VALUES (23,'Consumer','Daniel','Gonzalez',NULL)
INSERT INTO Customer VALUES (24,'Consumer','Paul','Edwards',NULL)
INSERT INTO Customer VALUES (25,'Consumer','Lee','Hall',NULL)
INSERT INTO Customer VALUES (26,'Consumer','Sandra','Williams',NULL)
INSERT INTO Customer VALUES (27,'Consumer','George','Allen',NULL)
INSERT INTO Customer VALUES (28,'Consumer','John','Allen',NULL)
INSERT INTO Customer VALUES (29,'Consumer','Lisa','Scott',NULL)
INSERT INTO Customer VALUES (30,'Consumer','Steven','Baker',NULL)
INSERT INTO Customer VALUES (31,'Consumer','Thomas','Martinez',NULL)
INSERT INTO Customer VALUES (32,'Consumer','Paul','Turner',NULL)
INSERT INTO Customer VALUES (33,'Consumer','Ruth','King',NULL)
INSERT INTO Customer VALUES (34,'Consumer','Richard','Brown',NULL)
INSERT INTO Customer VALUES (35,'Consumer','Kenneth','Campbell',NULL)
INSERT INTO Customer VALUES (36,'Consumer','Jeff','Walker',NULL)
INSERT INTO Customer VALUES (37,'Consumer','John','Nelson',NULL)
INSERT INTO Customer VALUES (38,'Consumer','Linda','Adams',NULL)
INSERT INTO Customer VALUES (39,'Consumer','John','Jackson',NULL)
INSERT INTO Customer VALUES (40,'Consumer','Susan','Wright',NULL)
INSERT INTO Customer VALUES (41,'Consumer','Deborah','Smith',NULL)
INSERT INTO Customer VALUES (42,'Consumer','James','Campbell',NULL)
INSERT INTO Customer VALUES (43,'Consumer','Patricia','Parker',NULL)
INSERT INTO Customer VALUES (44,'Consumer','Sarah','Thompson',NULL)
INSERT INTO Customer VALUES (45,'Consumer','Thomas','Lee',NULL)
INSERT INTO Customer VALUES (46,'Consumer','Jason','Hill',NULL)
INSERT INTO Customer VALUES (47,'Consumer','Sarah','Campbell',NULL)
INSERT INTO Customer VALUES (48,'Consumer','Donna','Martinez',NULL)
INSERT INTO Customer VALUES (49,'Consumer','Jason','Adams',NULL)
INSERT INTO Customer VALUES (50,'Consumer','William','Green',NULL)
INSERT INTO Customer VALUES (51,'Consumer','Sanjay','Scott',NULL)
INSERT INTO Customer VALUES (52,'Consumer','Daniel','Nelson',NULL)
INSERT INTO Customer VALUES (53,'Consumer','Michael','Johnson',NULL)
INSERT INTO Customer VALUES (54,'Consumer','Barbara','Campbell',NULL)
INSERT INTO Customer VALUES (55,'Consumer','Kevin','Smith',NULL)
INSERT INTO Customer VALUES (56,'Consumer','Elizabeth','Rodriguez',NULL)
INSERT INTO Customer VALUES (57,'Consumer','Sandra','Allen',NULL)
INSERT INTO Customer VALUES (58,'Consumer','Jason','Harris',NULL)
INSERT INTO Customer VALUES (59,'Consumer','Barbara','Green',NULL)
INSERT INTO Customer VALUES (60,'Consumer','Joseph','Perez',NULL)
INSERT INTO Customer VALUES (61,'Consumer','Lisa','Hall',NULL)
INSERT INTO Customer VALUES (62,'Consumer',' Robert','Wright',NULL)
INSERT INTO Customer VALUES (63,'Consumer','Carol','Miller',NULL)
INSERT INTO Customer VALUES (64,'Consumer','Linda','Johnson',NULL)
INSERT INTO Customer VALUES (65,'Consumer','Lisa','Moore',NULL)
INSERT INTO Customer VALUES (66,'Consumer','Michael','Turner',NULL)
INSERT INTO Customer VALUES (67,'Consumer','Brian','Hall',NULL)
INSERT INTO Customer VALUES (68,'Consumer','Richard','Moore',NULL)
INSERT INTO Customer VALUES (69,'Consumer','Donna','Lopez',NULL)
INSERT INTO Customer VALUES (70,'Consumer','Sarah','Smith',NULL)
INSERT INTO Customer VALUES (71,'Consumer','George','Jackson',NULL)
INSERT INTO Customer VALUES (72,'Consumer','Donald','Scott',NULL)
INSERT INTO Customer VALUES (73,'Consumer','Sarah','Nelson',NULL)
INSERT INTO Customer VALUES (74,'Consumer','Donald','Turner',NULL)
INSERT INTO Customer VALUES (75,'Consumer','Nancy','Rodriguez',NULL)
INSERT INTO Customer VALUES (76,'Consumer','Daniel','Green',NULL)
INSERT INTO Customer VALUES (77,'Consumer','Patricia','Moore',NULL)
INSERT INTO Customer VALUES (78,'Consumer','Deborah','Lewis',NULL)
INSERT INTO Customer VALUES (79,'Consumer','Donald','Collins',NULL)
INSERT INTO Customer VALUES (80,'Consumer','Susan','Phillips',NULL)
INSERT INTO Customer VALUES (81,'Consumer','Pedro','Brown',NULL)
INSERT INTO Customer VALUES (82,'Consumer','Kenneth','Young',NULL)
INSERT INTO Customer VALUES (83,'Consumer','Charles','Wilson',NULL)
INSERT INTO Customer VALUES (84,'Consumer','Kimberly','Wright',NULL)
INSERT INTO Customer VALUES (85,'Consumer','Daniel','Williams',NULL)
INSERT INTO Customer VALUES (86,'Consumer','Karen','Turner',NULL)
INSERT INTO Customer VALUES (87,'Consumer','Jeff','Phillips',NULL)
INSERT INTO Customer VALUES (88,'Consumer','Brian','Baker',NULL)
INSERT INTO Customer VALUES (89,'Consumer','Betty','Roberts',NULL)
INSERT INTO Customer VALUES (90,'Consumer','Edward','Martinez',NULL)
INSERT INTO Customer VALUES (91,'Consumer','Kenneth','Thompson',NULL)
INSERT INTO Customer VALUES (92,'Consumer','Lisa','Garcia',NULL)
INSERT INTO Customer VALUES (93,'Consumer','Joseph','Lee',NULL)
INSERT INTO Customer VALUES (94,'Consumer','Elizabeth','Hall',NULL)
INSERT INTO Customer VALUES (95,'Consumer','Charles','White',NULL)
INSERT INTO Customer VALUES (96,'Consumer','Richard','Thompson',NULL)
INSERT INTO Customer VALUES (97,'Consumer','Ruth','Smith',NULL)
INSERT INTO Customer VALUES (98,'Consumer','Charles','Edwards',NULL)
INSERT INTO Customer VALUES (99,'Consumer','Kevin','Lewis',NULL)
INSERT INTO Customer VALUES (100,'Consumer','Jennifer','Wright',NULL)
INSERT INTO Customer VALUES (101,'Consumer','Paul','Davis',NULL)
INSERT INTO Customer VALUES (102,'Consumer','Ronald','Garcia',NULL)
INSERT INTO Customer VALUES (103,'Consumer','Edward','Hill',NULL)
INSERT INTO Customer VALUES (104,'Consumer',' Robert','Anderson',NULL)
INSERT INTO Customer VALUES (105,'Consumer','Barbara','Miller',NULL)
INSERT INTO Customer VALUES (106,'Consumer','Charles','Campbell',NULL)
INSERT INTO Customer VALUES (107,'Consumer','Michael','Perez',NULL)
INSERT INTO Customer VALUES (108,'Consumer','Jeff','Roberts',NULL)
INSERT INTO Customer VALUES (109,'Consumer','Carol','Moore',NULL)
INSERT INTO Customer VALUES (110,'Consumer','John','Adams',NULL)
INSERT INTO Customer VALUES (111,'Consumer','Michael','Jackson',NULL)
INSERT INTO Customer VALUES (112,'Consumer','Laura','Adams',NULL)
INSERT INTO Customer VALUES (113,'Consumer','Jason','Garcia',NULL)
INSERT INTO Customer VALUES (114,'Consumer','Ronald','White',NULL)
INSERT INTO Customer VALUES (115,'Consumer','Mark','Davis',NULL)
INSERT INTO Customer VALUES (116,'Consumer','Jennifer','Johnson',NULL)
INSERT INTO Customer VALUES (117,'Business',NULL,NULL,'Puma Consulting')
INSERT INTO Customer VALUES (118,'Consumer','Charles','King',NULL)
INSERT INTO Customer VALUES (119,'Consumer','Nancy','Garcia',NULL)
INSERT INTO Customer VALUES (120,'Consumer','Paul','Hill',NULL)
INSERT INTO Customer VALUES (121,'Consumer','Paul','Roberts',NULL)
INSERT INTO Customer VALUES (122,'Consumer','Michael','Walker',NULL)
INSERT INTO Customer VALUES (123,'Consumer','George','Wilson',NULL)
INSERT INTO Customer VALUES (124,'Consumer','Brian','Mitchell',NULL)
INSERT INTO Customer VALUES (125,'Consumer','Jason','Wright',NULL)
INSERT INTO Customer VALUES (126,'Consumer','Daniel','Scott',NULL)
INSERT INTO Customer VALUES (127,'Consumer','Richard','Adams',NULL)
INSERT INTO Customer VALUES (128,'Consumer','Pedro','Brown',NULL)
INSERT INTO Customer VALUES (129,'Consumer','Ruth','Carter',NULL)
INSERT INTO Customer VALUES (130,'Consumer','Edward','Baker',NULL)
INSERT INTO Customer VALUES (131,'Consumer','George','Edwards',NULL)
INSERT INTO Customer VALUES (132,'Consumer','Sarah','Wright',NULL)
INSERT INTO Customer VALUES (133,'Consumer','Michael','Thomas',NULL)
INSERT INTO Customer VALUES (134,'Consumer','Mark','Wright',NULL)
INSERT INTO Customer VALUES (135,'Consumer','Nancy','Perez',NULL)
INSERT INTO Customer VALUES (136,'Consumer','Maria','Wilson',NULL)
INSERT INTO Customer VALUES (137,'Consumer','Patricia','King',NULL)
INSERT INTO Customer VALUES (138,'Consumer','Karen','Thompson',NULL)
INSERT INTO Customer VALUES (139,'Consumer','Sandra','Anderson',NULL)
INSERT INTO Customer VALUES (140,'Consumer','Carol','Allen',NULL)
INSERT INTO Customer VALUES (141,'Consumer','Elizabeth','Davis',NULL)
INSERT INTO Customer VALUES (142,'Consumer','Lee','Thompson',NULL)
INSERT INTO Customer VALUES (143,'Consumer','George','Hill',NULL)
INSERT INTO Customer VALUES (144,'Consumer','Richard','Davis',NULL)
INSERT INTO Customer VALUES (145,'Consumer','Charles','Lewis',NULL)
INSERT INTO Customer VALUES (146,'Consumer','Susan','Thomas',NULL)
INSERT INTO Customer VALUES (147,'Consumer','Sanjay','Robinson',NULL)
INSERT INTO Customer VALUES (148,'Consumer','Karen','Campbell',NULL)
INSERT INTO Customer VALUES (149,'Consumer','Kimberly','Allen',NULL)
INSERT INTO Customer VALUES (150,'Consumer','William','Jackson',NULL)
INSERT INTO Customer VALUES (151,'Consumer','Carol','Young',NULL)
INSERT INTO Customer VALUES (152,'Consumer','Donna','Adams',NULL)
INSERT INTO Customer VALUES (153,'Consumer','Lisa','Parker',NULL)
INSERT INTO Customer VALUES (154,'Consumer','Mark','Garcia',NULL)
INSERT INTO Customer VALUES (155,'Consumer','Jeff','Lewis',NULL)
INSERT INTO Customer VALUES (156,'Consumer','Ruth','Brown',NULL)
INSERT INTO Customer VALUES (157,'Consumer','Edward','Mitchell',NULL)
INSERT INTO Customer VALUES (158,'Consumer','Tessa','Allen',NULL)
INSERT INTO Customer VALUES (159,'Consumer','Richard','Phillips',NULL)
INSERT INTO Customer VALUES (160,'Consumer','Donna','Allen',NULL)
INSERT INTO Customer VALUES (161,'Consumer','Elizabeth','Phillips',NULL)
INSERT INTO Customer VALUES (162,'Consumer','Carol','Rodriguez',NULL)
INSERT INTO Customer VALUES (163,'Consumer','John','Lopez',NULL)
INSERT INTO Customer VALUES (164,'Consumer','Kevin','Edwards',NULL)
INSERT INTO Customer VALUES (165,'Consumer','Richard','Phillips',NULL)
INSERT INTO Customer VALUES (166,'Consumer','Joseph','Robinson',NULL)
INSERT INTO Customer VALUES (167,'Consumer','Carol','Hall',NULL)
INSERT INTO Customer VALUES (168,'Consumer','Anthony','Brown',NULL)
INSERT INTO Customer VALUES (169,'Consumer','Mark','Williams',NULL)
INSERT INTO Customer VALUES (170,'Consumer','Paul','Turner',NULL)
INSERT INTO Customer VALUES (171,'Consumer','Jennifer','Davis',NULL)
INSERT INTO Customer VALUES (172,'Consumer','Ruth','Lewis',NULL)
INSERT INTO Customer VALUES (173,'Consumer','Ruth','Jackson',NULL)
INSERT INTO Customer VALUES (174,'Consumer','Donna','Johnson',NULL)
INSERT INTO Customer VALUES (175,'Consumer','Ronald','Young',NULL)
INSERT INTO Customer VALUES (176,'Consumer','Elizabeth','Lopez',NULL)
INSERT INTO Customer VALUES (177,'Consumer','Susan','Adams',NULL)
INSERT INTO Customer VALUES (178,'Consumer','George','Adams',NULL)
INSERT INTO Customer VALUES (179,'Consumer','John','Brown',NULL)
INSERT INTO Customer VALUES (180,'Consumer','Jeff','Smith',NULL)
INSERT INTO Customer VALUES (181,'Consumer','Daniel','Young',NULL)
INSERT INTO Customer VALUES (182,'Consumer','Daniel','Evans',NULL)
INSERT INTO Customer VALUES (183,'Consumer','Brian','Williams',NULL)
INSERT INTO Customer VALUES (184,'Consumer','Brian','Lewis',NULL)
INSERT INTO Customer VALUES (185,'Consumer','Sanjay','Garcia',NULL)
INSERT INTO Customer VALUES (186,'Consumer','Edward','Campbell',NULL)
INSERT INTO Customer VALUES (187,'Consumer','Patricia','Green',NULL)
INSERT INTO Customer VALUES (188,'Consumer','Kenneth','Scott',NULL)
INSERT INTO Customer VALUES (189,'Consumer','William','Hall',NULL)
INSERT INTO Customer VALUES (190,'Consumer','Kevin','Davis',NULL)
INSERT INTO Customer VALUES (191,'Consumer','Paul','Davis',NULL)
INSERT INTO Customer VALUES (192,'Consumer','Kevin','Edwards',NULL)
INSERT INTO Customer VALUES (193,'Consumer',' Robert','Hill',NULL)
INSERT INTO Customer VALUES (194,'Consumer','Jeff','Martin',NULL)
INSERT INTO Customer VALUES (195,'Consumer','Carol','Clark',NULL)
INSERT INTO Customer VALUES (196,'Consumer','Sandra','Nelson',NULL)
INSERT INTO Customer VALUES (197,'Consumer','Michael','Taylor',NULL)
INSERT INTO Customer VALUES (198,'Consumer','Brian','Hernandez',NULL)
INSERT INTO Customer VALUES (199,'Consumer','David','Davis',NULL)
INSERT INTO Customer VALUES (200,'Consumer','Ruth','Allen',NULL)
INSERT INTO Customer VALUES (201,'Consumer','Pedro','Anderson',NULL)
INSERT INTO Customer VALUES (202,'Consumer','John','Mitchell',NULL)
INSERT INTO Customer VALUES (203,'Consumer','Jeff','Hill',NULL)
INSERT INTO Customer VALUES (204,'Consumer','Ronald','Turner',NULL)
INSERT INTO Customer VALUES (205,'Consumer','Thomas','Scott',NULL)
INSERT INTO Customer VALUES (206,'Consumer','Brian','Walker',NULL)
INSERT INTO Customer VALUES (207,'Consumer','Ruth','Garcia',NULL)
INSERT INTO Customer VALUES (208,'Consumer','Linda','Parker',NULL)
INSERT INTO Customer VALUES (209,'Consumer','Carol','Miller',NULL)
INSERT INTO Customer VALUES (210,'Consumer','Rick','Green',NULL)
INSERT INTO Customer VALUES (211,'Consumer','Steven','Garcia',NULL)
INSERT INTO Customer VALUES (212,'Consumer','Nancy','Thompson',NULL)
INSERT INTO Customer VALUES (213,'Consumer','Jeff','Hernandez',NULL)
INSERT INTO Customer VALUES (214,'Consumer','Thomas','Brown',NULL)
INSERT INTO Customer VALUES (215,'Consumer','James','Miller',NULL)
INSERT INTO Customer VALUES (216,'Consumer','Richard','Miller',NULL)
INSERT INTO Customer VALUES (217,'Consumer','Tessa','Nelson',NULL)
INSERT INTO Customer VALUES (218,'Consumer','Richard','Thomas',NULL)
INSERT INTO Customer VALUES (219,'Consumer','Lisa','White',NULL)
INSERT INTO Customer VALUES (220,'Consumer','Laura','Brown',NULL)
INSERT INTO Customer VALUES (221,'Consumer','Deborah','White',NULL)
INSERT INTO Customer VALUES (222,'Consumer','Tessa','Baker',NULL)
INSERT INTO Customer VALUES (223,'Consumer','Linda','Adams',NULL)
INSERT INTO Customer VALUES (224,'Consumer','Patricia','Clark',NULL)
INSERT INTO Customer VALUES (225,'Consumer','George','Jackson',NULL)
INSERT INTO Customer VALUES (226,'Consumer','Thomas','Harris',NULL)
INSERT INTO Customer VALUES (227,'Consumer','Michael','Brown',NULL)
INSERT INTO Customer VALUES (228,'Consumer','Anthony','Martinez',NULL)
INSERT INTO Customer VALUES (229,'Consumer','Sarah','Anderson',NULL)
INSERT INTO Customer VALUES (230,'Consumer','Jason','Edwards',NULL)
INSERT INTO Customer VALUES (231,'Consumer','Daniel','Smith',NULL)
INSERT INTO Customer VALUES (232,'Consumer','Mary','Perez',NULL)
INSERT INTO Customer VALUES (233,'Consumer','Pedro','Johnson',NULL)
INSERT INTO Customer VALUES (234,'Consumer','Daniel','Rodriguez',NULL)
INSERT INTO Customer VALUES (235,'Consumer','Sanjay','Carter',NULL)
INSERT INTO Customer VALUES (236,'Consumer','Maria','Hill',NULL)
INSERT INTO Customer VALUES (237,'Consumer','Lee','Gonzalez',NULL)
INSERT INTO Customer VALUES (238,'Consumer','Anthony','Hill',NULL)
INSERT INTO Customer VALUES (239,'Consumer','Lisa','Hall',NULL)
INSERT INTO Customer VALUES (240,'Consumer','Deborah','Mitchell',NULL)
INSERT INTO Customer VALUES (241,'Consumer','Ronald','Anderson',NULL)
INSERT INTO Customer VALUES (242,'Consumer','James','Nelson',NULL)
INSERT INTO Customer VALUES (243,'Consumer','Maria','Edwards',NULL)
INSERT INTO Customer VALUES (244,'Consumer','Brian','Wilson',NULL)
INSERT INTO Customer VALUES (245,'Consumer','Donald','Taylor',NULL)
INSERT INTO Customer VALUES (246,'Consumer','Jason','Campbell',NULL)
INSERT INTO Customer VALUES (247,'Consumer','Brian','Allen',NULL)
INSERT INTO Customer VALUES (248,'Consumer','James','Hill',NULL)
INSERT INTO Customer VALUES (249,'Consumer','Ruth','Carter',NULL)
INSERT INTO Customer VALUES (250,'Consumer','Elizabeth','Walker',NULL)
INSERT INTO Customer VALUES (251,'Consumer','Deborah','Moore',NULL)
INSERT INTO Customer VALUES (252,'Consumer','Barbara','Green',NULL)
INSERT INTO Customer VALUES (253,'Consumer','Lisa','Phillips',NULL)
INSERT INTO Customer VALUES (254,'Consumer','Thomas','Rodriguez',NULL)
INSERT INTO Customer VALUES (255,'Consumer','Ruth','Garcia',NULL)
INSERT INTO Customer VALUES (256,'Consumer','Steven','Garcia',NULL)
INSERT INTO Customer VALUES (257,'Consumer','Donald','White',NULL)
INSERT INTO Customer VALUES (258,'Consumer','Lisa','Allen',NULL)
INSERT INTO Customer VALUES (259,'Consumer','Sandra','Miller',NULL)
INSERT INTO Customer VALUES (260,'Consumer','Paul','Lewis',NULL)
INSERT INTO Customer VALUES (261,'Consumer','Charles','Harris',NULL)
INSERT INTO Customer VALUES (262,'Consumer','Thomas','Rodriguez',NULL)
INSERT INTO Customer VALUES (263,'Consumer','Steven','Martin',NULL)
INSERT INTO Customer VALUES (264,'Consumer','Barbara','Nelson',NULL)
INSERT INTO Customer VALUES (265,'Consumer','Sandra','Nelson',NULL)
INSERT INTO Customer VALUES (266,'Consumer','William','Wright',NULL)
INSERT INTO Customer VALUES (267,'Consumer','Donald','Johnson',NULL)
INSERT INTO Customer VALUES (268,'Consumer','Elizabeth','Puma',NULL)
INSERT INTO Customer VALUES (269,'Consumer','Jason','Jackson',NULL)
INSERT INTO Customer VALUES (270,'Consumer','Jeff','Hernandez',NULL)
INSERT INTO Customer VALUES (271,'Consumer','Donald','Mitchell',NULL)
INSERT INTO Customer VALUES (272,'Consumer','Kimberly','Lewis',NULL)
INSERT INTO Customer VALUES (273,'Consumer','James','Hernandez',NULL)
INSERT INTO Customer VALUES (274,'Consumer','Daniel','Anderson',NULL)
INSERT INTO Customer VALUES (275,'Consumer','Patricia','Taylor',NULL)
INSERT INTO Customer VALUES (276,'Consumer','Michael','Martin',NULL)
INSERT INTO Customer VALUES (277,'Consumer','Helen','Campbell',NULL)
INSERT INTO Customer VALUES (278,'Consumer','Steven','Wright',NULL)
INSERT INTO Customer VALUES (279,'Consumer','Kevin','Campbell',NULL)
INSERT INTO Customer VALUES (280,'Consumer','David','White',NULL)
INSERT INTO Customer VALUES (281,'Consumer','Thomas','King',NULL)
INSERT INTO Customer VALUES (282,'Consumer','Joseph','Clark',NULL)
INSERT INTO Customer VALUES (283,'Consumer','Jason','Robinson',NULL)
INSERT INTO Customer VALUES (284,'Consumer','Brian','Robinson',NULL)
INSERT INTO Customer VALUES (285,'Consumer','Sanjay','Wright',NULL)
INSERT INTO Customer VALUES (286,'Consumer','John','Evans',NULL)
INSERT INTO Customer VALUES (287,'Consumer','Sarah','Young',NULL)
INSERT INTO Customer VALUES (288,'Consumer','Pedro','Carter',NULL)
INSERT INTO Customer VALUES (289,'Consumer','Lisa','Miller',NULL)
INSERT INTO Customer VALUES (290,'Consumer','Susan','Harris',NULL)
INSERT INTO Customer VALUES (291,'Consumer','Richard','Turner',NULL)
INSERT INTO Customer VALUES (292,'Consumer','Pedro','Walker',NULL)
INSERT INTO Customer VALUES (293,'Consumer','Jennifer','Roberts',NULL)
INSERT INTO Customer VALUES (294,'Consumer','Donald','Adams',NULL)
INSERT INTO Customer VALUES (295,'Consumer','Kenneth','Anderson',NULL)
INSERT INTO Customer VALUES (296,'Consumer','Donna','Campbell',NULL)
INSERT INTO Customer VALUES (297,'Consumer','Sandra','Puma',NULL)
INSERT INTO Customer VALUES (298,'Consumer','Kimberly','Campbell',NULL)
INSERT INTO Customer VALUES (299,'Consumer','Donald','Allen',NULL)
INSERT INTO Customer VALUES (300,'Consumer','Thomas','Wilson',NULL)
INSERT INTO Customer VALUES (301,'Consumer','Jennifer','Davis',NULL)
INSERT INTO Customer VALUES (302,'Consumer','Laura','Wright',NULL)
INSERT INTO Customer VALUES (303,'Consumer','Mark','White',NULL)
INSERT INTO Customer VALUES (304,'Consumer','Nancy','Robinson',NULL)
INSERT INTO Customer VALUES (305,'Consumer','James','Robinson',NULL)
INSERT INTO Customer VALUES (306,'Consumer','Elizabeth','Mitchell',NULL)
INSERT INTO Customer VALUES (307,'Consumer','Nancy','Robinson',NULL)
INSERT INTO Customer VALUES (308,'Consumer','Betty','Lewis',NULL)
INSERT INTO Customer VALUES (309,'Consumer','Betty','Campbell',NULL)
INSERT INTO Customer VALUES (310,'Consumer',' Robert','Thomas',NULL)
INSERT INTO Customer VALUES (311,'Consumer','Jennifer','Hernandez',NULL)
INSERT INTO Customer VALUES (312,'Consumer','Donald','Nelson',NULL)
INSERT INTO Customer VALUES (313,'Consumer','Donald','Turner',NULL)
INSERT INTO Customer VALUES (314,'Consumer','John','Thompson',NULL)
INSERT INTO Customer VALUES (315,'Consumer','Sanjay','Rodriguez',NULL)
INSERT INTO Customer VALUES (316,'Consumer','Elizabeth','Rodriguez',NULL)
INSERT INTO Customer VALUES (317,'Consumer','Elizabeth','Wilson',NULL)
INSERT INTO Customer VALUES (318,'Consumer','Barbara','White',NULL)
INSERT INTO Customer VALUES (319,'Consumer',' Robert','Lewis',NULL)
INSERT INTO Customer VALUES (320,'Consumer','Patricia','Evans',NULL)
INSERT INTO Customer VALUES (321,'Consumer','Edward','Lopez',NULL)
INSERT INTO Customer VALUES (322,'Consumer','Edward','Wilson',NULL)
INSERT INTO Customer VALUES (323,'Consumer','Anthony','Taylor',NULL)
INSERT INTO Customer VALUES (324,'Consumer','Pedro','Gonzalez',NULL)
INSERT INTO Customer VALUES (325,'Consumer','Patricia','Harris',NULL)
INSERT INTO Customer VALUES (326,'Consumer','Karen','Walker',NULL)
INSERT INTO Customer VALUES (327,'Consumer','George','King',NULL)
INSERT INTO Customer VALUES (328,'Consumer','William','Wright',NULL)
INSERT INTO Customer VALUES (329,'Consumer','Jennifer','Wright',NULL)
INSERT INTO Customer VALUES (330,'Consumer','Tessa','Allen',NULL)
INSERT INTO Customer VALUES (331,'Consumer','Sanjay','Wright',NULL)
INSERT INTO Customer VALUES (332,'Consumer','Betty','Hernandez',NULL)
INSERT INTO Customer VALUES (333,'Consumer','Karen','Phillips',NULL)
INSERT INTO Customer VALUES (334,'Consumer','Elizabeth','Phillips',NULL)
INSERT INTO Customer VALUES (335,'Consumer','Betty','Baker',NULL)
INSERT INTO Customer VALUES (336,'Consumer','Kenneth','Jackson',NULL)
INSERT INTO Customer VALUES (337,'Consumer','Kevin','Taylor',NULL)
INSERT INTO Customer VALUES (338,'Consumer','George','Wright',NULL)
INSERT INTO Customer VALUES (339,'Consumer','Jeff','Wright',NULL)
INSERT INTO Customer VALUES (340,'Consumer','George','Martinez',NULL)
INSERT INTO Customer VALUES (341,'Consumer','Kevin','Hill',NULL)
INSERT INTO Customer VALUES (342,'Consumer','Tessa','Baker',NULL)
INSERT INTO Customer VALUES (343,'Consumer','Mark','Martin',NULL)
INSERT INTO Customer VALUES (344,'Consumer','Linda','Adams',NULL)
INSERT INTO Customer VALUES (345,'Consumer','Steven','Garcia',NULL)
INSERT INTO Customer VALUES (346,'Consumer','Elizabeth','Davis',NULL)
INSERT INTO Customer VALUES (347,'Consumer','Sanjay','Smith',NULL)
INSERT INTO Customer VALUES (348,'Consumer','Linda','Anderson',NULL)
INSERT INTO Customer VALUES (349,'Consumer','Joseph','Thomas',NULL)
INSERT INTO Customer VALUES (350,'Consumer','Ruth','Lopez',NULL)
INSERT INTO Customer VALUES (351,'Consumer','Jason','Moore',NULL)
INSERT INTO Customer VALUES (352,'Consumer','William','Edwards',NULL)
INSERT INTO Customer VALUES (353,'Consumer','Donald','Thomas',NULL)
INSERT INTO Customer VALUES (354,'Consumer','Richard','Hall',NULL)
INSERT INTO Customer VALUES (355,'Consumer','Anthony','Rodriguez',NULL)
INSERT INTO Customer VALUES (356,'Consumer','Barbara','Lopez',NULL)
INSERT INTO Customer VALUES (357,'Consumer','William','Thomas',NULL)
INSERT INTO Customer VALUES (358,'Consumer','Donald','Martinez',NULL)
INSERT INTO Customer VALUES (359,'Consumer','Paul','Parker',NULL)
INSERT INTO Customer VALUES (360,'Consumer','Sanjay','Johnson',NULL)
INSERT INTO Customer VALUES (361,'Consumer','Anthony','Clark',NULL)
INSERT INTO Customer VALUES (362,'Consumer','Ronald','Puma',NULL)
INSERT INTO Customer VALUES (363,'Consumer','Paul','Thomas',NULL)
INSERT INTO Customer VALUES (364,'Consumer','Richard','Jackson',NULL)
INSERT INTO Customer VALUES (365,'Consumer',' Robert','Lewis',NULL)
INSERT INTO Customer VALUES (366,'Consumer','Richard','Lopez',NULL)
INSERT INTO Customer VALUES (367,'Consumer','Susan','Wright',NULL)
INSERT INTO Customer VALUES (368,'Consumer','Steven','Phillips',NULL)
INSERT INTO Customer VALUES (369,'Consumer','Pedro','Carter',NULL)
INSERT INTO Customer VALUES (370,'Consumer','Susan','Nelson',NULL)
INSERT INTO Customer VALUES (371,'Consumer','Karen','Hall',NULL)
INSERT INTO Customer VALUES (372,'Consumer','Donna','Edwards',NULL)
INSERT INTO Customer VALUES (373,'Consumer','Patricia','Martin',NULL)
INSERT INTO Customer VALUES (374,'Consumer','William','Turner',NULL)
INSERT INTO Customer VALUES (375,'Consumer','Ronald','Smith',NULL)
INSERT INTO Customer VALUES (376,'Consumer','Deborah','Hill',NULL)
INSERT INTO Customer VALUES (377,'Consumer','Jennifer','Allen',NULL)
INSERT INTO Customer VALUES (378,'Consumer','Karen','Robinson',NULL)
INSERT INTO Customer VALUES (379,'Consumer','Mark','Jackson',NULL)
INSERT INTO Customer VALUES (380,'Consumer','Pedro','Martinez',NULL)
INSERT INTO Customer VALUES (381,'Consumer','Kenneth','Hill',NULL)
INSERT INTO Customer VALUES (382,'Consumer','Patricia','Hill',NULL)
INSERT INTO Customer VALUES (383,'Consumer','Linda','Rodriguez',NULL)
INSERT INTO Customer VALUES (384,'Consumer','Mary','Thomas',NULL)
INSERT INTO Customer VALUES (385,'Consumer','William','Martin',NULL)
INSERT INTO Customer VALUES (386,'Consumer','Lisa','Rodriguez',NULL)
INSERT INTO Customer VALUES (387,'Consumer','Joseph','Carter',NULL)
INSERT INTO Customer VALUES (388,'Consumer','Karen','Hill',NULL)
INSERT INTO Customer VALUES (389,'Consumer','Ruth','Lopez',NULL)
INSERT INTO Customer VALUES (390,'Consumer','Daniel','King',NULL)
INSERT INTO Customer VALUES (391,'Consumer','Mary','Harris',NULL)
INSERT INTO Customer VALUES (392,'Consumer','Charles','Lopez',NULL)
INSERT INTO Customer VALUES (393,'Consumer',' Robert','Turner',NULL)
INSERT INTO Customer VALUES (394,'Consumer','Karen','Wilson',NULL)
INSERT INTO Customer VALUES (395,'Consumer','Donna','White',NULL)
INSERT INTO Customer VALUES (396,'Consumer','Carol','Smith',NULL)
INSERT INTO Customer VALUES (397,'Consumer','Thomas','Hall',NULL)
INSERT INTO Customer VALUES (398,'Consumer','David','Wilson',NULL)
INSERT INTO Customer VALUES (399,'Consumer','Brian','Thomas',NULL)
INSERT INTO Customer VALUES (400,'Consumer','Donald','Lopez',NULL)
INSERT INTO Customer VALUES (401,'Consumer','Karen','Hall',NULL)
INSERT INTO Customer VALUES (402,'Consumer','Carol','Campbell',NULL)
INSERT INTO Customer VALUES (403,'Consumer','Ruth','Martinez',NULL)
INSERT INTO Customer VALUES (404,'Consumer','Mary','Smith',NULL)
INSERT INTO Customer VALUES (405,'Consumer','Rick','Rodriguez',NULL)
INSERT INTO Customer VALUES (406,'Consumer','Laura','Thompson',NULL)
INSERT INTO Customer VALUES (407,'Consumer','Deborah','Clark',NULL)
INSERT INTO Customer VALUES (408,'Consumer','Donald','Brown',NULL)
INSERT INTO Customer VALUES (409,'Consumer','David','Evans',NULL)
INSERT INTO Customer VALUES (410,'Consumer','Ronald','Lopez',NULL)
INSERT INTO Customer VALUES (411,'Consumer','Kenneth','Perez',NULL)
INSERT INTO Customer VALUES (412,'Consumer','Edward','Hill',NULL)
INSERT INTO Customer VALUES (413,'Consumer','Maria','Harris',NULL)
INSERT INTO Customer VALUES (414,'Consumer','Jeff','Brown',NULL)
INSERT INTO Customer VALUES (415,'Consumer','Mark','King',NULL)
INSERT INTO Customer VALUES (416,'Consumer','Elizabeth','Baker',NULL)
INSERT INTO Customer VALUES (417,'Consumer','Brian','Hill',NULL)
INSERT INTO Customer VALUES (418,'Consumer','Donna','Gonzalez',NULL)
INSERT INTO Customer VALUES (419,'Consumer','Ruth','Jones',NULL)
INSERT INTO Customer VALUES (420,'Consumer','Lee','Jackson',NULL)
INSERT INTO Customer VALUES (421,'Consumer','Donald','Taylor',NULL)
INSERT INTO Customer VALUES (422,'Consumer','Steven','Robinson',NULL)
INSERT INTO Customer VALUES (423,'Consumer','Rick','Collins',NULL)
INSERT INTO Customer VALUES (424,'Consumer','Linda','Gonzalez',NULL)
INSERT INTO Customer VALUES (425,'Consumer','Mark','Robinson',NULL)
INSERT INTO Customer VALUES (426,'Consumer','Sarah','Martin',NULL)
INSERT INTO Customer VALUES (427,'Consumer','Brian','Williams',NULL)
INSERT INTO Customer VALUES (428,'Consumer','Edward','Thompson',NULL)
INSERT INTO Customer VALUES (429,'Consumer','Susan','Taylor',NULL)
INSERT INTO Customer VALUES (430,'Consumer','William','Lee',NULL)
INSERT INTO Customer VALUES (431,'Consumer','Jennifer','Parker',NULL)
INSERT INTO Customer VALUES (432,'Consumer','Nancy','Carter',NULL)
INSERT INTO Customer VALUES (433,'Consumer','Joseph','Baker',NULL)
INSERT INTO Customer VALUES (434,'Consumer','Deborah','Davis',NULL)
INSERT INTO Customer VALUES (435,'Consumer','Anthony','Young',NULL)
INSERT INTO Customer VALUES (436,'Consumer','Nancy','Hall',NULL)
INSERT INTO Customer VALUES (437,'Consumer','Betty','Lewis',NULL)
INSERT INTO Customer VALUES (438,'Consumer','William','Nelson',NULL)
INSERT INTO Customer VALUES (439,'Consumer','Sanjay','Campbell',NULL)
INSERT INTO Customer VALUES (440,'Consumer','Mary','Green',NULL)
INSERT INTO Customer VALUES (441,'Consumer','Tessa','King',NULL)
INSERT INTO Customer VALUES (442,'Consumer','George','Anderson',NULL)
INSERT INTO Customer VALUES (443,'Consumer','Karen','Hill',NULL)
INSERT INTO Customer VALUES (444,'Consumer','Betty','Robinson',NULL)
INSERT INTO Customer VALUES (445,'Consumer','Edward','Perez',NULL)
INSERT INTO Customer VALUES (446,'Consumer','Sarah','Green',NULL)
INSERT INTO Customer VALUES (447,'Consumer','Mary','Garcia',NULL)
INSERT INTO Customer VALUES (448,'Consumer','Jennifer','Anderson',NULL)
INSERT INTO Customer VALUES (449,'Consumer','Mary','Turner',NULL)
INSERT INTO Customer VALUES (450,'Consumer','David','Harris',NULL)
INSERT INTO Customer VALUES (451,'Consumer','Pedro','Jackson',NULL)
INSERT INTO Customer VALUES (452,'Consumer','Steven','Jackson',NULL)
INSERT INTO Customer VALUES (453,'Consumer','Elizabeth','Smith',NULL)
INSERT INTO Customer VALUES (454,'Consumer','Donald','Baker',NULL)
INSERT INTO Customer VALUES (455,'Consumer','Jeff','White',NULL)
INSERT INTO Customer VALUES (456,'Consumer','Kenneth','Williams',NULL)
INSERT INTO Customer VALUES (457,'Consumer','Ronald','Phillips',NULL)
INSERT INTO Customer VALUES (458,'Consumer','Jeff','Scott',NULL)
INSERT INTO Customer VALUES (459,'Consumer','John','Walker',NULL)
INSERT INTO Customer VALUES (460,'Consumer','Daniel','Walker',NULL)
INSERT INTO Customer VALUES (461,'Consumer','Linda','Johnson',NULL)
INSERT INTO Customer VALUES (462,'Consumer','Paul','Smith',NULL)
INSERT INTO Customer VALUES (463,'Consumer','Betty','Hernandez',NULL)
INSERT INTO Customer VALUES (464,'Consumer','Donald','Garcia',NULL)
INSERT INTO Customer VALUES (465,'Consumer','Lee','Perez',NULL)
INSERT INTO Customer VALUES (466,'Consumer','Lisa','Clark',NULL)
INSERT INTO Customer VALUES (467,'Consumer','Brian','Harris',NULL)
INSERT INTO Customer VALUES (468,'Consumer','Donald','King',NULL)
INSERT INTO Customer VALUES (469,'Consumer','Deborah','Carter',NULL)
INSERT INTO Customer VALUES (470,'Consumer','Kenneth','Hill',NULL)
INSERT INTO Customer VALUES (471,'Consumer','Donna','Baker',NULL)
INSERT INTO Customer VALUES (472,'Consumer','Mary','Robinson',NULL)
INSERT INTO Customer VALUES (473,'Consumer','Kenneth','Hall',NULL)
INSERT INTO Customer VALUES (474,'Consumer','Barbara','Allen',NULL)
INSERT INTO Customer VALUES (475,'Consumer','Carol','Roberts',NULL)
INSERT INTO Customer VALUES (476,'Consumer','Tessa','Anderson',NULL)
INSERT INTO Customer VALUES (477,'Consumer','Kimberly','Puma',NULL)
INSERT INTO Customer VALUES (478,'Consumer','Donald','Davis',NULL)
INSERT INTO Customer VALUES (479,'Consumer','Edward','Puma',NULL)
INSERT INTO Customer VALUES (480,'Consumer','Susan','Wright',NULL)
INSERT INTO Customer VALUES (481,'Consumer','Donald','Turner',NULL)
INSERT INTO Customer VALUES (482,'Consumer','Jeff','Wilson',NULL)
INSERT INTO Customer VALUES (483,'Consumer','Thomas','Thomas',NULL)
INSERT INTO Customer VALUES (484,'Consumer','John','Young',NULL)
INSERT INTO Customer VALUES (485,'Consumer','Steven','Phillips',NULL)
INSERT INTO Customer VALUES (486,'Consumer','Steven','Harris',NULL)
INSERT INTO Customer VALUES (487,'Consumer','Donald','Taylor',NULL)
INSERT INTO Customer VALUES (488,'Consumer','Paul','Johnson',NULL)
INSERT INTO Customer VALUES (489,'Consumer','Jeff','Jones',NULL)
INSERT INTO Customer VALUES (490,'Consumer','Donald','Miller',NULL)
INSERT INTO Customer VALUES (491,'Consumer','Pedro','Clark',NULL)
INSERT INTO Customer VALUES (492,'Consumer','Donald','Walker',NULL)
INSERT INTO Customer VALUES (493,'Consumer','Thomas','Wright',NULL)
INSERT INTO Customer VALUES (494,'Consumer','Jennifer','Moore',NULL)
INSERT INTO Customer VALUES (495,'Consumer','Paul','Carter',NULL)
INSERT INTO Customer VALUES (496,'Consumer','Sarah','Adams',NULL)
INSERT INTO Customer VALUES (497,'Consumer','Mary','Rodriguez',NULL)
INSERT INTO Customer VALUES (498,'Consumer','Jason','Campbell',NULL)
INSERT INTO Customer VALUES (499,'Consumer','Donald','Campbell',NULL)
INSERT INTO Customer VALUES (500,'Consumer','Pedro','Lopez',NULL)
GO


INSERT INTO SalesInvoice VALUES (1,38718,38726.140793859,472,NULL)
INSERT INTO SalesInvoice VALUES (2,38719.0990911428,38747.177483682,388,NULL)
INSERT INTO SalesInvoice VALUES (3,38719.2312633406,38760.5729465349,279,NULL)
INSERT INTO SalesInvoice VALUES (4,38719.9215122434,38754.9209982502,309,NULL)
INSERT INTO SalesInvoice VALUES (5,38720.4845555224,38756.8343349879,757,NULL)
INSERT INTO SalesInvoice VALUES (6,38721.9953046514,38743.9500694127,493,NULL)
INSERT INTO SalesInvoice VALUES (7,38723.3378843757,38751.3624821028,209,NULL)
INSERT INTO SalesInvoice VALUES (8,38723.5446020246,38763.2800695646,649,NULL)
INSERT INTO SalesInvoice VALUES (9,38723.9069802573,38742.1701616779,597,NULL)
INSERT INTO SalesInvoice VALUES (10,38724.8563354403,38766.6597700722,736,NULL)
INSERT INTO SalesInvoice VALUES (11,38725.9380131387,38733.7070695694,329,NULL)
INSERT INTO SalesInvoice VALUES (12,38726.2965840168,38737.2744150853,52,NULL)
INSERT INTO SalesInvoice VALUES (13,38727.3520370904,38760.3536268323,234,NULL)
INSERT INTO SalesInvoice VALUES (14,38727.836117162,38741.3207648102,772,NULL)
INSERT INTO SalesInvoice VALUES (15,38728.4445165595,38766.9413273431,47,NULL)
INSERT INTO SalesInvoice VALUES (16,38729.8030210647,38744.2573964671,152,NULL)
INSERT INTO SalesInvoice VALUES (17,38731.1189765359,38735.6394550788,394,NULL)
INSERT INTO SalesInvoice VALUES (18,38731.1765002043,38752.7279870284,746,NULL)
INSERT INTO SalesInvoice VALUES (19,38731.7502887695,38739.0099258457,251,NULL)
INSERT INTO SalesInvoice VALUES (20,38732.2753296688,38739.3867178327,529,NULL)
INSERT INTO SalesInvoice VALUES (21,38732.9030929746,38746.4374583964,240,NULL)
INSERT INTO SalesInvoice VALUES (22,38732.9869616949,38761.2578305737,151,NULL)
INSERT INTO SalesInvoice VALUES (23,38733.2051470061,38769.3401730588,191,NULL)
INSERT INTO SalesInvoice VALUES (24,38733.4341754005,38767.1763315374,380,NULL)
INSERT INTO SalesInvoice VALUES (25,38733.7928842985,38754.9447830071,231,NULL)
INSERT INTO SalesInvoice VALUES (26,38734.873225962,38737.4101827192,696,NULL)
INSERT INTO SalesInvoice VALUES (27,38735.8722212943,38743.3282242907,306,NULL)
INSERT INTO SalesInvoice VALUES (28,38737.2276242182,38747.2626011986,37,NULL)
INSERT INTO SalesInvoice VALUES (29,38737.5374658517,38773.9676236816,461,NULL)
INSERT INTO SalesInvoice VALUES (30,38738.1140238226,38748.2884122791,346,NULL)
INSERT INTO SalesInvoice VALUES (31,38738.3643707275,38742.3946148848,98,NULL)
INSERT INTO SalesInvoice VALUES (32,38738.9511152976,38768.2640048507,453,NULL)
INSERT INTO SalesInvoice VALUES (33,38739.8826353079,38766.1122286331,754,NULL)
INSERT INTO SalesInvoice VALUES (34,38741.39628395,38783.3047008761,179,NULL)
INSERT INTO SalesInvoice VALUES (35,38741.7115064215,38787.0832965861,388,NULL)
INSERT INTO SalesInvoice VALUES (36,38742.0050497398,38783.6525533684,48,NULL)
INSERT INTO SalesInvoice VALUES (37,38742.8504968656,38760.1014531724,720,NULL)
INSERT INTO SalesInvoice VALUES (38,38743.7318774566,38791.895553286,74,NULL)
INSERT INTO SalesInvoice VALUES (39,38744.7258570813,38793.7893767066,682,NULL)
INSERT INTO SalesInvoice VALUES (40,38746.0297442152,38784.4907007577,136,NULL)
INSERT INTO SalesInvoice VALUES (41,38746.9561316035,38774.8872928472,514,NULL)
INSERT INTO SalesInvoice VALUES (42,38748.1709448483,38793.3938158297,281,NULL)
INSERT INTO SalesInvoice VALUES (43,38749.1099988511,38767.8238152888,196,NULL)
INSERT INTO SalesInvoice VALUES (44,38750.3530875098,38788.5246993056,683,NULL)
INSERT INTO SalesInvoice VALUES (45,38750.7371562395,38752.9872234833,459,NULL)
INSERT INTO SalesInvoice VALUES (46,38751.7743056589,38777.0075792641,155,NULL)
INSERT INTO SalesInvoice VALUES (47,38752.9845026683,38767.7670589205,323,NULL)
INSERT INTO SalesInvoice VALUES (48,38754.5022275757,38755.591440466,700,NULL)
INSERT INTO SalesInvoice VALUES (49,38755.6029234932,38796.7789032811,69,NULL)
INSERT INTO SalesInvoice VALUES (50,38756.587462653,38758.8036369451,253,NULL)
INSERT INTO SalesInvoice VALUES (51,38757.2792241018,38801.0022541686,564,NULL)
INSERT INTO SalesInvoice VALUES (52,38758.007366426,38761.5793654888,346,NULL)
INSERT INTO SalesInvoice VALUES (53,38758.1670354107,38802.6095281397,282,NULL)
INSERT INTO SalesInvoice VALUES (54,38758.4510909077,38790.5979351624,377,NULL)
INSERT INTO SalesInvoice VALUES (55,38759.7418912251,38794.6072000998,642,NULL)
INSERT INTO SalesInvoice VALUES (56,38760.6125474154,38809.0924658353,575,NULL)
INSERT INTO SalesInvoice VALUES (57,38762.0748171334,38779.3321228876,344,NULL)
INSERT INTO SalesInvoice VALUES (58,38763.1088004561,38767.9816789464,649,NULL)
INSERT INTO SalesInvoice VALUES (59,38764.1094713329,38784.6456617673,717,NULL)
INSERT INTO SalesInvoice VALUES (60,38764.7666976103,38799.2214578249,235,NULL)
INSERT INTO SalesInvoice VALUES (61,38766.0339768503,38793.9393219604,552,NULL)
INSERT INTO SalesInvoice VALUES (62,38766.0441064797,38774.2888031732,346,NULL)
INSERT INTO SalesInvoice VALUES (63,38767.1127049262,38786.1730569637,760,NULL)
INSERT INTO SalesInvoice VALUES (64,38768.6925997182,38807.4203570075,540,NULL)
INSERT INTO SalesInvoice VALUES (65,38770.0663995335,38790.1032181873,22,NULL)
INSERT INTO SalesInvoice VALUES (66,38771.3613219383,38776.4594021817,504,NULL)
INSERT INTO SalesInvoice VALUES (67,38772.2201040777,38787.011640473,695,NULL)
INSERT INTO SalesInvoice VALUES (68,38773.7770280238,38806.1511234016,695,NULL)
INSERT INTO SalesInvoice VALUES (69,38775.3217890288,38799.1146821273,12,NULL)
INSERT INTO SalesInvoice VALUES (70,38776.2600561458,38777.7842118518,215,NULL)
INSERT INTO SalesInvoice VALUES (71,38776.4330363059,38810.4293650436,692,NULL)
INSERT INTO SalesInvoice VALUES (72,38777.016443558,38823.1052152497,747,NULL)
INSERT INTO SalesInvoice VALUES (73,38778.4929111724,38805.082578113,281,NULL)
INSERT INTO SalesInvoice VALUES (74,38779.7402004078,38792.1233658655,438,NULL)
INSERT INTO SalesInvoice VALUES (75,38780.4818109062,38789.5553509163,136,NULL)
INSERT INTO SalesInvoice VALUES (76,38781.1145203662,38825.7654518439,548,NULL)
INSERT INTO SalesInvoice VALUES (77,38781.9660873679,38809.6263890496,368,NULL)
INSERT INTO SalesInvoice VALUES (78,38781.9783132148,38806.1954529126,605,NULL)
INSERT INTO SalesInvoice VALUES (79,38783.3428042146,38796.5448255355,723,NULL)
INSERT INTO SalesInvoice VALUES (80,38783.5388688458,38812.3195973515,176,NULL)
INSERT INTO SalesInvoice VALUES (81,38783.891876063,38808.4130754467,251,NULL)
INSERT INTO SalesInvoice VALUES (82,38784.5018324743,38834.1478083414,284,NULL)
INSERT INTO SalesInvoice VALUES (83,38785.7873301238,38805.5372342508,283,NULL)
INSERT INTO SalesInvoice VALUES (84,38786.5649878649,38817.1198806381,555,NULL)
INSERT INTO SalesInvoice VALUES (85,38787.4452184824,38836.318221908,449,NULL)
INSERT INTO SalesInvoice VALUES (86,38787.5967850526,38797.9533858781,363,NULL)
INSERT INTO SalesInvoice VALUES (87,38788.6301232483,38789.4537023986,343,NULL)
INSERT INTO SalesInvoice VALUES (88,38789.4480484015,38812.2921163166,97,NULL)
INSERT INTO SalesInvoice VALUES (89,38790.0195513724,38796.502853651,423,NULL)
INSERT INTO SalesInvoice VALUES (90,38790.9915324196,38834.9245157414,736,NULL)
INSERT INTO SalesInvoice VALUES (91,38791.6189292057,38827.0208866979,635,NULL)
INSERT INTO SalesInvoice VALUES (92,38791.6752061189,38820.6840665928,416,NULL)
INSERT INTO SalesInvoice VALUES (93,38793.0601596477,38827.302987873,206,NULL)
INSERT INTO SalesInvoice VALUES (94,38794.0915540058,38840.9590770236,250,NULL)
INSERT INTO SalesInvoice VALUES (95,38795.4064463569,38810.8101975233,416,NULL)
INSERT INTO SalesInvoice VALUES (96,38796.3444996488,38827.6825080785,538,NULL)
INSERT INTO SalesInvoice VALUES (97,38797.4698022684,38825.0330987443,504,NULL)
INSERT INTO SalesInvoice VALUES (98,38798.9758592098,38843.5204242071,679,NULL)
INSERT INTO SalesInvoice VALUES (99,38799.9688282093,38848.3293616435,665,NULL)
INSERT INTO SalesInvoice VALUES (100,38801.0953269203,38814.6459109301,557,NULL)
INSERT INTO SalesInvoice VALUES (101,38802.4563359253,38823.6116034562,529,NULL)
INSERT INTO SalesInvoice VALUES (102,38803.5861938809,38851.8609164304,79,NULL)
INSERT INTO SalesInvoice VALUES (103,38804.1771322494,38822.8913117482,440,NULL)
INSERT INTO SalesInvoice VALUES (104,38805.154829564,38826.5390499292,430,NULL)
INSERT INTO SalesInvoice VALUES (105,38805.2184351006,38854.5877059318,42,NULL)
INSERT INTO SalesInvoice VALUES (106,38805.7490867623,38817.9969093652,57,NULL)
INSERT INTO SalesInvoice VALUES (107,38807.196340494,38813.0914217734,665,NULL)
INSERT INTO SalesInvoice VALUES (108,38807.2663009714,38844.6639741173,168,NULL)
INSERT INTO SalesInvoice VALUES (109,38807.3874567196,38833.5779348128,337,NULL)
INSERT INTO SalesInvoice VALUES (110,38807.7330616085,38853.803371782,651,NULL)
INSERT INTO SalesInvoice VALUES (111,38808.9781646387,38853.7712506828,249,NULL)
INSERT INTO SalesInvoice VALUES (112,38809.1664794061,38833.3955197776,150,NULL)
INSERT INTO SalesInvoice VALUES (113,38809.8733360939,38847.5453158895,95,NULL)
INSERT INTO SalesInvoice VALUES (114,38810.7109115217,38830.2430435595,656,NULL)
INSERT INTO SalesInvoice VALUES (115,38811.7310518512,38852.2687037575,466,NULL)
INSERT INTO SalesInvoice VALUES (116,38812.9931984477,38852.3032058587,134,NULL)
INSERT INTO SalesInvoice VALUES (117,38814.0519461015,38857.269095617,230,NULL)
INSERT INTO SalesInvoice VALUES (118,38814.2831336775,38863.2754549001,584,NULL)
INSERT INTO SalesInvoice VALUES (119,38815.5552096673,38847.4106746713,31,NULL)
INSERT INTO SalesInvoice VALUES (120,38816.0102136216,38864.0255025731,348,NULL)
INSERT INTO SalesInvoice VALUES (121,38816.5466248304,38860.0282445648,640,NULL)
INSERT INTO SalesInvoice VALUES (122,38817.5121055047,38823.5120729269,510,NULL)
INSERT INTO SalesInvoice VALUES (123,38818.8579286589,38856.5903819773,633,NULL)
INSERT INTO SalesInvoice VALUES (124,38819.4805315929,38853.8181781507,557,NULL)
INSERT INTO SalesInvoice VALUES (125,38820.8028825126,38843.004477958,30,NULL)
INSERT INTO SalesInvoice VALUES (126,38821.4170132856,38865.3675050379,394,NULL)
INSERT INTO SalesInvoice VALUES (127,38822.9530054938,38825.7329273634,648,NULL)
INSERT INTO SalesInvoice VALUES (128,38823.2642628745,38865.5767142827,197,NULL)
INSERT INTO SalesInvoice VALUES (129,38823.2827064254,38846.7955603474,613,NULL)
INSERT INTO SalesInvoice VALUES (130,38824.0666742341,38869.4947967093,33,NULL)
INSERT INTO SalesInvoice VALUES (131,38824.9349660565,38870.38279091,96,NULL)
INSERT INTO SalesInvoice VALUES (132,38825.3335519072,38867.6845346321,350,NULL)
INSERT INTO SalesInvoice VALUES (133,38825.5086682828,38849.2054679956,702,NULL)
INSERT INTO SalesInvoice VALUES (134,38826.2800824246,38859.4612777774,746,NULL)
INSERT INTO SalesInvoice VALUES (135,38827.3258960855,38872.4782904775,335,NULL)
INSERT INTO SalesInvoice VALUES (136,38827.3640590984,38832.5136242312,459,NULL)
INSERT INTO SalesInvoice VALUES (137,38828.7340045916,38870.9499042295,697,NULL)
INSERT INTO SalesInvoice VALUES (138,38829.8151674581,38866.2689568753,637,NULL)
INSERT INTO SalesInvoice VALUES (139,38831.33755404,38862.0846475959,151,NULL)
INSERT INTO SalesInvoice VALUES (140,38832.6564057389,38869.1408440906,35,NULL)
INSERT INTO SalesInvoice VALUES (141,38834.008755453,38844.4350747408,652,NULL)
INSERT INTO SalesInvoice VALUES (142,38834.5959965619,38853.6684524808,215,NULL)
INSERT INTO SalesInvoice VALUES (143,38834.984639475,38870.7303115708,508,NULL)
INSERT INTO SalesInvoice VALUES (144,38835.0660021237,38859.7781118582,548,NULL)
INSERT INTO SalesInvoice VALUES (145,38835.7298384057,38847.1129797267,503,NULL)
INSERT INTO SalesInvoice VALUES (146,38836.183196733,38863.5006625491,50,NULL)
INSERT INTO SalesInvoice VALUES (147,38836.800835898,38846.7940140774,106,NULL)
INSERT INTO SalesInvoice VALUES (148,38838.2015572901,38872.0212897629,575,NULL)
INSERT INTO SalesInvoice VALUES (149,38839.6229241889,38887.7595579459,760,NULL)
INSERT INTO SalesInvoice VALUES (150,38840.4910739342,38889.5052465576,748,NULL)
INSERT INTO SalesInvoice VALUES (151,38842.0638306078,38846.0229734834,227,NULL)
INSERT INTO SalesInvoice VALUES (152,38843.1908230821,38860.5248459542,626,NULL)
INSERT INTO SalesInvoice VALUES (153,38844.1039013877,38863.6495698738,549,NULL)
INSERT INTO SalesInvoice VALUES (154,38845.5274215553,38850.0824112374,190,NULL)
INSERT INTO SalesInvoice VALUES (155,38845.7324980353,38877.5736483172,100,NULL)
INSERT INTO SalesInvoice VALUES (156,38845.8764781043,38872.8922191662,522,NULL)
INSERT INTO SalesInvoice VALUES (157,38847.3606966436,38855.2048211128,441,NULL)
INSERT INTO SalesInvoice VALUES (158,38848.7463005567,38853.16437755,657,NULL)
INSERT INTO SalesInvoice VALUES (159,38848.7983947491,38878.6280836671,722,NULL)
INSERT INTO SalesInvoice VALUES (160,38849.2977282359,38864.7863765227,749,NULL)
INSERT INTO SalesInvoice VALUES (161,38849.673184963,38873.8748374418,8,NULL)
INSERT INTO SalesInvoice VALUES (162,38851.0230020711,38888.9245880664,726,NULL)
INSERT INTO SalesInvoice VALUES (163,38852.1656401434,38888.9915925949,650,NULL)
INSERT INTO SalesInvoice VALUES (164,38852.2779531091,38862.4976666685,405,NULL)
INSERT INTO SalesInvoice VALUES (165,38852.6549348954,38875.5702424186,202,NULL)
INSERT INTO SalesInvoice VALUES (166,38853.5374524316,38872.9057118015,424,NULL)
INSERT INTO SalesInvoice VALUES (167,38854.6035358201,38874.6960374788,504,NULL)
INSERT INTO SalesInvoice VALUES (168,38855.6263468577,38868.3656875914,259,NULL)
INSERT INTO SalesInvoice VALUES (169,38856.9920214311,38877.980118151,627,NULL)
INSERT INTO SalesInvoice VALUES (170,38857.1009048821,38859.4906160532,140,NULL)
INSERT INTO SalesInvoice VALUES (171,38857.8843843771,38893.5147327247,355,NULL)
INSERT INTO SalesInvoice VALUES (172,38859.1641420852,38900.6189928676,474,NULL)
INSERT INTO SalesInvoice VALUES (173,38860.2085493586,38865.7578303155,252,NULL)
INSERT INTO SalesInvoice VALUES (174,38861.7518927881,38875.9723227115,520,NULL)
INSERT INTO SalesInvoice VALUES (175,38863.2631540809,38882.1949173407,432,NULL)
INSERT INTO SalesInvoice VALUES (176,38863.8933685077,38879.4060923449,141,NULL)
INSERT INTO SalesInvoice VALUES (177,38865.2269537446,38875.678336191,629,NULL)
INSERT INTO SalesInvoice VALUES (178,38866.3480299692,38891.3308940614,33,NULL)
INSERT INTO SalesInvoice VALUES (179,38866.8323482919,38880.348318473,262,NULL)
INSERT INTO SalesInvoice VALUES (180,38867.7908937716,38915.8694205344,92,NULL)
INSERT INTO SalesInvoice VALUES (181,38868.1289702728,38900.4065969574,680,NULL)
INSERT INTO SalesInvoice VALUES (182,38868.9419809737,38905.2467948345,764,NULL)
INSERT INTO SalesInvoice VALUES (183,38869.3016182836,38914.933889246,185,NULL)
INSERT INTO SalesInvoice VALUES (184,38870.8197381834,38918.3509448993,586,NULL)
INSERT INTO SalesInvoice VALUES (185,38871.6525306361,38912.3503522265,291,NULL)
INSERT INTO SalesInvoice VALUES (186,38872.98968819,38904.970710833,575,NULL)
INSERT INTO SalesInvoice VALUES (187,38874.0913833376,38890.8178281897,398,NULL)
INSERT INTO SalesInvoice VALUES (188,38874.9540836065,38877.1784426462,585,NULL)
INSERT INTO SalesInvoice VALUES (189,38876.1018352205,38911.3165912994,702,NULL)
INSERT INTO SalesInvoice VALUES (190,38876.7817039389,38897.0419542401,247,NULL)
INSERT INTO SalesInvoice VALUES (191,38877.6935375405,38896.5097650189,756,NULL)
INSERT INTO SalesInvoice VALUES (192,38879.0851947667,38892.2363456918,454,NULL)
INSERT INTO SalesInvoice VALUES (193,38880.1953531212,38884.4260009711,588,NULL)
INSERT INTO SalesInvoice VALUES (194,38881.1829365724,38907.6924850074,116,NULL)
INSERT INTO SalesInvoice VALUES (195,38881.8098531677,38919.319744046,731,NULL)
INSERT INTO SalesInvoice VALUES (196,38882.088984574,38930.3645868381,234,NULL)
INSERT INTO SalesInvoice VALUES (197,38882.9999674676,38887.3362469464,349,NULL)
INSERT INTO SalesInvoice VALUES (198,38883.3991819584,38884.0789892952,369,NULL)
INSERT INTO SalesInvoice VALUES (199,38884.120460626,38912.6183224996,449,NULL)
INSERT INTO SalesInvoice VALUES (200,38885.4883417901,38909.1542606253,486,NULL)
INSERT INTO SalesInvoice VALUES (201,38886.4764076353,38887.1250095269,617,NULL)
INSERT INTO SalesInvoice VALUES (202,38887.5538053535,38890.1261233975,700,NULL)
INSERT INTO SalesInvoice VALUES (203,38888.3394684037,38911.6866870222,582,NULL)
INSERT INTO SalesInvoice VALUES (204,38888.9527340015,38936.3313504515,694,NULL)
INSERT INTO SalesInvoice VALUES (205,38888.9683797061,38894.3194654683,454,NULL)
INSERT INTO SalesInvoice VALUES (206,38889.853694368,38922.7005193337,355,NULL)
INSERT INTO SalesInvoice VALUES (207,38890.8299570312,38918.520426238,675,NULL)
INSERT INTO SalesInvoice VALUES (208,38891.8319039471,38927.3432702663,585,NULL)
INSERT INTO SalesInvoice VALUES (209,38893.3783660605,38922.4426997355,393,NULL)
INSERT INTO SalesInvoice VALUES (210,38894.1067555186,38928.6088633972,615,NULL)
INSERT INTO SalesInvoice VALUES (211,38894.4303938245,38936.5163679094,661,NULL)
INSERT INTO SalesInvoice VALUES (212,38895.0883121552,38907.0777357913,582,NULL)
INSERT INTO SalesInvoice VALUES (213,38895.5016497773,38926.4179169414,564,NULL)
INSERT INTO SalesInvoice VALUES (214,38897.0525989269,38931.7570047275,388,NULL)
INSERT INTO SalesInvoice VALUES (215,38898.194237062,38915.6020806129,52,NULL)
INSERT INTO SalesInvoice VALUES (216,38898.9602039369,38946.4503461022,94,NULL)
INSERT INTO SalesInvoice VALUES (217,38899.6399077378,38937.6779796738,623,NULL)
INSERT INTO SalesInvoice VALUES (218,38899.8114589051,38916.1485186006,512,NULL)
INSERT INTO SalesInvoice VALUES (219,38900.9700921843,38909.9593020426,91,NULL)
INSERT INTO SalesInvoice VALUES (220,38901.2313676839,38932.4759984343,621,NULL)
INSERT INTO SalesInvoice VALUES (221,38901.907887569,38948.5161852056,617,NULL)
INSERT INTO SalesInvoice VALUES (222,38902.9069609405,38947.3830414101,196,NULL)
INSERT INTO SalesInvoice VALUES (223,38903.1940319716,38904.6828548173,676,NULL)
INSERT INTO SalesInvoice VALUES (224,38903.6984185893,38924.8796219648,44,NULL)
INSERT INTO SalesInvoice VALUES (225,38904.5074369054,38946.3599058355,175,NULL)
INSERT INTO SalesInvoice VALUES (226,38905.0500784313,38950.0921752126,326,NULL)
INSERT INTO SalesInvoice VALUES (227,38906.6124980787,38944.6826444893,685,NULL)
INSERT INTO SalesInvoice VALUES (228,38907.9848268082,38914.1906549955,526,NULL)
INSERT INTO SalesInvoice VALUES (229,38908.047176674,38943.4616725779,596,NULL)
INSERT INTO SalesInvoice VALUES (230,38908.3992350438,38957.6474073869,105,NULL)
INSERT INTO SalesInvoice VALUES (231,38909.5099863014,38912.6612698928,428,NULL)
INSERT INTO SalesInvoice VALUES (232,38910.3769759721,38947.8033795708,549,NULL)
INSERT INTO SalesInvoice VALUES (233,38911.4181171355,38945.1278975122,464,NULL)
INSERT INTO SalesInvoice VALUES (234,38912.894474653,38953.7012928802,685,NULL)
INSERT INTO SalesInvoice VALUES (235,38913.284841221,38952.336600624,60,NULL)
INSERT INTO SalesInvoice VALUES (236,38914.1089644203,38924.9747336357,67,NULL)
INSERT INTO SalesInvoice VALUES (237,38914.527937348,38955.6232900102,733,NULL)
INSERT INTO SalesInvoice VALUES (238,38915.9582549562,38916.3222291877,702,NULL)
INSERT INTO SalesInvoice VALUES (239,38915.9711788715,38917.12562865,538,NULL)
INSERT INTO SalesInvoice VALUES (240,38916.5124171105,38935.3284769997,607,NULL)
INSERT INTO SalesInvoice VALUES (241,38917.8546715241,38927.3349397285,771,NULL)
INSERT INTO SalesInvoice VALUES (242,38917.9790274074,38933.5108376757,276,NULL)
INSERT INTO SalesInvoice VALUES (243,38918.3083171962,38946.8535510504,102,NULL)
INSERT INTO SalesInvoice VALUES (244,38919.3189261802,38962.8555458767,161,NULL)
INSERT INTO SalesInvoice VALUES (245,38920.5130371829,38952.2105127547,539,NULL)
INSERT INTO SalesInvoice VALUES (246,38921.1349277687,38948.3109850496,391,NULL)
INSERT INTO SalesInvoice VALUES (247,38921.7144373587,38957.6093005431,656,NULL)
INSERT INTO SalesInvoice VALUES (248,38923.1605106611,38944.298150079,739,NULL)
INSERT INTO SalesInvoice VALUES (249,38923.9719800583,38925.5378336134,376,NULL)
INSERT INTO SalesInvoice VALUES (250,38925.1801694276,38943.4118633594,26,NULL)
INSERT INTO SalesInvoice VALUES (251,38926.2773127573,38952.5411135717,635,NULL)
INSERT INTO SalesInvoice VALUES (252,38926.9672232058,38941.4251383222,713,NULL)
INSERT INTO SalesInvoice VALUES (253,38928.1106570758,38946.6216385736,771,NULL)
INSERT INTO SalesInvoice VALUES (254,38928.4763190984,38939.1596688232,207,NULL)
INSERT INTO SalesInvoice VALUES (255,38928.9884275972,38946.7764748095,62,NULL)
INSERT INTO SalesInvoice VALUES (256,38929.1799890555,38938.4145184751,371,NULL)
INSERT INTO SalesInvoice VALUES (257,38929.2301122861,38930.4651815079,289,NULL)
INSERT INTO SalesInvoice VALUES (258,38929.8141911377,38934.3020157585,260,NULL)
INSERT INTO SalesInvoice VALUES (259,38931.3192668176,38981.0448440909,387,NULL)
INSERT INTO SalesInvoice VALUES (260,38932.3979436512,38943.0099179466,184,NULL)
INSERT INTO SalesInvoice VALUES (261,38933.6329357497,38949.7927186737,516,NULL)
INSERT INTO SalesInvoice VALUES (262,38934.4114157589,38955.9161557788,529,NULL)
INSERT INTO SalesInvoice VALUES (263,38935.5362967368,38973.2518544114,747,NULL)
INSERT INTO SalesInvoice VALUES (264,38936.9018925915,38971.6666288157,361,NULL)
INSERT INTO SalesInvoice VALUES (265,38937.078282349,38967.5209270603,419,NULL)
INSERT INTO SalesInvoice VALUES (266,38937.600879455,38944.0455667695,238,NULL)
INSERT INTO SalesInvoice VALUES (267,38938.5486090875,38961.8650974069,237,NULL)
INSERT INTO SalesInvoice VALUES (268,38940.1123979432,38985.7932681216,160,NULL)
INSERT INTO SalesInvoice VALUES (269,38940.6949489331,38955.7371891584,692,NULL)
INSERT INTO SalesInvoice VALUES (270,38941.9360175049,38957.2197990595,242,NULL)
INSERT INTO SalesInvoice VALUES (271,38942.0215176129,38956.2769193091,63,NULL)
INSERT INTO SalesInvoice VALUES (272,38943.4994722728,38990.4028516784,662,NULL)
INSERT INTO SalesInvoice VALUES (273,38943.8004589407,38970.5327777343,444,NULL)
INSERT INTO SalesInvoice VALUES (274,38944.4881534001,38984.8651840509,375,NULL)
INSERT INTO SalesInvoice VALUES (275,38945.2911845235,38968.2636705657,534,NULL)
INSERT INTO SalesInvoice VALUES (276,38945.8087827582,38945.8153974831,371,NULL)
INSERT INTO SalesInvoice VALUES (277,38947.2871034333,38963.634992902,157,NULL)
INSERT INTO SalesInvoice VALUES (278,38947.3819688838,38953.9520186251,360,NULL)
INSERT INTO SalesInvoice VALUES (279,38948.4027246345,38957.1177101489,371,NULL)
INSERT INTO SalesInvoice VALUES (280,38949.8227829632,38978.0007858584,166,NULL)
INSERT INTO SalesInvoice VALUES (281,38950.2728693385,38999.0106922548,411,NULL)
INSERT INTO SalesInvoice VALUES (282,38951.8206677529,38967.0912302839,542,NULL)
INSERT INTO SalesInvoice VALUES (283,38951.9034830365,38956.1410924928,618,NULL)
INSERT INTO SalesInvoice VALUES (284,38952.2870025553,38979.4266499475,385,NULL)
INSERT INTO SalesInvoice VALUES (285,38952.5938275495,38975.1441519386,137,NULL)
INSERT INTO SalesInvoice VALUES (286,38953.3339662552,38978.6254758041,307,NULL)
INSERT INTO SalesInvoice VALUES (287,38954.8199069889,38970.165527768,656,NULL)
INSERT INTO SalesInvoice VALUES (288,38956.2498786701,38992.3468858765,241,NULL)
INSERT INTO SalesInvoice VALUES (289,38956.7514019753,38990.0006157942,53,NULL)
INSERT INTO SalesInvoice VALUES (290,38957.5445999165,38983.6274264416,397,NULL)
INSERT INTO SalesInvoice VALUES (291,38958.1516741433,39005.939981688,252,NULL)
INSERT INTO SalesInvoice VALUES (292,38958.9233817589,38959.9979952773,290,NULL)
INSERT INTO SalesInvoice VALUES (293,38959.3296017735,38990.7372556337,569,NULL)
INSERT INTO SalesInvoice VALUES (294,38960.3658375364,38996.8795608918,481,NULL)
INSERT INTO SalesInvoice VALUES (295,38961.1094057489,38985.9590599954,650,NULL)
INSERT INTO SalesInvoice VALUES (296,38962.1035222265,38972.5743629247,450,NULL)
INSERT INTO SalesInvoice VALUES (297,38962.390924525,38992.2485465958,558,NULL)
INSERT INTO SalesInvoice VALUES (298,38963.5224004087,39009.8117185164,393,NULL)
INSERT INTO SalesInvoice VALUES (299,38964.6373495467,39003.53016419,544,NULL)
INSERT INTO SalesInvoice VALUES (300,38965.2148658154,39014.2120576219,279,NULL)
INSERT INTO SalesInvoice VALUES (301,38966.082332703,39012.7972247636,523,NULL)
INSERT INTO SalesInvoice VALUES (302,38967.5531371819,38973.4796130975,385,NULL)
INSERT INTO SalesInvoice VALUES (303,38968.3918248465,38971.8438033335,120,NULL)
INSERT INTO SalesInvoice VALUES (304,38969.8168393054,39005.1503753123,554,NULL)
INSERT INTO SalesInvoice VALUES (305,38969.9826667905,39001.774156288,105,NULL)
INSERT INTO SalesInvoice VALUES (306,38970.4376633247,38985.2681803289,283,NULL)
INSERT INTO SalesInvoice VALUES (307,38971.770593359,38986.658529788,68,NULL)
INSERT INTO SalesInvoice VALUES (308,38971.8095135281,39018.4566190631,561,NULL)
INSERT INTO SalesInvoice VALUES (309,38973.261950449,38986.6637714849,509,NULL)
INSERT INTO SalesInvoice VALUES (310,38974.7601228662,39010.1980384453,368,NULL)
INSERT INTO SalesInvoice VALUES (311,38976.0872667574,39023.5168566124,384,NULL)
INSERT INTO SalesInvoice VALUES (312,38977.540801715,39012.2404438541,432,NULL)
INSERT INTO SalesInvoice VALUES (313,38978.0851176286,39007.7587853352,511,NULL)
INSERT INTO SalesInvoice VALUES (314,38978.977719154,39007.855778913,46,NULL)
INSERT INTO SalesInvoice VALUES (315,38979.0194878783,39010.0199013535,248,NULL)
INSERT INTO SalesInvoice VALUES (316,38979.3614630302,39002.0146764571,12,NULL)
INSERT INTO SalesInvoice VALUES (317,38980.0529175386,39018.3560429211,738,NULL)
INSERT INTO SalesInvoice VALUES (318,38980.552111787,39027.7744387256,12,NULL)
INSERT INTO SalesInvoice VALUES (319,38981.7271986719,38985.7124062051,773,NULL)
INSERT INTO SalesInvoice VALUES (320,38982.158027266,39008.4741580905,82,NULL)
INSERT INTO SalesInvoice VALUES (321,38982.8144075357,39025.9783828316,348,NULL)
INSERT INTO SalesInvoice VALUES (322,38983.0672495776,39013.3988609784,732,NULL)
INSERT INTO SalesInvoice VALUES (323,38984.2168084459,39013.6476880234,676,NULL)
INSERT INTO SalesInvoice VALUES (324,38985.0045907929,38996.8214859397,458,NULL)
INSERT INTO SalesInvoice VALUES (325,38986.102812699,39002.7104278363,445,NULL)
INSERT INTO SalesInvoice VALUES (326,38986.3998512648,38993.8242364494,456,NULL)
INSERT INTO SalesInvoice VALUES (327,38987.157125693,39005.6279406509,398,NULL)
INSERT INTO SalesInvoice VALUES (328,38988.1142875094,39017.5365671422,184,NULL)
INSERT INTO SalesInvoice VALUES (329,38989.192973726,38990.1428322752,597,NULL)
INSERT INTO SalesInvoice VALUES (330,38989.7156464207,39029.6078514808,387,NULL)
INSERT INTO SalesInvoice VALUES (331,38990.5363813159,39029.4733385039,372,NULL)
INSERT INTO SalesInvoice VALUES (332,38991.1606643758,39012.33309699,539,NULL)
INSERT INTO SalesInvoice VALUES (333,38991.5329334308,39040.1623366737,556,NULL)
INSERT INTO SalesInvoice VALUES (334,38992.712190282,39039.184341895,237,NULL)
INSERT INTO SalesInvoice VALUES (335,38992.782717515,39005.4173510933,139,NULL)
INSERT INTO SalesInvoice VALUES (336,38993.9428107336,39022.407673603,274,NULL)
INSERT INTO SalesInvoice VALUES (337,38995.2782721121,39009.1016313551,304,NULL)
INSERT INTO SalesInvoice VALUES (338,38995.9471598174,39026.4793250228,554,NULL)
INSERT INTO SalesInvoice VALUES (339,38997.2857872511,39016.4751025915,74,NULL)
INSERT INTO SalesInvoice VALUES (340,38998.3061932581,39004.6709802044,472,NULL)
INSERT INTO SalesInvoice VALUES (341,38999.3842981405,39043.0337857231,601,NULL)
INSERT INTO SalesInvoice VALUES (342,39000.0434807281,39036.6401701287,50,NULL)
INSERT INTO SalesInvoice VALUES (343,39000.0634828241,39035.742616199,584,NULL)
INSERT INTO SalesInvoice VALUES (344,39001.4402041461,39038.1076384362,367,NULL)
INSERT INTO SalesInvoice VALUES (345,39002.1276695994,39018.5454462358,155,NULL)
INSERT INTO SalesInvoice VALUES (346,39003.1105169998,39033.5308691545,254,NULL)
INSERT INTO SalesInvoice VALUES (347,39004.3873111286,39049.0932073771,307,NULL)
INSERT INTO SalesInvoice VALUES (348,39004.7589712217,39015.4112427823,31,NULL)
INSERT INTO SalesInvoice VALUES (349,39005.5057708872,39006.1549047651,216,NULL)
INSERT INTO SalesInvoice VALUES (350,39006.9836095906,39044.3166509483,15,NULL)
INSERT INTO SalesInvoice VALUES (351,39008.5055870782,39023.1821610954,673,NULL)
INSERT INTO SalesInvoice VALUES (352,39009.6610612374,39027.7093284112,182,NULL)
INSERT INTO SalesInvoice VALUES (353,39010.4598308719,39023.5982047463,68,NULL)
INSERT INTO SalesInvoice VALUES (354,39011.874205236,39045.1300054609,612,NULL)
INSERT INTO SalesInvoice VALUES (355,39013.4049571062,39038.8470441133,706,NULL)
INSERT INTO SalesInvoice VALUES (356,39014.6026630238,39039.5990668808,387,NULL)
INSERT INTO SalesInvoice VALUES (357,39015.7475715486,39053.130818897,143,NULL)
INSERT INTO SalesInvoice VALUES (358,39016.454077456,39058.0953419165,593,NULL)
INSERT INTO SalesInvoice VALUES (359,39017.5481669903,39020.6698202909,482,NULL)
INSERT INTO SalesInvoice VALUES (360,39018.9022441862,39057.3977372442,713,NULL)
INSERT INTO SalesInvoice VALUES (361,39019.20499984,39027.0779193616,576,NULL)
INSERT INTO SalesInvoice VALUES (362,39020.3767640738,39036.9514568513,494,NULL)
INSERT INTO SalesInvoice VALUES (363,39021.2764258601,39021.2899866883,755,NULL)
INSERT INTO SalesInvoice VALUES (364,39022.3590759305,39062.6838345491,408,NULL)
INSERT INTO SalesInvoice VALUES (365,39022.580828804,39039.4985462367,26,NULL)
INSERT INTO SalesInvoice VALUES (366,39023.7277966471,39036.9996207411,192,NULL)
INSERT INTO SalesInvoice VALUES (367,39024.0873204724,39054.4397923247,289,NULL)
INSERT INTO SalesInvoice VALUES (368,39025.3862062063,39047.5077966261,385,NULL)
INSERT INTO SalesInvoice VALUES (369,39026.7601105771,39040.6780798685,553,NULL)
INSERT INTO SalesInvoice VALUES (370,39027.4860692572,39053.2603911536,234,NULL)
INSERT INTO SalesInvoice VALUES (371,39028.9892687693,39074.9767481685,308,NULL)
INSERT INTO SalesInvoice VALUES (372,39029.0245899181,39077.4492443848,768,NULL)
INSERT INTO SalesInvoice VALUES (373,39030.3072996331,39066.6136295935,544,NULL)
INSERT INTO SalesInvoice VALUES (374,39030.307683287,39074.9715843925,142,NULL)
INSERT INTO SalesInvoice VALUES (375,39031.0759951795,39043.7674335729,392,NULL)
INSERT INTO SalesInvoice VALUES (376,39032.2718351427,39072.0627870855,309,NULL)
INSERT INTO SalesInvoice VALUES (377,39033.2649602057,39067.564774866,686,NULL)
INSERT INTO SalesInvoice VALUES (378,39034.6750366131,39057.1808920839,666,NULL)
INSERT INTO SalesInvoice VALUES (379,39034.9687368909,39064.0995537445,313,NULL)
INSERT INTO SalesInvoice VALUES (380,39035.2133298565,39071.2695128646,559,NULL)
INSERT INTO SalesInvoice VALUES (381,39036.2388358838,39048.6410132957,739,NULL)
INSERT INTO SalesInvoice VALUES (382,39037.4220515697,39082.2940227587,332,NULL)
INSERT INTO SalesInvoice VALUES (383,39037.9469173516,39074.2204362223,291,NULL)
INSERT INTO SalesInvoice VALUES (384,39038.6114193641,39071.792830885,210,NULL)
INSERT INTO SalesInvoice VALUES (385,39038.8706974851,39039.899445943,324,NULL)
INSERT INTO SalesInvoice VALUES (386,39039.7740164417,39073.1619962804,548,NULL)
INSERT INTO SalesInvoice VALUES (387,39040.5069626568,39054.4525651035,234,NULL)
INSERT INTO SalesInvoice VALUES (388,39041.7875114793,39058.1860733993,170,NULL)
INSERT INTO SalesInvoice VALUES (389,39042.7729745931,39044.1772760211,328,NULL)
INSERT INTO SalesInvoice VALUES (390,39043.8010751875,39050.5636174494,49,NULL)
INSERT INTO SalesInvoice VALUES (391,39044.7081674532,39072.7206972673,383,NULL)
INSERT INTO SalesInvoice VALUES (392,39044.8552599485,39078.6428549787,670,NULL)
INSERT INTO SalesInvoice VALUES (393,39045.9625817205,39055.0797631078,706,NULL)
INSERT INTO SalesInvoice VALUES (394,39046.7412516796,39059.1886605328,646,NULL)
INSERT INTO SalesInvoice VALUES (395,39047.586046039,39063.0523768845,240,NULL)
INSERT INTO SalesInvoice VALUES (396,39049.0076499772,39049.9849559848,513,NULL)
INSERT INTO SalesInvoice VALUES (397,39049.3946463379,39069.5063677764,317,NULL)
INSERT INTO SalesInvoice VALUES (398,39049.4150923705,39053.9745934227,515,NULL)
INSERT INTO SalesInvoice VALUES (399,39050.9845708051,39080.9203349164,113,NULL)
INSERT INTO SalesInvoice VALUES (400,39052.4993356037,39079.8569721495,356,NULL)
INSERT INTO SalesInvoice VALUES (401,39052.8138074006,39052.8983028233,712,NULL)
INSERT INTO SalesInvoice VALUES (402,39053.1591306901,39086.5058762783,43,NULL)
INSERT INTO SalesInvoice VALUES (403,39054.691737271,39065.4869065445,460,NULL)
INSERT INTO SalesInvoice VALUES (404,39054.8145447985,39068.6981044377,80,NULL)
INSERT INTO SalesInvoice VALUES (405,39055.4200540705,39100.5059875078,715,NULL)
INSERT INTO SalesInvoice VALUES (406,39056.6862518794,39096.0666412954,409,NULL)
INSERT INTO SalesInvoice VALUES (407,39057.7007231389,39089.9547996645,223,NULL)
INSERT INTO SalesInvoice VALUES (408,39058.5374574652,39097.9437806477,670,NULL)
INSERT INTO SalesInvoice VALUES (409,39059.9152659846,39079.7589313639,214,NULL)
INSERT INTO SalesInvoice VALUES (410,39060.9298229486,39088.4658325927,517,NULL)
INSERT INTO SalesInvoice VALUES (411,39061.1860256289,39074.2520929794,748,NULL)
INSERT INTO SalesInvoice VALUES (412,39062.3909988706,39104.1649505375,703,NULL)
INSERT INTO SalesInvoice VALUES (413,39063.7128354757,39066.7036395423,608,NULL)
INSERT INTO SalesInvoice VALUES (414,39064.7303799768,39083.9455060735,659,NULL)
INSERT INTO SalesInvoice VALUES (415,39065.583390414,39076.28995697,355,NULL)
INSERT INTO SalesInvoice VALUES (416,39066.3004939758,39106.3399991098,473,NULL)
INSERT INTO SalesInvoice VALUES (417,39066.7752014667,39069.3396786258,689,NULL)
INSERT INTO SalesInvoice VALUES (418,39067.5612948939,39068.757471071,732,NULL)
INSERT INTO SalesInvoice VALUES (419,39068.2953351568,39102.5087795126,665,NULL)
INSERT INTO SalesInvoice VALUES (420,39069.8202911963,39094.9071490961,89,NULL)
INSERT INTO SalesInvoice VALUES (421,39071.3529546447,39113.0939933625,139,NULL)
INSERT INTO SalesInvoice VALUES (422,39072.7449852343,39091.5435680947,242,NULL)
INSERT INTO SalesInvoice VALUES (423,39072.8282660932,39119.4339660784,155,NULL)
INSERT INTO SalesInvoice VALUES (424,39073.1099174541,39107.651506339,645,NULL)
INSERT INTO SalesInvoice VALUES (425,39073.9271844095,39119.4623028858,9,NULL)
INSERT INTO SalesInvoice VALUES (426,39074.1534431452,39107.0323525815,327,NULL)
INSERT INTO SalesInvoice VALUES (427,39074.9965055164,39093.9623973377,354,NULL)
INSERT INTO SalesInvoice VALUES (428,39075.5294406692,39102.7096595008,415,NULL)
INSERT INTO SalesInvoice VALUES (429,39076.7651127084,39090.9747327728,149,NULL)
INSERT INTO SalesInvoice VALUES (430,39076.9828680296,39080.8822693302,141,NULL)
INSERT INTO SalesInvoice VALUES (431,39077.3858829038,39113.568879503,252,NULL)
INSERT INTO SalesInvoice VALUES (432,39078.5920212691,39100.5979333901,477,NULL)
INSERT INTO SalesInvoice VALUES (433,39079.029613973,39081.0182950921,712,NULL)
INSERT INTO SalesInvoice VALUES (434,39079.2362890124,39092.328965994,153,NULL)
INSERT INTO SalesInvoice VALUES (435,39079.9226478778,39092.8547200663,628,NULL)
INSERT INTO SalesInvoice VALUES (436,39080.1421051113,39093.8592924529,405,NULL)
INSERT INTO SalesInvoice VALUES (437,39080.2340467462,39109.8034102473,425,NULL)
INSERT INTO SalesInvoice VALUES (438,39080.6488215587,39089.6265856348,78,NULL)
INSERT INTO SalesInvoice VALUES (439,39081.0540077001,39112.9787946142,46,NULL)
INSERT INTO SalesInvoice VALUES (440,39081.630098337,39116.2449641359,64,NULL)
INSERT INTO SalesInvoice VALUES (441,39082.3185510486,39127.4664786031,576,NULL)
INSERT INTO SalesInvoice VALUES (442,39083.5141493274,39091.2428322234,542,NULL)
INSERT INTO SalesInvoice VALUES (443,39083.8794921317,39088.6316779276,3,NULL)
INSERT INTO SalesInvoice VALUES (444,39085.3919539337,39126.0149355941,330,NULL)
INSERT INTO SalesInvoice VALUES (445,39085.7520107826,39128.2195615747,588,NULL)
INSERT INTO SalesInvoice VALUES (446,39085.8923743136,39103.9918555106,375,NULL)
INSERT INTO SalesInvoice VALUES (447,39086.6805975421,39106.2142160182,583,NULL)
INSERT INTO SalesInvoice VALUES (448,39088.1776218993,39116.8456016302,547,NULL)
INSERT INTO SalesInvoice VALUES (449,39089.6447032508,39119.8867856021,750,NULL)
INSERT INTO SalesInvoice VALUES (450,39089.7666478384,39112.21258116,720,NULL)
INSERT INTO SalesInvoice VALUES (451,39090.2940968349,39116.1048638892,594,NULL)
INSERT INTO SalesInvoice VALUES (452,39091.4937546959,39102.2264542891,281,NULL)
INSERT INTO SalesInvoice VALUES (453,39092.6229908701,39128.2149169233,516,NULL)
INSERT INTO SalesInvoice VALUES (454,39093.7669225666,39095.8210939603,686,NULL)
INSERT INTO SalesInvoice VALUES (455,39094.0586526605,39103.3476846668,96,NULL)
INSERT INTO SalesInvoice VALUES (456,39094.5480508704,39133.1909542391,602,NULL)
INSERT INTO SalesInvoice VALUES (457,39094.5516425863,39139.520033192,775,NULL)
INSERT INTO SalesInvoice VALUES (458,39096.0141241784,39136.8115998846,486,NULL)
INSERT INTO SalesInvoice VALUES (459,39096.7122725985,39135.4173426342,380,NULL)
INSERT INTO SalesInvoice VALUES (460,39097.2922158817,39112.07636993,306,NULL)
INSERT INTO SalesInvoice VALUES (461,39098.1556533481,39103.6603325258,474,NULL)
INSERT INTO SalesInvoice VALUES (462,39098.7742058858,39143.309708826,484,NULL)
INSERT INTO SalesInvoice VALUES (463,39099.540551811,39124.1311941461,162,NULL)
INSERT INTO SalesInvoice VALUES (464,39100.130906966,39143.8406130596,85,NULL)
INSERT INTO SalesInvoice VALUES (465,39100.6404131878,39125.0983504504,21,NULL)
INSERT INTO SalesInvoice VALUES (466,39100.7409967569,39107.8282870201,767,NULL)
INSERT INTO SalesInvoice VALUES (467,39100.8615116592,39140.0632979588,91,NULL)
INSERT INTO SalesInvoice VALUES (468,39101.350501835,39145.1090642919,643,NULL)
INSERT INTO SalesInvoice VALUES (469,39101.9588898207,39122.4392662246,551,NULL)
INSERT INTO SalesInvoice VALUES (470,39103.3627939378,39126.2061869353,701,NULL)
INSERT INTO SalesInvoice VALUES (471,39104.1212251271,39149.520974306,734,NULL)
INSERT INTO SalesInvoice VALUES (472,39104.6435418462,39119.650021286,684,NULL)
INSERT INTO SalesInvoice VALUES (473,39105.6772733883,39126.0698548602,94,NULL)
INSERT INTO SalesInvoice VALUES (474,39106.6946569454,39109.2881792737,34,NULL)
INSERT INTO SalesInvoice VALUES (475,39107.2722880438,39113.6061819948,59,NULL)
INSERT INTO SalesInvoice VALUES (476,39108.4039612062,39137.2554873474,29,NULL)
INSERT INTO SalesInvoice VALUES (477,39109.7891142664,39144.9454820944,495,NULL)
INSERT INTO SalesInvoice VALUES (478,39111.3082057317,39116.6412235083,233,NULL)
INSERT INTO SalesInvoice VALUES (479,39111.397546336,39148.8717959999,658,NULL)
INSERT INTO SalesInvoice VALUES (480,39111.6202039323,39113.3669235301,702,NULL)
INSERT INTO SalesInvoice VALUES (481,39111.8245791366,39135.0117765642,453,NULL)
INSERT INTO SalesInvoice VALUES (482,39111.9552373013,39125.7641363043,433,NULL)
INSERT INTO SalesInvoice VALUES (483,39113.4158164343,39149.9179137589,251,NULL)
INSERT INTO SalesInvoice VALUES (484,39114.2808356047,39116.6774602169,596,NULL)
INSERT INTO SalesInvoice VALUES (485,39115.2496071643,39123.8216807912,364,NULL)
INSERT INTO SalesInvoice VALUES (486,39116.2856559101,39128.0798698945,13,NULL)
INSERT INTO SalesInvoice VALUES (487,39116.9158028004,39127.2796917647,646,NULL)
INSERT INTO SalesInvoice VALUES (488,39117.3782514467,39133.2409559813,52,NULL)
INSERT INTO SalesInvoice VALUES (489,39117.9983142839,39153.0315243418,484,NULL)
INSERT INTO SalesInvoice VALUES (490,39119.292627775,39142.1132989098,12,NULL)
INSERT INTO SalesInvoice VALUES (491,39119.7820646461,39168.4335767677,263,NULL)
INSERT INTO SalesInvoice VALUES (492,39121.2311874686,39151.3965148547,353,NULL)
INSERT INTO SalesInvoice VALUES (493,39121.5693306276,39133.240288377,445,NULL)
INSERT INTO SalesInvoice VALUES (494,39121.7539435286,39131.466354506,753,NULL)
INSERT INTO SalesInvoice VALUES (495,39123.2439542974,39154.1040242237,11,NULL)
INSERT INTO SalesInvoice VALUES (496,39123.6236066869,39144.0770736584,361,NULL)
INSERT INTO SalesInvoice VALUES (497,39124.8385921702,39140.6611285414,394,NULL)
INSERT INTO SalesInvoice VALUES (498,39125.3848643569,39174.7697630252,79,NULL)
INSERT INTO SalesInvoice VALUES (499,39125.7764715655,39155.1539368392,62,NULL)
INSERT INTO SalesInvoice VALUES (500,39126.0620451334,39128.0773872082,103,NULL)
INSERT INTO SalesInvoice VALUES (501,39127.4016793736,39128.074121369,386,NULL)
INSERT INTO SalesInvoice VALUES (502,39128.9283212697,39148.8531336577,81,NULL)
INSERT INTO SalesInvoice VALUES (503,39129.0822753136,39177.3521677241,275,NULL)
INSERT INTO SalesInvoice VALUES (504,39130.4067354669,39164.1165638371,688,NULL)
INSERT INTO SalesInvoice VALUES (505,39131.8888014437,39136.7847119933,240,NULL)
INSERT INTO SalesInvoice VALUES (506,39132.3009568227,39158.4611822219,487,NULL)
INSERT INTO SalesInvoice VALUES (507,39133.6498393618,39179.1380235308,417,NULL)
INSERT INTO SalesInvoice VALUES (508,39135.188227747,39165.4398795184,7,NULL)
INSERT INTO SalesInvoice VALUES (509,39135.4571028284,39159.7814523378,405,NULL)
INSERT INTO SalesInvoice VALUES (510,39136.0142109291,39167.6541137358,34,NULL)
INSERT INTO SalesInvoice VALUES (511,39137.266638776,39139.3624730349,135,NULL)
INSERT INTO SalesInvoice VALUES (512,39138.282325986,39172.3621083935,212,NULL)
INSERT INTO SalesInvoice VALUES (513,39138.9367562289,39178.0617200502,240,NULL)
INSERT INTO SalesInvoice VALUES (514,39140.0858359517,39179.0760984088,704,NULL)
INSERT INTO SalesInvoice VALUES (515,39141.3644656486,39186.169523742,263,NULL)
INSERT INTO SalesInvoice VALUES (516,39141.4068161437,39189.3727125634,501,NULL)
INSERT INTO SalesInvoice VALUES (517,39141.5152469493,39163.6000175308,20,NULL)
INSERT INTO SalesInvoice VALUES (518,39141.6258707462,39145.5791275134,148,NULL)
INSERT INTO SalesInvoice VALUES (519,39142.03069566,39183.0398346172,642,NULL)
INSERT INTO SalesInvoice VALUES (520,39142.4213293651,39163.143744201,499,NULL)
INSERT INTO SalesInvoice VALUES (521,39142.4954531679,39151.7860267876,583,NULL)
INSERT INTO SalesInvoice VALUES (522,39142.6586795781,39168.2542058418,576,NULL)
INSERT INTO SalesInvoice VALUES (523,39142.7528206558,39184.8851429825,472,NULL)
INSERT INTO SalesInvoice VALUES (524,39143.7269823886,39169.8542996148,123,NULL)
INSERT INTO SalesInvoice VALUES (525,39143.9605425199,39178.009506353,677,NULL)
INSERT INTO SalesInvoice VALUES (526,39145.2487306284,39181.5263745253,250,NULL)
INSERT INTO SalesInvoice VALUES (527,39146.2950814656,39161.6516498233,639,NULL)
INSERT INTO SalesInvoice VALUES (528,39146.8630391522,39162.505672604,334,NULL)
INSERT INTO SalesInvoice VALUES (529,39147.9939273202,39150.2529262653,685,NULL)
INSERT INTO SalesInvoice VALUES (530,39148.8706058137,39192.9722162479,142,NULL)
INSERT INTO SalesInvoice VALUES (531,39148.9491270995,39175.9510365113,104,NULL)
INSERT INTO SalesInvoice VALUES (532,39149.6317327435,39177.3801008864,123,NULL)
INSERT INTO SalesInvoice VALUES (533,39150.6950566343,39178.2027524063,308,NULL)
INSERT INTO SalesInvoice VALUES (534,39152.104358147,39183.1796713422,360,NULL)
INSERT INTO SalesInvoice VALUES (535,39152.4498564016,39196.4920668979,155,NULL)
INSERT INTO SalesInvoice VALUES (536,39153.4369954446,39187.1452983616,424,NULL)
INSERT INTO SalesInvoice VALUES (537,39153.5232770725,39201.9708523934,245,NULL)
INSERT INTO SalesInvoice VALUES (538,39153.8089140994,39176.6147063703,505,NULL)
INSERT INTO SalesInvoice VALUES (539,39154.4938425061,39161.8422270945,406,NULL)
INSERT INTO SalesInvoice VALUES (540,39155.9468463412,39204.4708995185,509,NULL)
INSERT INTO SalesInvoice VALUES (541,39157.2986409578,39172.8147347212,492,NULL)
INSERT INTO SalesInvoice VALUES (542,39157.4695541899,39204.4663261081,708,NULL)
INSERT INTO SalesInvoice VALUES (543,39158.3433894215,39204.6262781232,268,NULL)
INSERT INTO SalesInvoice VALUES (544,39159.7598237751,39187.2428184663,534,NULL)
INSERT INTO SalesInvoice VALUES (545,39160.3171777851,39182.3825165821,503,NULL)
INSERT INTO SalesInvoice VALUES (546,39160.5087305031,39196.5838801678,474,NULL)
INSERT INTO SalesInvoice VALUES (547,39160.8908651827,39197.2855547745,526,NULL)
INSERT INTO SalesInvoice VALUES (548,39162.4729725798,39197.26717333,264,NULL)
INSERT INTO SalesInvoice VALUES (549,39163.9511040105,39178.9864235405,145,NULL)
INSERT INTO SalesInvoice VALUES (550,39165.0975583096,39166.0058418872,282,NULL)
INSERT INTO SalesInvoice VALUES (551,39165.9108049039,39212.683705323,753,NULL)
INSERT INTO SalesInvoice VALUES (552,39166.2380615299,39189.7595019121,310,NULL)
INSERT INTO SalesInvoice VALUES (553,39167.7654119495,39179.8192892564,364,NULL)
INSERT INTO SalesInvoice VALUES (554,39168.6975038235,39217.6560483427,344,NULL)
INSERT INTO SalesInvoice VALUES (555,39170.0113256817,39179.6772060233,202,NULL)
INSERT INTO SalesInvoice VALUES (556,39170.9198757507,39206.1987324567,642,NULL)
INSERT INTO SalesInvoice VALUES (557,39171.0152385088,39191.1050064763,73,NULL)
INSERT INTO SalesInvoice VALUES (558,39171.3624394375,39200.7671010955,521,NULL)
INSERT INTO SalesInvoice VALUES (559,39171.4873556708,39198.853740874,167,NULL)
INSERT INTO SalesInvoice VALUES (560,39172.376972851,39177.0033932315,75,NULL)
INSERT INTO SalesInvoice VALUES (561,39172.9907950209,39207.0856211145,412,NULL)
INSERT INTO SalesInvoice VALUES (562,39173.5398055663,39219.4639806812,762,NULL)
INSERT INTO SalesInvoice VALUES (563,39174.9178706109,39213.1678531528,211,NULL)
INSERT INTO SalesInvoice VALUES (564,39176.0324390134,39202.564676468,493,NULL)
INSERT INTO SalesInvoice VALUES (565,39176.1222640003,39184.0847463268,208,NULL)
INSERT INTO SalesInvoice VALUES (566,39176.837837481,39184.1718244712,140,NULL)
INSERT INTO SalesInvoice VALUES (567,39178.3780747099,39218.6469898924,333,NULL)
INSERT INTO SalesInvoice VALUES (568,39179.050165423,39218.0090302495,550,NULL)
INSERT INTO SalesInvoice VALUES (569,39180.4836958017,39227.5736787239,591,NULL)
INSERT INTO SalesInvoice VALUES (570,39181.8529133116,39195.4131569001,149,NULL)
INSERT INTO SalesInvoice VALUES (571,39183.3072881136,39189.2151848923,450,NULL)
INSERT INTO SalesInvoice VALUES (572,39184.0360833564,39206.8318620499,148,NULL)
INSERT INTO SalesInvoice VALUES (573,39185.2927469233,39213.0390988573,183,NULL)
INSERT INTO SalesInvoice VALUES (574,39185.6211277902,39196.9113699991,185,NULL)
INSERT INTO SalesInvoice VALUES (575,39186.5761653778,39223.1136960806,468,NULL)
INSERT INTO SalesInvoice VALUES (576,39186.9452906777,39188.1911241639,574,NULL)
INSERT INTO SalesInvoice VALUES (577,39187.7142663869,39212.7496975871,205,NULL)
INSERT INTO SalesInvoice VALUES (578,39188.9459757212,39201.8641419334,478,NULL)
INSERT INTO SalesInvoice VALUES (579,39189.2105102036,39236.6195885029,80,NULL)
INSERT INTO SalesInvoice VALUES (580,39190.0289337494,39204.3870059224,496,NULL)
INSERT INTO SalesInvoice VALUES (581,39190.2996031564,39212.9009543247,765,NULL)
INSERT INTO SalesInvoice VALUES (582,39191.0652857383,39233.9060604348,463,NULL)
INSERT INTO SalesInvoice VALUES (583,39191.6335495301,39199.3088967319,132,NULL)
INSERT INTO SalesInvoice VALUES (584,39192.4369301831,39229.0729743229,559,NULL)
INSERT INTO SalesInvoice VALUES (585,39193.6919331977,39220.5949508619,747,NULL)
INSERT INTO SalesInvoice VALUES (586,39195.1114951323,39220.9820518281,574,NULL)
INSERT INTO SalesInvoice VALUES (587,39196.2094685178,39232.8547983043,585,NULL)
INSERT INTO SalesInvoice VALUES (588,39196.5833387601,39225.0786254177,754,NULL)
INSERT INTO SalesInvoice VALUES (589,39196.6537259907,39224.4485148344,550,NULL)
INSERT INTO SalesInvoice VALUES (590,39197.0042244799,39234.1784260752,663,NULL)
INSERT INTO SalesInvoice VALUES (591,39198.2740519138,39219.3922675133,405,NULL)
INSERT INTO SalesInvoice VALUES (592,39199.4222823366,39226.481751775,384,NULL)
INSERT INTO SalesInvoice VALUES (593,39200.6578270113,39216.4911055711,608,NULL)
INSERT INTO SalesInvoice VALUES (594,39201.9613087077,39226.5106880361,67,NULL)
INSERT INTO SalesInvoice VALUES (595,39202.643452162,39246.5612713268,624,NULL)
INSERT INTO SalesInvoice VALUES (596,39204.0661412807,39209.069924409,212,NULL)
INSERT INTO SalesInvoice VALUES (597,39204.9205048364,39250.0388455854,209,NULL)
INSERT INTO SalesInvoice VALUES (598,39205.6046960677,39223.8764486926,279,NULL)
INSERT INTO SalesInvoice VALUES (599,39207.020147575,39229.3261398477,58,NULL)
INSERT INTO SalesInvoice VALUES (600,39207.8345438075,39240.6500443336,136,NULL)
INSERT INTO SalesInvoice VALUES (601,39208.6589719465,39221.5124763632,701,NULL)
INSERT INTO SalesInvoice VALUES (602,39210.0521918964,39219.3642345136,221,NULL)
INSERT INTO SalesInvoice VALUES (603,39211.1339593353,39252.9562282278,22,NULL)
INSERT INTO SalesInvoice VALUES (604,39212.2597362263,39235.9532984787,730,NULL)
INSERT INTO SalesInvoice VALUES (605,39212.9849269816,39236.9535013701,469,NULL)
INSERT INTO SalesInvoice VALUES (606,39213.6399450551,39215.2990299626,164,NULL)
INSERT INTO SalesInvoice VALUES (607,39214.4750024081,39246.4654723929,563,NULL)
INSERT INTO SalesInvoice VALUES (608,39215.3974988313,39243.9834891825,17,NULL)
INSERT INTO SalesInvoice VALUES (609,39216.5553158033,39262.9987241557,83,NULL)
INSERT INTO SalesInvoice VALUES (610,39217.9432557617,39262.690423381,463,NULL)
INSERT INTO SalesInvoice VALUES (611,39217.9731983199,39257.5218977855,385,NULL)
INSERT INTO SalesInvoice VALUES (612,39218.4568246908,39234.7436496946,456,NULL)
INSERT INTO SalesInvoice VALUES (613,39220.0221013358,39233.5011264569,408,NULL)
INSERT INTO SalesInvoice VALUES (614,39220.4582001608,39225.0130798101,41,NULL)
INSERT INTO SalesInvoice VALUES (615,39221.7114061312,39258.3737598156,760,NULL)
INSERT INTO SalesInvoice VALUES (616,39222.5262053491,39244.7783729179,717,NULL)
INSERT INTO SalesInvoice VALUES (617,39223.9763013889,39246.2773978328,54,NULL)
INSERT INTO SalesInvoice VALUES (618,39224.7271608349,39274.6069203837,750,NULL)
INSERT INTO SalesInvoice VALUES (619,39226.0411915295,39253.0134477083,684,NULL)
INSERT INTO SalesInvoice VALUES (620,39227.6213757758,39274.8159017891,30,NULL)
INSERT INTO SalesInvoice VALUES (621,39227.8713925135,39261.3576453065,105,NULL)
INSERT INTO SalesInvoice VALUES (622,39229.0974395978,39268.5484370221,594,NULL)
INSERT INTO SalesInvoice VALUES (623,39230.3034251244,39274.0819688043,463,NULL)
INSERT INTO SalesInvoice VALUES (624,39230.6244643117,39254.076251716,319,NULL)
INSERT INTO SalesInvoice VALUES (625,39230.9983624121,39235.3343042262,185,NULL)
INSERT INTO SalesInvoice VALUES (626,39232.1434989635,39237.0187618394,217,NULL)
INSERT INTO SalesInvoice VALUES (627,39233.4427319755,39246.4912519843,66,NULL)
INSERT INTO SalesInvoice VALUES (628,39233.6510703789,39245.3641340059,21,NULL)
INSERT INTO SalesInvoice VALUES (629,39234.105831463,39261.1620391605,447,NULL)
INSERT INTO SalesInvoice VALUES (630,39234.8932569378,39263.3720210891,621,NULL)
INSERT INTO SalesInvoice VALUES (631,39235.1357306022,39268.9552384823,68,NULL)
INSERT INTO SalesInvoice VALUES (632,39236.674064538,39239.8450153808,739,NULL)
INSERT INTO SalesInvoice VALUES (633,39237.8178788938,39243.490853437,493,NULL)
INSERT INTO SalesInvoice VALUES (634,39239.1699508816,39258.1635977644,496,NULL)
INSERT INTO SalesInvoice VALUES (635,39240.3204767098,39278.1147749451,422,NULL)
INSERT INTO SalesInvoice VALUES (636,39241.4485788206,39288.289258553,113,NULL)
INSERT INTO SalesInvoice VALUES (637,39242.7482905127,39249.1038568525,44,NULL)
INSERT INTO SalesInvoice VALUES (638,39243.3430623996,39243.6119189639,21,NULL)
INSERT INTO SalesInvoice VALUES (639,39243.6443707938,39282.1583526454,388,NULL)
INSERT INTO SalesInvoice VALUES (640,39244.923424489,39245.4918130646,774,NULL)
INSERT INTO SalesInvoice VALUES (641,39245.4683762719,39292.4101168527,298,NULL)
INSERT INTO SalesInvoice VALUES (642,39245.7168246449,39280.5939492774,576,NULL)
INSERT INTO SalesInvoice VALUES (643,39246.551201555,39256.4272952859,13,NULL)
INSERT INTO SalesInvoice VALUES (644,39247.6367881828,39273.5927929236,156,NULL)
INSERT INTO SalesInvoice VALUES (645,39248.4979759796,39259.9428771627,201,NULL)
INSERT INTO SalesInvoice VALUES (646,39249.9927568755,39266.6695212452,580,NULL)
INSERT INTO SalesInvoice VALUES (647,39251.2959699024,39268.5182238959,770,NULL)
INSERT INTO SalesInvoice VALUES (648,39251.7602344767,39277.6053236161,502,NULL)
INSERT INTO SalesInvoice VALUES (649,39252.6094325247,39296.9566132419,524,NULL)
INSERT INTO SalesInvoice VALUES (650,39254.0137145395,39297.7162544108,651,NULL)
INSERT INTO SalesInvoice VALUES (651,39254.0593345443,39254.9325318524,592,NULL)
INSERT INTO SalesInvoice VALUES (652,39254.27565133,39285.1442821389,71,NULL)
INSERT INTO SalesInvoice VALUES (653,39255.2358098141,39256.7556435447,268,NULL)
INSERT INTO SalesInvoice VALUES (654,39256.328153695,39297.0582568183,37,NULL)
INSERT INTO SalesInvoice VALUES (655,39257.1611172433,39289.3645865411,748,NULL)
INSERT INTO SalesInvoice VALUES (656,39258.1939417988,39263.4670689422,585,NULL)
INSERT INTO SalesInvoice VALUES (657,39258.9420578254,39281.6181774369,311,NULL)
INSERT INTO SalesInvoice VALUES (658,39259.0699221558,39308.8005866579,337,NULL)
INSERT INTO SalesInvoice VALUES (659,39260.4714472879,39300.9669288328,485,NULL)
INSERT INTO SalesInvoice VALUES (660,39260.6713838591,39292.6710028324,299,NULL)
INSERT INTO SalesInvoice VALUES (661,39261.8152189329,39267.8537627803,312,NULL)
INSERT INTO SalesInvoice VALUES (662,39262.5906837602,39288.4583102604,289,NULL)
INSERT INTO SalesInvoice VALUES (663,39263.9615614068,39284.8922353039,446,NULL)
INSERT INTO SalesInvoice VALUES (664,39264.3505263547,39303.1824527056,607,NULL)
INSERT INTO SalesInvoice VALUES (665,39264.6969570574,39271.0329868513,443,NULL)
INSERT INTO SalesInvoice VALUES (666,39265.2845204613,39296.3456725901,244,NULL)
INSERT INTO SalesInvoice VALUES (667,39265.6159457961,39284.6300085552,252,NULL)
INSERT INTO SalesInvoice VALUES (668,39267.014045132,39283.2634799483,261,NULL)
INSERT INTO SalesInvoice VALUES (669,39268.4121111372,39294.3441815475,480,NULL)
INSERT INTO SalesInvoice VALUES (670,39269.3812579009,39301.7126547344,245,NULL)
INSERT INTO SalesInvoice VALUES (671,39270.048998091,39315.8844652547,168,NULL)
INSERT INTO SalesInvoice VALUES (672,39270.7946696586,39294.1831122613,476,NULL)
INSERT INTO SalesInvoice VALUES (673,39272.2023623238,39308.2722828266,441,NULL)
INSERT INTO SalesInvoice VALUES (674,39273.5784520813,39286.6646377978,294,NULL)
INSERT INTO SalesInvoice VALUES (675,39273.7004823597,39310.3606975733,531,NULL)
INSERT INTO SalesInvoice VALUES (676,39273.7757866465,39322.8996131168,226,NULL)
INSERT INTO SalesInvoice VALUES (677,39274.665435869,39323.3250162699,538,NULL)
INSERT INTO SalesInvoice VALUES (678,39275.6337538069,39289.1685262238,107,NULL)
INSERT INTO SalesInvoice VALUES (679,39276.5902944724,39304.5629415099,88,NULL)
INSERT INTO SalesInvoice VALUES (680,39277.915037382,39297.4376255197,148,NULL)
INSERT INTO SalesInvoice VALUES (681,39278.2967835651,39284.4663279557,68,NULL)
INSERT INTO SalesInvoice VALUES (682,39278.9827386976,39281.5562215444,543,NULL)
INSERT INTO SalesInvoice VALUES (683,39280.055176629,39291.0055457991,86,NULL)
INSERT INTO SalesInvoice VALUES (684,39280.2919380548,39300.5824973151,497,NULL)
INSERT INTO SalesInvoice VALUES (685,39281.7559409841,39326.9113963005,331,NULL)
INSERT INTO SalesInvoice VALUES (686,39282.0755306725,39288.5596235427,217,NULL)
INSERT INTO SalesInvoice VALUES (687,39283.3777017503,39323.6171006252,251,NULL)
INSERT INTO SalesInvoice VALUES (688,39284.8354063402,39312.3937764428,773,NULL)
INSERT INTO SalesInvoice VALUES (689,39285.7808436252,39316.1052315842,155,NULL)
INSERT INTO SalesInvoice VALUES (690,39286.5837904046,39308.7251680898,571,NULL)
INSERT INTO SalesInvoice VALUES (691,39286.6157067093,39294.9675547171,506,NULL)
INSERT INTO SalesInvoice VALUES (692,39286.7921731136,39297.2106960132,358,NULL)
INSERT INTO SalesInvoice VALUES (693,39286.9228346427,39299.4823686581,204,NULL)
INSERT INTO SalesInvoice VALUES (694,39287.1408473197,39308.5286627906,236,NULL)
INSERT INTO SalesInvoice VALUES (695,39287.3043298151,39312.4392590959,143,NULL)
INSERT INTO SalesInvoice VALUES (696,39287.3661000531,39303.4089346031,647,NULL)
INSERT INTO SalesInvoice VALUES (697,39288.6293135485,39300.7687282665,461,NULL)
INSERT INTO SalesInvoice VALUES (698,39290.0847014055,39326.5023486817,138,NULL)
INSERT INTO SalesInvoice VALUES (699,39290.3722956989,39296.4501013844,600,NULL)
INSERT INTO SalesInvoice VALUES (700,39290.4658833396,39318.7586868743,125,NULL)
INSERT INTO SalesInvoice VALUES (701,39291.562457661,39337.0600132996,57,NULL)
INSERT INTO SalesInvoice VALUES (702,39292.5672632573,39336.9134333502,684,NULL)
INSERT INTO SalesInvoice VALUES (703,39293.6601076877,39329.5834404466,94,NULL)
INSERT INTO SalesInvoice VALUES (704,39294.9827343116,39333.0564056465,174,NULL)
INSERT INTO SalesInvoice VALUES (705,39295.9015496678,39335.9424570012,25,NULL)
INSERT INTO SalesInvoice VALUES (706,39296.6879299338,39345.1627988039,761,NULL)
INSERT INTO SalesInvoice VALUES (707,39296.8728858069,39322.2150883254,126,NULL)
INSERT INTO SalesInvoice VALUES (708,39297.1580642539,39303.4366897978,408,NULL)
INSERT INTO SalesInvoice VALUES (709,39297.3904739217,39318.7915309932,663,NULL)
INSERT INTO SalesInvoice VALUES (710,39298.5596729801,39336.934416052,648,NULL)
INSERT INTO SalesInvoice VALUES (711,39299.4172492971,39324.0688933638,150,NULL)
INSERT INTO SalesInvoice VALUES (712,39300.9340436696,39335.0000443849,536,NULL)
INSERT INTO SalesInvoice VALUES (713,39302.2601652376,39345.7485367898,184,NULL)
INSERT INTO SalesInvoice VALUES (714,39303.7702838575,39344.0858665641,425,NULL)
INSERT INTO SalesInvoice VALUES (715,39304.9218365675,39353.1029084846,540,NULL)
INSERT INTO SalesInvoice VALUES (716,39305.3822109065,39340.4237469261,727,NULL)
INSERT INTO SalesInvoice VALUES (717,39306.7424950989,39337.2017206499,37,NULL)
INSERT INTO SalesInvoice VALUES (718,39307.0431593591,39339.6151564931,685,NULL)
INSERT INTO SalesInvoice VALUES (719,39307.9184296913,39345.5514304882,505,NULL)
INSERT INTO SalesInvoice VALUES (720,39309.2669766023,39331.4806162931,361,NULL)
INSERT INTO SalesInvoice VALUES (721,39310.6960293622,39327.3069754357,125,NULL)
INSERT INTO SalesInvoice VALUES (722,39311.7240611273,39345.3118516601,11,NULL)
INSERT INTO SalesInvoice VALUES (723,39313.2332423747,39344.6510856269,535,NULL)
INSERT INTO SalesInvoice VALUES (724,39313.4949445098,39357.3714546993,601,NULL)
INSERT INTO SalesInvoice VALUES (725,39314.237168259,39338.3126500348,412,NULL)
INSERT INTO SalesInvoice VALUES (726,39314.5189346138,39353.0222891244,426,NULL)
INSERT INTO SalesInvoice VALUES (727,39314.9518831573,39330.961157881,569,NULL)
INSERT INTO SalesInvoice VALUES (728,39316.3783582338,39329.2000341478,344,NULL)
INSERT INTO SalesInvoice VALUES (729,39316.3820158286,39343.8499275983,188,NULL)
INSERT INTO SalesInvoice VALUES (730,39317.31852464,39358.2535074746,315,NULL)
INSERT INTO SalesInvoice VALUES (731,39318.5102625309,39332.01917417,339,NULL)
INSERT INTO SalesInvoice VALUES (732,39319.0309784519,39343.6282664195,566,NULL)
INSERT INTO SalesInvoice VALUES (733,39320.2548664988,39353.3922920293,480,NULL)
INSERT INTO SalesInvoice VALUES (734,39321.5234305571,39346.4027232398,372,NULL)
INSERT INTO SalesInvoice VALUES (735,39322.2433620723,39369.2413259587,12,NULL)
INSERT INTO SalesInvoice VALUES (736,39323.0678111255,39350.0211451487,184,NULL)
INSERT INTO SalesInvoice VALUES (737,39323.3184992519,39372.3266208901,72,NULL)
INSERT INTO SalesInvoice VALUES (738,39323.8732636587,39330.2433113512,638,NULL)
INSERT INTO SalesInvoice VALUES (739,39324.6380723252,39341.8890949334,738,NULL)
INSERT INTO SalesInvoice VALUES (740,39325.4447873842,39374.1463897425,428,NULL)
INSERT INTO SalesInvoice VALUES (741,39326.1057195636,39359.3456210591,406,NULL)
INSERT INTO SalesInvoice VALUES (742,39327.2211608801,39364.0991582344,104,NULL)
INSERT INTO SalesInvoice VALUES (743,39327.5769041902,39339.5270500938,402,NULL)
INSERT INTO SalesInvoice VALUES (744,39328.9259251612,39369.4254879566,335,NULL)
INSERT INTO SalesInvoice VALUES (745,39330.3342076177,39352.3000611383,252,NULL)
INSERT INTO SalesInvoice VALUES (746,39331.1701894661,39333.7180767101,736,NULL)
INSERT INTO SalesInvoice VALUES (747,39332.1213986203,39379.0864377964,191,NULL)
INSERT INTO SalesInvoice VALUES (748,39333.2536205224,39379.82090566,713,NULL)
INSERT INTO SalesInvoice VALUES (749,39334.4460400752,39344.241790469,665,NULL)
INSERT INTO SalesInvoice VALUES (750,39335.937803934,39374.2053553192,636,NULL)
INSERT INTO SalesInvoice VALUES (751,39335.99633517,39344.6342800059,101,NULL)
INSERT INTO SalesInvoice VALUES (752,39337.4585982601,39375.3348442985,480,NULL)
INSERT INTO SalesInvoice VALUES (753,39338.0411554657,39380.2128856938,461,NULL)
INSERT INTO SalesInvoice VALUES (754,39338.1208099814,39360.8728445876,749,NULL)
INSERT INTO SalesInvoice VALUES (755,39338.3798299745,39349.8786244683,321,NULL)
INSERT INTO SalesInvoice VALUES (756,39338.5981831618,39382.6509698284,198,NULL)
INSERT INTO SalesInvoice VALUES (757,39339.7229062316,39383.5156053712,376,NULL)
INSERT INTO SalesInvoice VALUES (758,39340.1219445528,39342.4904134439,560,NULL)
INSERT INTO SalesInvoice VALUES (759,39340.4687592433,39367.1029702351,40,NULL)
INSERT INTO SalesInvoice VALUES (760,39341.0440898162,39384.0395912115,414,NULL)
INSERT INTO SalesInvoice VALUES (761,39341.642010133,39345.2456810696,773,NULL)
INSERT INTO SalesInvoice VALUES (762,39342.0585221835,39349.1548281183,650,NULL)
INSERT INTO SalesInvoice VALUES (763,39343.4766481643,39363.8493014186,708,NULL)
INSERT INTO SalesInvoice VALUES (764,39343.778756534,39384.645277567,588,NULL)
INSERT INTO SalesInvoice VALUES (765,39345.0796226136,39370.1534385901,190,NULL)
INSERT INTO SalesInvoice VALUES (766,39345.7536526776,39383.7327373564,659,NULL)
INSERT INTO SalesInvoice VALUES (767,39346.0895734253,39350.4945840396,41,NULL)
INSERT INTO SalesInvoice VALUES (768,39346.2831449348,39362.09363266,342,NULL)
INSERT INTO SalesInvoice VALUES (769,39347.5896740366,39361.9292819951,100,NULL)
INSERT INTO SalesInvoice VALUES (770,39348.5010905272,39363.027977384,264,NULL)
INSERT INTO SalesInvoice VALUES (771,39349.2479873855,39377.0550375921,377,NULL)
INSERT INTO SalesInvoice VALUES (772,39350.6873586073,39383.5333028069,191,NULL)
INSERT INTO SalesInvoice VALUES (773,39351.2877295592,39363.4154389196,402,NULL)
INSERT INTO SalesInvoice VALUES (774,39352.4222720099,39387.5104131707,770,NULL)
INSERT INTO SalesInvoice VALUES (775,39353.2652547721,39400.9215253963,58,NULL)
INSERT INTO SalesInvoice VALUES (776,39353.6672007147,39361.6261081228,300,NULL)
INSERT INTO SalesInvoice VALUES (777,39354.7382924203,39400.2015020944,680,NULL)
INSERT INTO SalesInvoice VALUES (778,39355.5784871589,39403.3460009672,12,NULL)
INSERT INTO SalesInvoice VALUES (779,39355.7763774976,39364.0932829054,188,NULL)
INSERT INTO SalesInvoice VALUES (780,39357.1780597917,39370.0497349834,295,NULL)
INSERT INTO SalesInvoice VALUES (781,39357.4836411197,39385.5835073806,32,NULL)
INSERT INTO SalesInvoice VALUES (782,39358.3868855091,39382.6883125875,497,NULL)
INSERT INTO SalesInvoice VALUES (783,39359.7862533407,39398.9077150293,463,NULL)
INSERT INTO SalesInvoice VALUES (784,39361.021988525,39385.2381896623,154,NULL)
INSERT INTO SalesInvoice VALUES (785,39362.4276005351,39399.0071799493,29,NULL)
INSERT INTO SalesInvoice VALUES (786,39363.8730497728,39403.3966997437,534,NULL)
INSERT INTO SalesInvoice VALUES (787,39365.0701841215,39410.513872896,345,NULL)
INSERT INTO SalesInvoice VALUES (788,39365.1123425879,39406.8871143565,411,NULL)
INSERT INTO SalesInvoice VALUES (789,39366.5564286545,39389.8082800643,583,NULL)
INSERT INTO SalesInvoice VALUES (790,39367.953289012,39392.7605357946,677,NULL)
INSERT INTO SalesInvoice VALUES (791,39368.6531216069,39400.3780653728,627,NULL)
INSERT INTO SalesInvoice VALUES (792,39369.6557235519,39404.99584653,471,NULL)
INSERT INTO SalesInvoice VALUES (793,39370.701350965,39379.94632867,536,NULL)
INSERT INTO SalesInvoice VALUES (794,39371.7323813492,39386.8232588774,556,NULL)
INSERT INTO SalesInvoice VALUES (795,39372.2680154249,39406.0660990329,682,NULL)
INSERT INTO SalesInvoice VALUES (796,39373.0601395359,39413.7917600975,538,NULL)
INSERT INTO SalesInvoice VALUES (797,39373.6005117069,39403.5778794948,722,NULL)
INSERT INTO SalesInvoice VALUES (798,39373.6902796093,39402.2721662296,471,NULL)
INSERT INTO SalesInvoice VALUES (799,39373.8259204034,39413.618976786,122,NULL)
INSERT INTO SalesInvoice VALUES (800,39374.7481572316,39412.3217373258,577,NULL)
INSERT INTO SalesInvoice VALUES (801,39376.0646562005,39376.1127168054,272,NULL)
INSERT INTO SalesInvoice VALUES (802,39376.573433313,39419.3850299523,104,NULL)
INSERT INTO SalesInvoice VALUES (803,39377.5133705577,39400.5899170501,306,NULL)
INSERT INTO SalesInvoice VALUES (804,39377.6480673381,39387.3411635567,436,NULL)
INSERT INTO SalesInvoice VALUES (805,39378.5645367816,39424.266651506,458,NULL)
INSERT INTO SalesInvoice VALUES (806,39379.6510038581,39400.3063182333,751,NULL)
INSERT INTO SalesInvoice VALUES (807,39380.5696056049,39422.3891355479,65,NULL)
INSERT INTO SalesInvoice VALUES (808,39380.6404462534,39399.8897538622,162,NULL)
INSERT INTO SalesInvoice VALUES (809,39382.1702548636,39392.3214688574,674,NULL)
INSERT INTO SalesInvoice VALUES (810,39383.1058918711,39404.605002747,337,NULL)
INSERT INTO SalesInvoice VALUES (811,39383.8701212646,39423.0398848128,4,NULL)
INSERT INTO SalesInvoice VALUES (812,39384.2441839133,39392.5819559663,753,NULL)
INSERT INTO SalesInvoice VALUES (813,39384.935170812,39395.733158019,434,NULL)
INSERT INTO SalesInvoice VALUES (814,39385.3242328414,39388.4217956325,20,NULL)
INSERT INTO SalesInvoice VALUES (815,39386.3609382681,39413.2243869912,623,NULL)
INSERT INTO SalesInvoice VALUES (816,39387.1296533268,39406.7506198154,271,NULL)
INSERT INTO SalesInvoice VALUES (817,39388.5166289932,39435.4935331185,526,NULL)
INSERT INTO SalesInvoice VALUES (818,39389.2998113321,39434.9393694792,454,NULL)
INSERT INTO SalesInvoice VALUES (819,39389.7028550063,39410.055862592,772,NULL)
INSERT INTO SalesInvoice VALUES (820,39389.9400892515,39428.1507323141,448,NULL)
INSERT INTO SalesInvoice VALUES (821,39390.6813466273,39409.1298781539,658,NULL)
INSERT INTO SalesInvoice VALUES (822,39391.8072231837,39401.8297420126,39,NULL)
INSERT INTO SalesInvoice VALUES (823,39392.0882215871,39428.6661078429,74,NULL)
INSERT INTO SalesInvoice VALUES (824,39393.4748963616,39420.451684534,1,NULL)
INSERT INTO SalesInvoice VALUES (825,39393.8152120374,39440.7932784019,584,NULL)
INSERT INTO SalesInvoice VALUES (826,39394.1593753381,39403.9446600013,771,NULL)
INSERT INTO SalesInvoice VALUES (827,39394.2905030368,39434.8316128867,379,NULL)
INSERT INTO SalesInvoice VALUES (828,39395.6851534022,39428.2533788203,3,NULL)
INSERT INTO SalesInvoice VALUES (829,39396.0040982272,39428.0644455396,498,NULL)
INSERT INTO SalesInvoice VALUES (830,39397.2924273535,39433.9029705226,696,NULL)
INSERT INTO SalesInvoice VALUES (831,39397.3046852843,39445.4914190834,107,NULL)
INSERT INTO SalesInvoice VALUES (832,39398.0358223833,39417.052375862,597,NULL)
INSERT INTO SalesInvoice VALUES (833,39399.3130725763,39404.1142319185,641,NULL)
INSERT INTO SalesInvoice VALUES (834,39399.8917159965,39400.7201159623,547,NULL)
INSERT INTO SalesInvoice VALUES (835,39401.0227796904,39419.8456081681,218,NULL)
INSERT INTO SalesInvoice VALUES (836,39401.2128493111,39444.9561277533,424,NULL)
INSERT INTO SalesInvoice VALUES (837,39402.4875983638,39418.8746620165,697,NULL)
INSERT INTO SalesInvoice VALUES (838,39403.2449478057,39404.6833864228,123,NULL)
INSERT INTO SalesInvoice VALUES (839,39403.745302324,39430.4249504741,203,NULL)
INSERT INTO SalesInvoice VALUES (840,39405.103476526,39433.7092398156,122,NULL)
INSERT INTO SalesInvoice VALUES (841,39405.6215298824,39449.899592895,667,NULL)
INSERT INTO SalesInvoice VALUES (842,39407.1019856379,39450.4679709336,632,NULL)
INSERT INTO SalesInvoice VALUES (843,39407.1619172729,39418.6085035313,96,NULL)
INSERT INTO SalesInvoice VALUES (844,39407.3610243175,39413.4572386909,243,NULL)
INSERT INTO SalesInvoice VALUES (845,39408.8598049947,39412.5346927019,585,NULL)
INSERT INTO SalesInvoice VALUES (846,39410.4051870871,39430.4166173784,755,NULL)
INSERT INTO SalesInvoice VALUES (847,39411.0319761501,39427.2996254512,601,NULL)
INSERT INTO SalesInvoice VALUES (848,39411.7656784186,39428.5111412131,45,NULL)
INSERT INTO SalesInvoice VALUES (849,39412.0324622457,39443.5105390562,504,NULL)
INSERT INTO SalesInvoice VALUES (850,39412.1673763317,39453.2375656412,549,NULL)
INSERT INTO SalesInvoice VALUES (851,39413.3839186201,39432.21002792,686,NULL)
INSERT INTO SalesInvoice VALUES (852,39413.3998378582,39463.3629938004,624,NULL)
INSERT INTO SalesInvoice VALUES (853,39414.4763111653,39461.6823500192,291,NULL)
INSERT INTO SalesInvoice VALUES (854,39415.5962397387,39445.2686624442,232,NULL)
INSERT INTO SalesInvoice VALUES (855,39416.9174335281,39425.1503687997,132,NULL)
INSERT INTO SalesInvoice VALUES (856,39417.8926302639,39455.9454291735,538,NULL)
INSERT INTO SalesInvoice VALUES (857,39417.9294659332,39436.4422080675,263,NULL)
INSERT INTO SalesInvoice VALUES (858,39419.0019019261,39465.6141621,106,NULL)
INSERT INTO SalesInvoice VALUES (859,39419.6927249989,39443.1837595375,395,NULL)
INSERT INTO SalesInvoice VALUES (860,39419.8617716422,39439.4607248472,31,NULL)
INSERT INTO SalesInvoice VALUES (861,39421.1535075684,39467.5717586274,307,NULL)
INSERT INTO SalesInvoice VALUES (862,39422.2407143586,39436.7414788721,365,NULL)
INSERT INTO SalesInvoice VALUES (863,39423.3536671898,39464.877997663,405,NULL)
INSERT INTO SalesInvoice VALUES (864,39424.6302234652,39431.1719672729,153,NULL)
INSERT INTO SalesInvoice VALUES (865,39425.044970602,39467.6662331354,243,NULL)
INSERT INTO SalesInvoice VALUES (866,39426.5889388338,39453.9808127867,405,NULL)
INSERT INTO SalesInvoice VALUES (867,39427.7123042112,39467.1186596946,645,NULL)
INSERT INTO SalesInvoice VALUES (868,39429.1140374531,39444.7540779002,228,NULL)
INSERT INTO SalesInvoice VALUES (869,39429.3909495923,39457.1284389784,473,NULL)
INSERT INTO SalesInvoice VALUES (870,39429.8707343613,39459.6052024663,334,NULL)
INSERT INTO SalesInvoice VALUES (871,39430.5767139091,39455.6514890733,560,NULL)
INSERT INTO SalesInvoice VALUES (872,39431.1973854883,39465.8700148758,697,NULL)
INSERT INTO SalesInvoice VALUES (873,39431.255688553,39462.7576667392,181,NULL)
INSERT INTO SalesInvoice VALUES (874,39432.2398132369,39443.3470770724,332,NULL)
INSERT INTO SalesInvoice VALUES (875,39432.9516113704,39471.2231457603,597,NULL)
INSERT INTO SalesInvoice VALUES (876,39434.1323973634,39437.5509473923,293,NULL)
INSERT INTO SalesInvoice VALUES (877,39434.3808140467,39476.1337394673,160,NULL)
INSERT INTO SalesInvoice VALUES (878,39434.4074585261,39442.7614757694,82,NULL)
INSERT INTO SalesInvoice VALUES (879,39435.6226957728,39484.7401964133,186,NULL)
INSERT INTO SalesInvoice VALUES (880,39436.0827438582,39476.8810576096,177,NULL)
INSERT INTO SalesInvoice VALUES (881,39436.4377982621,39444.1053685624,510,NULL)
INSERT INTO SalesInvoice VALUES (882,39437.3922065167,39449.4947736225,332,NULL)
INSERT INTO SalesInvoice VALUES (883,39438.2159955325,39467.0159151959,206,NULL)
INSERT INTO SalesInvoice VALUES (884,39438.6211333324,39473.9903320905,260,NULL)
INSERT INTO SalesInvoice VALUES (885,39439.1541794816,39482.6561688735,583,NULL)
INSERT INTO SalesInvoice VALUES (886,39440.4706663171,39466.7972778981,322,NULL)
INSERT INTO SalesInvoice VALUES (887,39441.0739720229,39490.021097795,763,NULL)
INSERT INTO SalesInvoice VALUES (888,39441.5718900556,39457.0917995184,674,NULL)
INSERT INTO SalesInvoice VALUES (889,39442.8789942605,39467.3405767069,185,NULL)
INSERT INTO SalesInvoice VALUES (890,39443.3618513601,39450.1520286335,187,NULL)
INSERT INTO SalesInvoice VALUES (891,39444.1001388807,39487.251315815,155,NULL)
INSERT INTO SalesInvoice VALUES (892,39445.2388769268,39480.6694283663,433,NULL)
INSERT INTO SalesInvoice VALUES (893,39445.6002504001,39456.7301109511,676,NULL)
INSERT INTO SalesInvoice VALUES (894,39446.3412755156,39479.9190264395,674,NULL)
INSERT INTO SalesInvoice VALUES (895,39447.0626768053,39485.6076420331,484,NULL)
INSERT INTO SalesInvoice VALUES (896,39448.4451700306,39466.3244830141,566,NULL)
INSERT INTO SalesInvoice VALUES (897,39448.6107335831,39480.7182764352,57,NULL)
INSERT INTO SalesInvoice VALUES (898,39449.6850833847,39484.7089428452,426,NULL)
INSERT INTO SalesInvoice VALUES (899,39450.9712040625,39486.8614208025,141,NULL)
INSERT INTO SalesInvoice VALUES (900,39452.4846746069,39478.5818358486,26,NULL)
INSERT INTO SalesInvoice VALUES (901,39453.1974921824,39489.9938338929,134,NULL)
INSERT INTO SalesInvoice VALUES (902,39453.5629965582,39483.8642883841,55,NULL)
INSERT INTO SalesInvoice VALUES (903,39454.9203743439,39471.1414760747,674,NULL)
INSERT INTO SalesInvoice VALUES (904,39456.0933019767,39504.3497448366,20,NULL)
INSERT INTO SalesInvoice VALUES (905,39456.1539426721,39498.2026460898,315,NULL)
INSERT INTO SalesInvoice VALUES (906,39456.4107460005,39501.2056189271,174,NULL)
INSERT INTO SalesInvoice VALUES (907,39456.9151627543,39460.752214697,682,NULL)
INSERT INTO SalesInvoice VALUES (908,39458.255245685,39463.9376684437,200,NULL)
INSERT INTO SalesInvoice VALUES (909,39458.5904216188,39459.5994360396,730,NULL)
INSERT INTO SalesInvoice VALUES (910,39458.7871594384,39476.5918008776,32,NULL)
INSERT INTO SalesInvoice VALUES (911,39459.232757618,39482.6065783287,224,NULL)
INSERT INTO SalesInvoice VALUES (912,39459.5582518716,39487.6068993261,392,NULL)
INSERT INTO SalesInvoice VALUES (913,39460.0817072354,39462.3226234941,549,NULL)
INSERT INTO SalesInvoice VALUES (914,39461.2301948494,39476.4263255006,535,NULL)
INSERT INTO SalesInvoice VALUES (915,39461.4449179335,39487.1157090543,775,NULL)
INSERT INTO SalesInvoice VALUES (916,39462.1313702414,39505.2531871405,433,NULL)
INSERT INTO SalesInvoice VALUES (917,39463.5551752345,39479.7039308161,763,NULL)
INSERT INTO SalesInvoice VALUES (918,39464.2884519581,39476.4214179524,387,NULL)
INSERT INTO SalesInvoice VALUES (919,39465.6754546149,39488.0382103536,773,NULL)
INSERT INTO SalesInvoice VALUES (920,39465.9136715568,39485.1233389587,336,NULL)
INSERT INTO SalesInvoice VALUES (921,39466.3760268659,39467.6548272412,390,NULL)
INSERT INTO SalesInvoice VALUES (922,39466.3908305717,39499.8779026308,211,NULL)
INSERT INTO SalesInvoice VALUES (923,39467.6051602192,39496.0797295096,540,NULL)
INSERT INTO SalesInvoice VALUES (924,39468.9876455656,39483.5339858666,537,NULL)
INSERT INTO SalesInvoice VALUES (925,39469.5767677178,39512.9454399929,459,NULL)
INSERT INTO SalesInvoice VALUES (926,39471.1372691976,39497.2479310872,743,NULL)
INSERT INTO SalesInvoice VALUES (927,39472.0031853562,39492.9495273812,394,NULL)
INSERT INTO SalesInvoice VALUES (928,39472.4002029458,39479.4174340394,111,NULL)
INSERT INTO SalesInvoice VALUES (929,39473.2869741829,39515.4230679147,724,NULL)
INSERT INTO SalesInvoice VALUES (930,39473.9866820215,39500.7876591204,665,NULL)
INSERT INTO SalesInvoice VALUES (931,39474.793602711,39475.3896045311,388,NULL)
INSERT INTO SalesInvoice VALUES (932,39475.862941382,39488.5242136835,176,NULL)
INSERT INTO SalesInvoice VALUES (933,39477.3646741039,39504.5986128342,511,NULL)
INSERT INTO SalesInvoice VALUES (934,39477.625915608,39501.3048338636,509,NULL)
INSERT INTO SalesInvoice VALUES (935,39478.4662938204,39523.450592486,621,NULL)
INSERT INTO SalesInvoice VALUES (936,39478.9446541901,39520.7250389273,474,NULL)
INSERT INTO SalesInvoice VALUES (937,39480.525647082,39507.7908043989,458,NULL)
INSERT INTO SalesInvoice VALUES (938,39480.9269443733,39527.1949303487,746,NULL)
INSERT INTO SalesInvoice VALUES (939,39481.0016554287,39498.1943305731,503,NULL)
INSERT INTO SalesInvoice VALUES (940,39481.2332839087,39515.9532178176,153,NULL)
INSERT INTO SalesInvoice VALUES (941,39481.3151318468,39518.1383998242,429,NULL)
INSERT INTO SalesInvoice VALUES (942,39482.780228198,39495.0335981786,455,NULL)
INSERT INTO SalesInvoice VALUES (943,39483.1146278066,39524.0053448643,2,NULL)
INSERT INTO SalesInvoice VALUES (944,39484.6430758094,39519.4425045329,397,NULL)
INSERT INTO SalesInvoice VALUES (945,39485.4207975797,39520.1287056465,10,NULL)
INSERT INTO SalesInvoice VALUES (946,39486.3216016028,39535.0259239095,519,NULL)
INSERT INTO SalesInvoice VALUES (947,39487.5461381265,39525.0742029549,645,NULL)
INSERT INTO SalesInvoice VALUES (948,39487.6816983014,39497.1846246132,108,NULL)
INSERT INTO SalesInvoice VALUES (949,39487.7021978841,39535.6291717665,3,NULL)
INSERT INTO SalesInvoice VALUES (950,39487.7376862801,39523.8395449743,370,NULL)
INSERT INTO SalesInvoice VALUES (951,39488.2354554896,39501.6169553226,169,NULL)
INSERT INTO SalesInvoice VALUES (952,39488.583094493,39492.6990387946,133,NULL)
INSERT INTO SalesInvoice VALUES (953,39489.7502986338,39527.5438031299,84,NULL)
INSERT INTO SalesInvoice VALUES (954,39491.2610120821,39507.1191400371,259,NULL)
INSERT INTO SalesInvoice VALUES (955,39491.2937856853,39516.4112345979,434,NULL)
INSERT INTO SalesInvoice VALUES (956,39492.6940660304,39506.0135993016,124,NULL)
INSERT INTO SalesInvoice VALUES (957,39493.0611582103,39513.4207435214,80,NULL)
INSERT INTO SalesInvoice VALUES (958,39494.2130995501,39520.8446763892,448,NULL)
INSERT INTO SalesInvoice VALUES (959,39495.7122345942,39544.9207278268,272,NULL)
INSERT INTO SalesInvoice VALUES (960,39496.2491550178,39515.9582089741,260,NULL)
INSERT INTO SalesInvoice VALUES (961,39496.8996858044,39499.1759820521,57,NULL)
INSERT INTO SalesInvoice VALUES (962,39497.9709254163,39503.8093080481,728,NULL)
INSERT INTO SalesInvoice VALUES (963,39498.0004438246,39502.1521960537,503,NULL)
INSERT INTO SalesInvoice VALUES (964,39499.3901142823,39547.4835376143,318,NULL)
INSERT INTO SalesInvoice VALUES (965,39500.1134698788,39511.2463300655,769,NULL)
INSERT INTO SalesInvoice VALUES (966,39501.3295457639,39501.4488623195,598,NULL)
INSERT INTO SalesInvoice VALUES (967,39501.7379298229,39526.4640900286,251,NULL)
INSERT INTO SalesInvoice VALUES (968,39502.77792877,39535.7377899228,505,NULL)
INSERT INTO SalesInvoice VALUES (969,39503.1041227275,39541.6942768658,380,NULL)
INSERT INTO SalesInvoice VALUES (970,39504.5143986015,39509.0428614717,190,NULL)
INSERT INTO SalesInvoice VALUES (971,39505.585168123,39543.0714199042,360,NULL)
INSERT INTO SalesInvoice VALUES (972,39506.3416504003,39552.7352994692,56,NULL)
INSERT INTO SalesInvoice VALUES (973,39506.794292649,39549.500108157,304,NULL)
INSERT INTO SalesInvoice VALUES (974,39507.6324003429,39508.4756501007,700,NULL)
INSERT INTO SalesInvoice VALUES (975,39508.3481010927,39527.547146723,25,NULL)
INSERT INTO SalesInvoice VALUES (976,39509.8515453578,39537.9588717046,391,NULL)
INSERT INTO SalesInvoice VALUES (977,39510.5570411964,39532.5539012557,357,NULL)
INSERT INTO SalesInvoice VALUES (978,39510.722284177,39543.7871378991,66,NULL)
INSERT INTO SalesInvoice VALUES (979,39511.7489551822,39523.184608632,203,NULL)
INSERT INTO SalesInvoice VALUES (980,39512.8849534787,39550.3077555673,535,NULL)
INSERT INTO SalesInvoice VALUES (981,39513.5143807282,39521.816523632,441,NULL)
INSERT INTO SalesInvoice VALUES (982,39514.5331768813,39540.5907862743,288,NULL)
INSERT INTO SalesInvoice VALUES (983,39515.9947164577,39532.1130801924,756,NULL)
INSERT INTO SalesInvoice VALUES (984,39516.8256171892,39553.2499964215,448,NULL)
INSERT INTO SalesInvoice VALUES (985,39518.3776523849,39552.2611616916,165,NULL)
INSERT INTO SalesInvoice VALUES (986,39518.5386716091,39531.7291418068,274,NULL)
INSERT INTO SalesInvoice VALUES (987,39519.1389892328,39562.6114243915,319,NULL)
INSERT INTO SalesInvoice VALUES (988,39520.6315161473,39526.2720218277,15,NULL)
INSERT INTO SalesInvoice VALUES (989,39521.4546096846,39569.8545556105,13,NULL)
INSERT INTO SalesInvoice VALUES (990,39521.8611755964,39561.524721335,684,NULL)
INSERT INTO SalesInvoice VALUES (991,39522.5416858767,39523.7915118315,489,NULL)
INSERT INTO SalesInvoice VALUES (992,39523.0745626363,39550.2530537023,375,NULL)
INSERT INTO SalesInvoice VALUES (993,39524.2108663792,39565.3371494174,262,NULL)
INSERT INTO SalesInvoice VALUES (994,39524.5737992964,39573.6376485104,491,NULL)
INSERT INTO SalesInvoice VALUES (995,39524.8918606342,39526.2647200418,488,NULL)
INSERT INTO SalesInvoice VALUES (996,39525.6930752546,39566.342994694,536,NULL)
INSERT INTO SalesInvoice VALUES (997,39526.8602223145,39561.2238091525,281,NULL)
INSERT INTO SalesInvoice VALUES (998,39527.2808290451,39537.2587439815,254,NULL)
INSERT INTO SalesInvoice VALUES (999,39528.0976531908,39571.0891086947,605,NULL)
INSERT INTO SalesInvoice VALUES (1000,39529.4255287864,39567.3754988301,469,NULL)
INSERT INTO SalesInvoice VALUES (1001,39530.1494617603,39542.3047740587,298,NULL)
INSERT INTO SalesInvoice VALUES (1002,39531.4283613157,39548.5648838465,484,NULL)
INSERT INTO SalesInvoice VALUES (1003,39532.8709606445,39544.8259889501,88,NULL)
INSERT INTO SalesInvoice VALUES (1004,39533.2927763583,39558.9156006554,584,NULL)
INSERT INTO SalesInvoice VALUES (1005,39534.2207626998,39554.0613044273,745,NULL)
INSERT INTO SalesInvoice VALUES (1006,39535.4973650856,39561.0200236496,328,NULL)
INSERT INTO SalesInvoice VALUES (1007,39536.0843357225,39549.2749477231,471,NULL)
INSERT INTO SalesInvoice VALUES (1008,39536.4064069902,39565.2337219128,193,NULL)
INSERT INTO SalesInvoice VALUES (1009,39537.987346041,39571.2881445084,189,NULL)
INSERT INTO SalesInvoice VALUES (1010,39539.2287683197,39568.3405999696,95,NULL)
INSERT INTO SalesInvoice VALUES (1011,39539.5721283083,39578.5666269921,77,NULL)
INSERT INTO SalesInvoice VALUES (1012,39540.499114381,39557.289633942,572,NULL)
INSERT INTO SalesInvoice VALUES (1013,39541.7315098527,39570.9582855883,764,NULL)
INSERT INTO SalesInvoice VALUES (1014,39542.8907990893,39587.8111226331,132,NULL)
INSERT INTO SalesInvoice VALUES (1015,39544.0638862438,39549.8203942918,464,NULL)
INSERT INTO SalesInvoice VALUES (1016,39545.24051618,39577.7093932831,726,NULL)
INSERT INTO SalesInvoice VALUES (1017,39545.8502486335,39557.8932881154,444,NULL)
INSERT INTO SalesInvoice VALUES (1018,39547.3758904319,39579.5617913717,678,NULL)
INSERT INTO SalesInvoice VALUES (1019,39548.4965783837,39562.9769285853,603,NULL)
INSERT INTO SalesInvoice VALUES (1020,39549.028773736,39594.7989667196,104,NULL)
INSERT INTO SalesInvoice VALUES (1021,39549.0729940938,39586.770771247,9,NULL)
INSERT INTO SalesInvoice VALUES (1022,39549.3535116337,39597.0249800963,386,NULL)
INSERT INTO SalesInvoice VALUES (1023,39550.2187765154,39565.6716399774,130,NULL)
INSERT INTO SalesInvoice VALUES (1024,39551.0364933246,39576.3696094209,519,NULL)
INSERT INTO SalesInvoice VALUES (1025,39551.9556407276,39586.4481940845,11,NULL)
INSERT INTO SalesInvoice VALUES (1026,39552.2447582013,39591.282397611,548,NULL)
INSERT INTO SalesInvoice VALUES (1027,39553.676382898,39569.0314622038,1,NULL)
INSERT INTO SalesInvoice VALUES (1028,39554.9128881881,39559.3216427002,302,NULL)
INSERT INTO SalesInvoice VALUES (1029,39556.4569217375,39574.2000481954,620,NULL)
INSERT INTO SalesInvoice VALUES (1030,39557.1925465253,39563.6373444723,233,NULL)
INSERT INTO SalesInvoice VALUES (1031,39557.6298619012,39596.4654228429,79,NULL)
INSERT INTO SalesInvoice VALUES (1032,39558.9321725705,39576.9932803001,593,NULL)
INSERT INTO SalesInvoice VALUES (1033,39559.2513719447,39574.8086861377,409,NULL)
INSERT INTO SalesInvoice VALUES (1034,39559.3224684481,39605.1005536703,594,NULL)
INSERT INTO SalesInvoice VALUES (1035,39559.8271418192,39572.2590170368,396,NULL)
INSERT INTO SalesInvoice VALUES (1036,39560.7127228185,39583.6243756428,259,NULL)
INSERT INTO SalesInvoice VALUES (1037,39561.537385351,39580.5072070171,676,NULL)
INSERT INTO SalesInvoice VALUES (1038,39562.5517357261,39585.2276537589,623,NULL)
INSERT INTO SalesInvoice VALUES (1039,39562.8231908172,39595.3823168425,256,NULL)
INSERT INTO SalesInvoice VALUES (1040,39563.0157548672,39595.8660254794,210,NULL)
INSERT INTO SalesInvoice VALUES (1041,39563.441881687,39602.6156326491,334,NULL)
INSERT INTO SalesInvoice VALUES (1042,39563.5294528093,39590.6401836425,750,NULL)
INSERT INTO SalesInvoice VALUES (1043,39563.7656167266,39601.5847458296,36,NULL)
INSERT INTO SalesInvoice VALUES (1044,39564.2125937596,39569.5837317931,484,NULL)
INSERT INTO SalesInvoice VALUES (1045,39565.127523573,39594.3578073587,761,NULL)
INSERT INTO SalesInvoice VALUES (1046,39565.5373260979,39593.5376027041,436,NULL)
INSERT INTO SalesInvoice VALUES (1047,39567.0135738837,39587.9297047327,252,NULL)
INSERT INTO SalesInvoice VALUES (1048,39567.3042452253,39570.3244278117,684,NULL)
INSERT INTO SalesInvoice VALUES (1049,39567.4535624657,39608.0278000887,32,NULL)
INSERT INTO SalesInvoice VALUES (1050,39568.0124812917,39616.4715357834,371,NULL)
INSERT INTO SalesInvoice VALUES (1051,39569.2783924445,39617.4241177176,219,NULL)
INSERT INTO SalesInvoice VALUES (1052,39569.6471306784,39608.4446473172,550,NULL)
INSERT INTO SalesInvoice VALUES (1053,39570.9970173269,39601.108116066,0,NULL)
INSERT INTO SalesInvoice VALUES (1054,39571.7604448737,39620.2858619717,542,NULL)
INSERT INTO SalesInvoice VALUES (1055,39571.7895930496,39591.563321387,703,NULL)
INSERT INTO SalesInvoice VALUES (1056,39572.1880924774,39583.421036649,178,NULL)
INSERT INTO SalesInvoice VALUES (1057,39572.4034506665,39579.7936520042,408,NULL)
INSERT INTO SalesInvoice VALUES (1058,39573.5026313336,39578.753158085,358,NULL)
INSERT INTO SalesInvoice VALUES (1059,39574.846833987,39594.8315069746,62,NULL)
INSERT INTO SalesInvoice VALUES (1060,39575.1775096491,39580.1917804205,734,NULL)
INSERT INTO SalesInvoice VALUES (1061,39576.7292220344,39617.503395407,770,NULL)
INSERT INTO SalesInvoice VALUES (1062,39577.2917349273,39578.4473383247,698,NULL)
INSERT INTO SalesInvoice VALUES (1063,39578.1993355413,39593.381472766,575,NULL)
INSERT INTO SalesInvoice VALUES (1064,39579.0206950076,39616.0415752008,24,NULL)
INSERT INTO SalesInvoice VALUES (1065,39580.5724654077,39610.4141888681,124,NULL)
INSERT INTO SalesInvoice VALUES (1066,39581.0148828049,39591.4610924733,422,NULL)
INSERT INTO SalesInvoice VALUES (1067,39581.4447421914,39609.5243516977,559,NULL)
INSERT INTO SalesInvoice VALUES (1068,39582.7792888855,39608.958576071,680,NULL)
INSERT INTO SalesInvoice VALUES (1069,39583.2283513761,39624.4924291591,597,NULL)
INSERT INTO SalesInvoice VALUES (1070,39584.1825318353,39590.1151479036,55,NULL)
INSERT INTO SalesInvoice VALUES (1071,39585.756640793,39613.4619975059,29,NULL)
INSERT INTO SalesInvoice VALUES (1072,39586.8919596159,39621.4018984059,96,NULL)
INSERT INTO SalesInvoice VALUES (1073,39588.3243742336,39612.147685141,499,NULL)
INSERT INTO SalesInvoice VALUES (1074,39589.5734002444,39608.4474288009,673,NULL)
INSERT INTO SalesInvoice VALUES (1075,39590.61458419,39614.9831474478,752,NULL)
INSERT INTO SalesInvoice VALUES (1076,39591.3863314896,39592.7966529877,72,NULL)
INSERT INTO SalesInvoice VALUES (1077,39592.9270251212,39596.4725180297,256,NULL)
INSERT INTO SalesInvoice VALUES (1078,39593.9928474964,39641.7191675968,698,NULL)
INSERT INTO SalesInvoice VALUES (1079,39594.5405603512,39600.3962945811,701,NULL)
INSERT INTO SalesInvoice VALUES (1080,39594.9208201636,39599.4610397394,91,NULL)
INSERT INTO SalesInvoice VALUES (1081,39595.821222258,39602.7485262656,404,NULL)
INSERT INTO SalesInvoice VALUES (1082,39596.7409250183,39608.8634219517,604,NULL)
INSERT INTO SalesInvoice VALUES (1083,39598.0722448751,39616.70402611,708,NULL)
INSERT INTO SalesInvoice VALUES (1084,39598.5738195418,39635.2555740199,238,NULL)
INSERT INTO SalesInvoice VALUES (1085,39598.7384275179,39644.2675787828,678,NULL)
INSERT INTO SalesInvoice VALUES (1086,39600.2335504439,39631.7216831833,498,NULL)
INSERT INTO SalesInvoice VALUES (1087,39600.6899194009,39641.2851304448,357,NULL)
INSERT INTO SalesInvoice VALUES (1088,39601.5013957487,39651.4479843393,38,NULL)
INSERT INTO SalesInvoice VALUES (1089,39602.3854049556,39613.0944759195,274,NULL)
INSERT INTO SalesInvoice VALUES (1090,39603.6561021696,39605.7356401542,630,NULL)
INSERT INTO SalesInvoice VALUES (1091,39604.3764402897,39616.5905142044,339,NULL)
INSERT INTO SalesInvoice VALUES (1092,39605.9307744102,39631.3504798753,148,NULL)
INSERT INTO SalesInvoice VALUES (1093,39606.097074391,39621.6263065951,245,NULL)
INSERT INTO SalesInvoice VALUES (1094,39606.672193043,39618.8544156706,89,NULL)
INSERT INTO SalesInvoice VALUES (1095,39607.4075935059,39629.9648572244,135,NULL)
INSERT INTO SalesInvoice VALUES (1096,39608.3672841841,39609.501300896,29,NULL)
INSERT INTO SalesInvoice VALUES (1097,39609.9416997877,39613.5270108751,52,NULL)
INSERT INTO SalesInvoice VALUES (1098,39610.0521503963,39648.4832087655,308,NULL)
INSERT INTO SalesInvoice VALUES (1099,39610.9283971117,39660.1463125263,63,NULL)
INSERT INTO SalesInvoice VALUES (1100,39611.235730097,39650.4798173852,325,NULL)
INSERT INTO SalesInvoice VALUES (1101,39611.354876428,39640.2966396052,723,NULL)
INSERT INTO SalesInvoice VALUES (1102,39611.7369309454,39640.1316489638,28,NULL)
INSERT INTO SalesInvoice VALUES (1103,39612.6313748528,39621.6125538443,700,NULL)
INSERT INTO SalesInvoice VALUES (1104,39613.1222543975,39641.1007647809,160,NULL)
INSERT INTO SalesInvoice VALUES (1105,39613.1825842238,39653.0495572348,561,NULL)
INSERT INTO SalesInvoice VALUES (1106,39613.7747774702,39663.4627982795,457,NULL)
INSERT INTO SalesInvoice VALUES (1107,39614.4531488793,39660.8458243798,691,NULL)
INSERT INTO SalesInvoice VALUES (1108,39615.3369514375,39664.8619944385,420,NULL)
INSERT INTO SalesInvoice VALUES (1109,39616.4309173921,39622.4820962509,49,NULL)
INSERT INTO SalesInvoice VALUES (1110,39616.6632904035,39649.126896119,416,NULL)
INSERT INTO SalesInvoice VALUES (1111,39617.0712693721,39643.2140767076,118,NULL)
INSERT INTO SalesInvoice VALUES (1112,39617.591761281,39653.3808388704,78,NULL)
INSERT INTO SalesInvoice VALUES (1113,39619.0312778022,39662.935370744,515,NULL)
INSERT INTO SalesInvoice VALUES (1114,39620.5171293386,39620.5244429882,289,NULL)
INSERT INTO SalesInvoice VALUES (1115,39621.966878787,39639.9437520222,287,NULL)
INSERT INTO SalesInvoice VALUES (1116,39622.7666106491,39623.4356491974,570,NULL)
INSERT INTO SalesInvoice VALUES (1117,39623.8125406452,39660.429466124,682,NULL)
INSERT INTO SalesInvoice VALUES (1118,39625.3590148369,39631.019620376,727,NULL)
INSERT INTO SalesInvoice VALUES (1119,39626.7959536854,39673.7229300738,208,NULL)
INSERT INTO SalesInvoice VALUES (1120,39627.4352315822,39659.8666590668,659,NULL)
INSERT INTO SalesInvoice VALUES (1121,39627.9917966998,39672.2397449735,28,NULL)
INSERT INTO SalesInvoice VALUES (1122,39628.1610363983,39676.7616253483,459,NULL)
INSERT INTO SalesInvoice VALUES (1123,39629.4287913726,39675.5277645077,124,NULL)
INSERT INTO SalesInvoice VALUES (1124,39630.2248366665,39674.7092638176,621,NULL)
INSERT INTO SalesInvoice VALUES (1125,39630.3415583559,39658.5449482884,330,NULL)
INSERT INTO SalesInvoice VALUES (1126,39631.0476034623,39636.2626511953,379,NULL)
INSERT INTO SalesInvoice VALUES (1127,39631.3257994916,39669.2428283337,391,NULL)
INSERT INTO SalesInvoice VALUES (1128,39631.6144971808,39653.22515065,596,NULL)
INSERT INTO SalesInvoice VALUES (1129,39631.8556999573,39643.7430556287,389,NULL)
INSERT INTO SalesInvoice VALUES (1130,39633.3103875799,39682.2283975191,645,NULL)
INSERT INTO SalesInvoice VALUES (1131,39633.3664200449,39650.3054691866,687,NULL)
INSERT INTO SalesInvoice VALUES (1132,39633.8949880937,39657.69666576,453,NULL)
INSERT INTO SalesInvoice VALUES (1133,39634.4600551172,39642.4242961044,659,NULL)
INSERT INTO SalesInvoice VALUES (1134,39635.2418850364,39646.3549959029,669,NULL)
INSERT INTO SalesInvoice VALUES (1135,39635.6755237726,39654.9225054383,238,NULL)
INSERT INTO SalesInvoice VALUES (1136,39636.2499680633,39653.8252779851,287,NULL)
INSERT INTO SalesInvoice VALUES (1137,39637.7025769849,39647.5611493294,677,NULL)
INSERT INTO SalesInvoice VALUES (1138,39638.8772195726,39643.7941086241,701,NULL)
INSERT INTO SalesInvoice VALUES (1139,39639.7880046102,39646.4549605361,160,NULL)
INSERT INTO SalesInvoice VALUES (1140,39640.0408468698,39655.7675846591,134,NULL)
INSERT INTO SalesInvoice VALUES (1141,39640.4412156505,39643.0733302587,502,NULL)
INSERT INTO SalesInvoice VALUES (1142,39641.5066214725,39666.4322242623,738,NULL)
INSERT INTO SalesInvoice VALUES (1143,39641.5531000845,39669.9842921931,252,NULL)
INSERT INTO SalesInvoice VALUES (1144,39642.4218305218,39687.9682648586,306,NULL)
INSERT INTO SalesInvoice VALUES (1145,39643.243630882,39678.547183293,260,NULL)
INSERT INTO SalesInvoice VALUES (1146,39643.5338226807,39683.0001344525,288,NULL)
INSERT INTO SalesInvoice VALUES (1147,39643.9402328641,39648.2631397976,216,NULL)
INSERT INTO SalesInvoice VALUES (1148,39644.8531817206,39680.2142146301,723,NULL)
INSERT INTO SalesInvoice VALUES (1149,39646.0343341561,39657.4783422734,653,NULL)
INSERT INTO SalesInvoice VALUES (1150,39647.1115817467,39693.2067699399,145,NULL)
INSERT INTO SalesInvoice VALUES (1151,39647.6372243482,39684.6990791056,504,NULL)
INSERT INTO SalesInvoice VALUES (1152,39648.8379238437,39695.188549418,763,NULL)
INSERT INTO SalesInvoice VALUES (1153,39649.7058817299,39666.0626888491,119,NULL)
INSERT INTO SalesInvoice VALUES (1154,39651.0188280823,39700.039065946,690,NULL)
INSERT INTO SalesInvoice VALUES (1155,39652.4795927448,39653.4332745684,15,NULL)
INSERT INTO SalesInvoice VALUES (1156,39652.657043644,39671.3126345643,341,NULL)
INSERT INTO SalesInvoice VALUES (1157,39652.9183132,39674.8602534926,120,NULL)
INSERT INTO SalesInvoice VALUES (1158,39653.3819116088,39701.535906691,730,NULL)
INSERT INTO SalesInvoice VALUES (1159,39653.9075553895,39680.0438834457,222,NULL)
INSERT INTO SalesInvoice VALUES (1160,39655.2740113871,39668.8486260465,174,NULL)
INSERT INTO SalesInvoice VALUES (1161,39655.4556973133,39688.5709145474,771,NULL)
INSERT INTO SalesInvoice VALUES (1162,39656.5823444422,39669.9894719146,361,NULL)
INSERT INTO SalesInvoice VALUES (1163,39657.8567096049,39687.608173949,706,NULL)
INSERT INTO SalesInvoice VALUES (1164,39658.2312874312,39690.2890159442,414,NULL)
INSERT INTO SalesInvoice VALUES (1165,39658.5296197449,39661.2495007567,109,NULL)
INSERT INTO SalesInvoice VALUES (1166,39659.8665693124,39666.8978563399,773,NULL)
INSERT INTO SalesInvoice VALUES (1167,39660.3590993649,39676.5726897275,238,NULL)
INSERT INTO SalesInvoice VALUES (1168,39660.6465034067,39667.214954992,469,NULL)
INSERT INTO SalesInvoice VALUES (1169,39662.0712316629,39686.9814746915,470,NULL)
INSERT INTO SalesInvoice VALUES (1170,39663.4411473853,39673.4447191116,347,NULL)
INSERT INTO SalesInvoice VALUES (1171,39663.7076954951,39701.9862165131,295,NULL)
INSERT INTO SalesInvoice VALUES (1172,39665.0551641919,39693.9289984854,373,NULL)
INSERT INTO SalesInvoice VALUES (1173,39665.1329609393,39676.027694227,36,NULL)
INSERT INTO SalesInvoice VALUES (1174,39665.7839945479,39701.5525382103,770,NULL)
INSERT INTO SalesInvoice VALUES (1175,39666.3527170144,39666.533173559,753,NULL)
INSERT INTO SalesInvoice VALUES (1176,39666.882958269,39671.9648181946,402,NULL)
INSERT INTO SalesInvoice VALUES (1177,39668.3701106061,39709.484786382,495,NULL)
INSERT INTO SalesInvoice VALUES (1178,39668.9953349651,39680.2699847378,402,NULL)
INSERT INTO SalesInvoice VALUES (1179,39669.7227286273,39681.700933029,87,NULL)
INSERT INTO SalesInvoice VALUES (1180,39669.823164378,39716.4818828823,721,NULL)
INSERT INTO SalesInvoice VALUES (1181,39670.5344133371,39672.2155344986,717,NULL)
INSERT INTO SalesInvoice VALUES (1182,39670.7915875,39709.3410926182,350,NULL)
INSERT INTO SalesInvoice VALUES (1183,39671.9859823514,39674.2749690962,138,NULL)
INSERT INTO SalesInvoice VALUES (1184,39673.1675833716,39674.5820499506,465,NULL)
INSERT INTO SalesInvoice VALUES (1185,39674.3040609742,39709.5986617807,302,NULL)
INSERT INTO SalesInvoice VALUES (1186,39675.7329645327,39686.4079643079,732,NULL)
INSERT INTO SalesInvoice VALUES (1187,39677.2478544823,39697.1175668015,321,NULL)
INSERT INTO SalesInvoice VALUES (1188,39678.6456491638,39699.4695249623,604,NULL)
INSERT INTO SalesInvoice VALUES (1189,39678.8040515364,39721.2131780319,763,NULL)
INSERT INTO SalesInvoice VALUES (1190,39679.6801335424,39704.4351193679,97,NULL)
INSERT INTO SalesInvoice VALUES (1191,39679.7274444897,39708.2468603963,157,NULL)
INSERT INTO SalesInvoice VALUES (1192,39680.2136546246,39695.3337055301,694,NULL)
INSERT INTO SalesInvoice VALUES (1193,39680.8220821154,39728.1444656665,396,NULL)
INSERT INTO SalesInvoice VALUES (1194,39681.1923149066,39722.1282816316,723,NULL)
INSERT INTO SalesInvoice VALUES (1195,39681.6110071533,39725.41926682,263,NULL)
INSERT INTO SalesInvoice VALUES (1196,39682.7303655899,39694.7612278752,252,NULL)
INSERT INTO SalesInvoice VALUES (1197,39684.252296695,39709.7778198888,210,NULL)
INSERT INTO SalesInvoice VALUES (1198,39684.4449039742,39720.6992005263,766,NULL)
INSERT INTO SalesInvoice VALUES (1199,39685.9415637328,39714.6497131782,567,NULL)
INSERT INTO SalesInvoice VALUES (1200,39687.3545283563,39698.1333364506,712,NULL)
INSERT INTO SalesInvoice VALUES (1201,39688.7339106332,39702.3489124755,608,NULL)
INSERT INTO SalesInvoice VALUES (1202,39689.8962297495,39695.4583045125,342,NULL)
INSERT INTO SalesInvoice VALUES (1203,39690.8539145704,39722.4011042906,344,NULL)
INSERT INTO SalesInvoice VALUES (1204,39691.0765048445,39740.7896658836,372,NULL)
INSERT INTO SalesInvoice VALUES (1205,39691.9279631201,39740.6457782225,91,NULL)
INSERT INTO SalesInvoice VALUES (1206,39692.4981902441,39697.0345690583,348,NULL)
INSERT INTO SalesInvoice VALUES (1207,39693.2150362583,39718.6290318664,213,NULL)
INSERT INTO SalesInvoice VALUES (1208,39693.3907753219,39728.1994657128,388,NULL)
INSERT INTO SalesInvoice VALUES (1209,39693.7314787068,39702.9368142425,433,NULL)
INSERT INTO SalesInvoice VALUES (1210,39694.993882587,39744.1642265786,494,NULL)
INSERT INTO SalesInvoice VALUES (1211,39695.2154831088,39695.6456466211,52,NULL)
INSERT INTO SalesInvoice VALUES (1212,39696.7558444029,39743.7108210801,415,NULL)
INSERT INTO SalesInvoice VALUES (1213,39698.0943628989,39717.7176854211,151,NULL)
INSERT INTO SalesInvoice VALUES (1214,39698.3556604308,39701.4887987858,690,NULL)
INSERT INTO SalesInvoice VALUES (1215,39699.4096135434,39720.0362660074,68,NULL)
INSERT INTO SalesInvoice VALUES (1216,39699.9927637306,39749.2243027974,545,NULL)
INSERT INTO SalesInvoice VALUES (1217,39700.3107416277,39739.6777272036,304,NULL)
INSERT INTO SalesInvoice VALUES (1218,39700.5329390442,39700.8837888842,327,NULL)
INSERT INTO SalesInvoice VALUES (1219,39700.6787301842,39734.0418221676,194,NULL)
INSERT INTO SalesInvoice VALUES (1220,39701.8046020675,39738.539347227,697,NULL)
INSERT INTO SalesInvoice VALUES (1221,39701.9695508648,39739.3625777052,71,NULL)
INSERT INTO SalesInvoice VALUES (1222,39703.0176414324,39715.2658540611,178,NULL)
INSERT INTO SalesInvoice VALUES (1223,39703.9530056121,39721.1995374885,407,NULL)
INSERT INTO SalesInvoice VALUES (1224,39704.6568341993,39752.7221751511,131,NULL)
INSERT INTO SalesInvoice VALUES (1225,39705.6581032024,39727.0875981058,227,NULL)
INSERT INTO SalesInvoice VALUES (1226,39707.0093870633,39744.2628737536,508,NULL)
INSERT INTO SalesInvoice VALUES (1227,39708.3688449004,39732.3189784475,486,NULL)
INSERT INTO SalesInvoice VALUES (1228,39709.8033257492,39710.8010277423,664,NULL)
INSERT INTO SalesInvoice VALUES (1229,39710.7543480869,39732.269472826,716,NULL)
INSERT INTO SalesInvoice VALUES (1230,39712.2362092048,39761.6842500699,78,NULL)
INSERT INTO SalesInvoice VALUES (1231,39712.3201816557,39751.1543347367,29,NULL)
INSERT INTO SalesInvoice VALUES (1232,39713.3029268664,39736.8307301615,322,NULL)
INSERT INTO SalesInvoice VALUES (1233,39713.532776781,39725.4872887915,111,NULL)
INSERT INTO SalesInvoice VALUES (1234,39714.9629155765,39727.9951126307,170,NULL)
INSERT INTO SalesInvoice VALUES (1235,39716.172782715,39721.2842374078,113,NULL)
INSERT INTO SalesInvoice VALUES (1236,39716.2802824137,39742.4895717552,349,NULL)
INSERT INTO SalesInvoice VALUES (1237,39717.4967823254,39767.3432262268,487,NULL)
INSERT INTO SalesInvoice VALUES (1238,39718.9817737968,39729.1756873739,101,NULL)
INSERT INTO SalesInvoice VALUES (1239,39720.183329566,39761.7361219806,305,NULL)
INSERT INTO SalesInvoice VALUES (1240,39720.9108606223,39728.9191298089,63,NULL)
INSERT INTO SalesInvoice VALUES (1241,39721.1682367073,39747.8537218198,157,NULL)
INSERT INTO SalesInvoice VALUES (1242,39722.2973237882,39770.9331703597,72,NULL)
INSERT INTO SalesInvoice VALUES (1243,39723.5755743804,39745.4796693784,190,NULL)
INSERT INTO SalesInvoice VALUES (1244,39724.3252021981,39768.7710131786,365,NULL)
INSERT INTO SalesInvoice VALUES (1245,39724.772848315,39749.924539475,714,NULL)
INSERT INTO SalesInvoice VALUES (1246,39724.796162139,39746.9613351971,127,NULL)
INSERT INTO SalesInvoice VALUES (1247,39724.8775619629,39739.9235272512,337,NULL)
INSERT INTO SalesInvoice VALUES (1248,39725.9064977065,39768.6866508335,132,NULL)
INSERT INTO SalesInvoice VALUES (1249,39726.2007422015,39741.6117248604,351,NULL)
INSERT INTO SalesInvoice VALUES (1250,39726.599160398,39731.7973951821,92,NULL)
INSERT INTO SalesInvoice VALUES (1251,39726.9051343983,39768.5807919657,600,NULL)
INSERT INTO SalesInvoice VALUES (1252,39727.9817521493,39735.7552913188,695,NULL)
INSERT INTO SalesInvoice VALUES (1253,39729.0107929221,39755.2603628388,605,NULL)
INSERT INTO SalesInvoice VALUES (1254,39729.1272380234,39754.5636867707,386,NULL)
INSERT INTO SalesInvoice VALUES (1255,39729.9844051163,39732.1737734351,465,NULL)
INSERT INTO SalesInvoice VALUES (1256,39730.3952158606,39764.9151613138,667,NULL)
INSERT INTO SalesInvoice VALUES (1257,39731.7000157533,39758.5943281463,188,NULL)
INSERT INTO SalesInvoice VALUES (1258,39732.1090391312,39755.490736842,472,NULL)
INSERT INTO SalesInvoice VALUES (1259,39733.2337822358,39749.5416729623,723,NULL)
INSERT INTO SalesInvoice VALUES (1260,39734.4074218109,39753.5182929307,409,NULL)
INSERT INTO SalesInvoice VALUES (1261,39734.9915940786,39738.7570366818,112,NULL)
INSERT INTO SalesInvoice VALUES (1262,39736.366613687,39737.1085194366,705,NULL)
INSERT INTO SalesInvoice VALUES (1263,39737.2230565997,39744.3239719929,223,NULL)
INSERT INTO SalesInvoice VALUES (1264,39737.5779488542,39742.297893137,624,NULL)
INSERT INTO SalesInvoice VALUES (1265,39737.9957800268,39785.3641611249,60,NULL)
INSERT INTO SalesInvoice VALUES (1266,39738.0009293829,39768.667352962,558,NULL)
INSERT INTO SalesInvoice VALUES (1267,39738.7768944299,39761.5141514143,153,NULL)
INSERT INTO SalesInvoice VALUES (1268,39739.5524638005,39743.1896048272,103,NULL)
INSERT INTO SalesInvoice VALUES (1269,39740.8554669822,39786.5837663025,443,NULL)
INSERT INTO SalesInvoice VALUES (1270,39742.109745944,39744.3281321332,403,NULL)
INSERT INTO SalesInvoice VALUES (1271,39742.2321500062,39754.4188082211,351,NULL)
INSERT INTO SalesInvoice VALUES (1272,39743.2151158413,39787.7074752093,158,NULL)
INSERT INTO SalesInvoice VALUES (1273,39743.5610065459,39762.7764377627,647,NULL)
INSERT INTO SalesInvoice VALUES (1274,39744.9638808583,39746.0089483074,232,NULL)
INSERT INTO SalesInvoice VALUES (1275,39745.9321305158,39746.0497084674,383,NULL)
INSERT INTO SalesInvoice VALUES (1276,39746.489001519,39791.9620984985,549,NULL)
INSERT INTO SalesInvoice VALUES (1277,39748.0627507748,39762.33235201,365,NULL)
INSERT INTO SalesInvoice VALUES (1278,39748.2402887496,39764.9750237222,650,NULL)
INSERT INTO SalesInvoice VALUES (1279,39749.2041608373,39796.5396611915,664,NULL)
INSERT INTO SalesInvoice VALUES (1280,39749.9034802816,39797.8726171054,395,NULL)
INSERT INTO SalesInvoice VALUES (1281,39750.6348846968,39794.2545962249,236,NULL)
INSERT INTO SalesInvoice VALUES (1282,39751.3796940133,39794.1158698738,433,NULL)
INSERT INTO SalesInvoice VALUES (1283,39751.8869022404,39801.414017405,575,NULL)
INSERT INTO SalesInvoice VALUES (1284,39753.3320158021,39787.1631341805,615,NULL)
INSERT INTO SalesInvoice VALUES (1285,39754.6681699724,39803.1802593859,490,NULL)
INSERT INTO SalesInvoice VALUES (1286,39755.5656329107,39802.1527690147,426,NULL)
INSERT INTO SalesInvoice VALUES (1287,39756.1921547481,39790.53689913,185,NULL)
INSERT INTO SalesInvoice VALUES (1288,39756.6275583644,39791.7293118437,152,NULL)
INSERT INTO SalesInvoice VALUES (1289,39757.0967166961,39764.1765426947,639,NULL)
INSERT INTO SalesInvoice VALUES (1290,39758.2274434922,39791.170930432,540,NULL)
INSERT INTO SalesInvoice VALUES (1291,39759.450059153,39801.4529469333,190,NULL)
INSERT INTO SalesInvoice VALUES (1292,39760.7110015544,39806.3064010576,138,NULL)
INSERT INTO SalesInvoice VALUES (1293,39760.7865738058,39775.8823588153,16,NULL)
INSERT INTO SalesInvoice VALUES (1294,39762.1345827393,39809.5590484549,711,NULL)
INSERT INTO SalesInvoice VALUES (1295,39762.4256246706,39772.1125662884,415,NULL)
INSERT INTO SalesInvoice VALUES (1296,39763.2718515255,39775.7836112542,564,NULL)
INSERT INTO SalesInvoice VALUES (1297,39763.6546407754,39789.3966748111,218,NULL)
INSERT INTO SalesInvoice VALUES (1298,39764.709756044,39768.4662380029,329,NULL)
INSERT INTO SalesInvoice VALUES (1299,39765.8471415318,39785.3048636528,145,NULL)
INSERT INTO SalesInvoice VALUES (1300,39766.0724471954,39800.2047229802,69,NULL)
INSERT INTO SalesInvoice VALUES (1301,39767.1382433203,39770.4916665794,454,NULL)
INSERT INTO SalesInvoice VALUES (1302,39768.6936503563,39814.4660625044,629,NULL)
INSERT INTO SalesInvoice VALUES (1303,39769.864660649,39805.0177230995,603,NULL)
INSERT INTO SalesInvoice VALUES (1304,39770.3569841054,39780.1998564761,663,NULL)
INSERT INTO SalesInvoice VALUES (1305,39770.5976417041,39815.2894311398,298,NULL)
INSERT INTO SalesInvoice VALUES (1306,39771.4486869291,39811.5403049241,698,NULL)
INSERT INTO SalesInvoice VALUES (1307,39772.7042209701,39775.6371407668,528,NULL)
INSERT INTO SalesInvoice VALUES (1308,39772.759717177,39782.5831034239,352,NULL)
INSERT INTO SalesInvoice VALUES (1309,39773.3926897575,39797.5630330422,264,NULL)
INSERT INTO SalesInvoice VALUES (1310,39773.9869996886,39804.258728655,408,NULL)
INSERT INTO SalesInvoice VALUES (1311,39774.3937859509,39811.4652721223,363,NULL)
INSERT INTO SalesInvoice VALUES (1312,39775.1587616846,39803.216748774,155,NULL)
INSERT INTO SalesInvoice VALUES (1313,39775.6790888039,39806.6773676404,416,NULL)
INSERT INTO SalesInvoice VALUES (1314,39776.7336359259,39777.4473728995,189,NULL)
INSERT INTO SalesInvoice VALUES (1315,39777.1755085544,39809.8031041049,642,NULL)
INSERT INTO SalesInvoice VALUES (1316,39777.2920275974,39798.4432138226,569,NULL)
INSERT INTO SalesInvoice VALUES (1317,39777.5540362843,39793.7006716627,15,NULL)
INSERT INTO SalesInvoice VALUES (1318,39777.8505763439,39817.9706423345,92,NULL)
INSERT INTO SalesInvoice VALUES (1319,39778.1874130511,39799.4564113503,501,NULL)
INSERT INTO SalesInvoice VALUES (1320,39778.3033634019,39783.1453472842,708,NULL)
INSERT INTO SalesInvoice VALUES (1321,39778.6747662083,39781.7170028793,652,NULL)
INSERT INTO SalesInvoice VALUES (1322,39778.7914674977,39789.9996035164,106,NULL)
INSERT INTO SalesInvoice VALUES (1323,39779.1023719206,39811.5831653528,410,NULL)
INSERT INTO SalesInvoice VALUES (1324,39780.4468700476,39810.5400364883,591,NULL)
INSERT INTO SalesInvoice VALUES (1325,39781.2697173405,39816.7805976644,177,NULL)
INSERT INTO SalesInvoice VALUES (1326,39782.7176620267,39819.0173626018,343,NULL)
INSERT INTO SalesInvoice VALUES (1327,39783.9711042865,39816.8194345296,471,NULL)
INSERT INTO SalesInvoice VALUES (1328,39785.0125109757,39802.6397492777,425,NULL)
INSERT INTO SalesInvoice VALUES (1329,39786.5123226432,39803.8073682219,222,NULL)
INSERT INTO SalesInvoice VALUES (1330,39787.0687495529,39830.7192182878,631,NULL)
INSERT INTO SalesInvoice VALUES (1331,39787.4071745681,39797.5562975517,702,NULL)
INSERT INTO SalesInvoice VALUES (1332,39788.4379083446,39825.1859584189,104,NULL)
INSERT INTO SalesInvoice VALUES (1333,39788.6474358923,39810.3710229322,273,NULL)
INSERT INTO SalesInvoice VALUES (1334,39789.7674317529,39835.0022747306,732,NULL)
INSERT INTO SalesInvoice VALUES (1335,39790.0539885579,39804.8717740856,182,NULL)
INSERT INTO SalesInvoice VALUES (1336,39790.3382645932,39799.9394369176,203,NULL)
INSERT INTO SalesInvoice VALUES (1337,39790.3570464451,39821.1635585821,60,NULL)
INSERT INTO SalesInvoice VALUES (1338,39791.9288562236,39828.4417501109,66,NULL)
INSERT INTO SalesInvoice VALUES (1339,39793.0684094546,39800.2343653812,217,NULL)
INSERT INTO SalesInvoice VALUES (1340,39793.127505988,39796.9964880849,347,NULL)
INSERT INTO SalesInvoice VALUES (1341,39794.5297948634,39821.7695767808,221,NULL)
INSERT INTO SalesInvoice VALUES (1342,39795.6648328259,39819.0467320802,177,NULL)
INSERT INTO SalesInvoice VALUES (1343,39796.2488877055,39829.0362501523,764,NULL)
INSERT INTO SalesInvoice VALUES (1344,39796.5726611548,39808.681961263,765,NULL)
INSERT INTO SalesInvoice VALUES (1345,39797.2481135369,39801.9717614602,506,NULL)
INSERT INTO SalesInvoice VALUES (1346,39797.8545737714,39823.6826001757,185,NULL)
INSERT INTO SalesInvoice VALUES (1347,39798.7340844436,39808.2640300258,308,NULL)
INSERT INTO SalesInvoice VALUES (1348,39799.3117716359,39811.5716185323,605,NULL)
INSERT INTO SalesInvoice VALUES (1349,39799.9120320341,39839.6275092472,356,NULL)
INSERT INTO SalesInvoice VALUES (1350,39801.2478887291,39833.5918399955,452,NULL)
INSERT INTO SalesInvoice VALUES (1351,39802.1644070868,39842.1504539972,432,NULL)
INSERT INTO SalesInvoice VALUES (1352,39802.4810426879,39822.2457446776,774,NULL)
INSERT INTO SalesInvoice VALUES (1353,39802.6071913484,39815.7975773189,738,NULL)
INSERT INTO SalesInvoice VALUES (1354,39803.5920240077,39848.3882036594,596,NULL)
INSERT INTO SalesInvoice VALUES (1355,39804.9480373961,39843.0512105576,240,NULL)
INSERT INTO SalesInvoice VALUES (1356,39806.2067192486,39831.361822971,454,NULL)
INSERT INTO SalesInvoice VALUES (1357,39807.2936505636,39820.6874534895,233,NULL)
INSERT INTO SalesInvoice VALUES (1358,39808.3465353689,39815.356431412,476,NULL)
INSERT INTO SalesInvoice VALUES (1359,39808.8524468249,39833.7720966764,741,NULL)
INSERT INTO SalesInvoice VALUES (1360,39809.9946983827,39856.9043455297,375,NULL)
INSERT INTO SalesInvoice VALUES (1361,39810.6173966728,39815.2658416023,299,NULL)
INSERT INTO SalesInvoice VALUES (1362,39810.6710548884,39816.4826492326,402,NULL)
INSERT INTO SalesInvoice VALUES (1363,39812.073845265,39840.4662807037,482,NULL)
INSERT INTO SalesInvoice VALUES (1364,39812.0776327634,39832.6410698686,525,NULL)
INSERT INTO SalesInvoice VALUES (1365,39812.1701529621,39812.9402619218,566,NULL)
INSERT INTO SalesInvoice VALUES (1366,39813.050104909,39860.0058478202,393,NULL)
INSERT INTO SalesInvoice VALUES (1367,39814.5542859481,39821.3066923141,571,NULL)
INSERT INTO SalesInvoice VALUES (1368,39815.3983017613,39823.581484707,107,NULL)
INSERT INTO SalesInvoice VALUES (1369,39815.4968498818,39827.9932572317,268,NULL)
INSERT INTO SalesInvoice VALUES (1370,39815.5163890681,39829.4820902689,342,NULL)
INSERT INTO SalesInvoice VALUES (1371,39816.2653111583,39828.2888131292,191,NULL)
INSERT INTO SalesInvoice VALUES (1372,39816.56896388,39832.4306056058,166,NULL)
INSERT INTO SalesInvoice VALUES (1373,39818.047462666,39843.9031094858,386,NULL)
INSERT INTO SalesInvoice VALUES (1374,39819.4873506753,39866.4279681604,610,NULL)
INSERT INTO SalesInvoice VALUES (1375,39820.452813688,39820.7783125777,374,NULL)
INSERT INTO SalesInvoice VALUES (1376,39820.5025608343,39857.9881798601,604,NULL)
INSERT INTO SalesInvoice VALUES (1377,39821.4925919612,39834.2853011971,279,NULL)
INSERT INTO SalesInvoice VALUES (1378,39822.9649052587,39869.5738132033,199,NULL)
INSERT INTO SalesInvoice VALUES (1379,39823.0794207093,39831.4546606757,11,NULL)
INSERT INTO SalesInvoice VALUES (1380,39824.392386967,39839.0918804367,270,NULL)
INSERT INTO SalesInvoice VALUES (1381,39824.7137565112,39858.9623843539,154,NULL)
INSERT INTO SalesInvoice VALUES (1382,39825.1721225359,39855.044286866,348,NULL)
INSERT INTO SalesInvoice VALUES (1383,39826.6909019648,39850.5445243651,304,NULL)
INSERT INTO SalesInvoice VALUES (1384,39827.7615267282,39857.9890691434,432,NULL)
INSERT INTO SalesInvoice VALUES (1385,39829.1818446929,39832.8258260288,289,NULL)
INSERT INTO SalesInvoice VALUES (1386,39830.1817279063,39851.0513915047,42,NULL)
INSERT INTO SalesInvoice VALUES (1387,39830.3902073511,39850.4482654326,476,NULL)
INSERT INTO SalesInvoice VALUES (1388,39831.5503209021,39855.321189484,658,NULL)
INSERT INTO SalesInvoice VALUES (1389,39832.4457791088,39872.5310030129,162,NULL)
INSERT INTO SalesInvoice VALUES (1390,39833.4913207384,39854.8298707056,50,NULL)
INSERT INTO SalesInvoice VALUES (1391,39834.1550938896,39848.0030433784,624,NULL)
INSERT INTO SalesInvoice VALUES (1392,39834.5932436413,39866.8119298178,259,NULL)
INSERT INTO SalesInvoice VALUES (1393,39834.8737335323,39871.5276849172,256,NULL)
INSERT INTO SalesInvoice VALUES (1394,39836.1594676062,39858.8325428883,595,NULL)
INSERT INTO SalesInvoice VALUES (1395,39836.9759474838,39840.0984829123,286,NULL)
INSERT INTO SalesInvoice VALUES (1396,39837.9428873422,39879.3599214828,466,NULL)
INSERT INTO SalesInvoice VALUES (1397,39838.7858808734,39881.2492523274,425,NULL)
INSERT INTO SalesInvoice VALUES (1398,39839.948049263,39848.105919749,735,NULL)
INSERT INTO SalesInvoice VALUES (1399,39840.8938235967,39842.4576060457,768,NULL)
INSERT INTO SalesInvoice VALUES (1400,39841.1165707152,39862.0685891687,48,NULL)
INSERT INTO SalesInvoice VALUES (1401,39842.3062962732,39847.1832753687,1,NULL)
INSERT INTO SalesInvoice VALUES (1402,39842.9732476802,39847.0272525683,517,NULL)
INSERT INTO SalesInvoice VALUES (1403,39844.2292038051,39884.9952918017,180,NULL)
INSERT INTO SalesInvoice VALUES (1404,39845.4403104706,39882.814987042,334,NULL)
INSERT INTO SalesInvoice VALUES (1405,39846.4426926605,39889.6643866205,673,NULL)
INSERT INTO SalesInvoice VALUES (1406,39846.8380871251,39890.727189943,104,NULL)
INSERT INTO SalesInvoice VALUES (1407,39847.0645485791,39875.9659521912,431,NULL)
INSERT INTO SalesInvoice VALUES (1408,39847.1425947894,39877.4420546667,595,NULL)
INSERT INTO SalesInvoice VALUES (1409,39847.3727331776,39883.9123417125,308,NULL)
INSERT INTO SalesInvoice VALUES (1410,39847.6706175397,39886.4425559449,315,NULL)
INSERT INTO SalesInvoice VALUES (1411,39848.1863960161,39873.4781468115,248,NULL)
INSERT INTO SalesInvoice VALUES (1412,39849.5001193494,39880.5163740327,86,NULL)
INSERT INTO SalesInvoice VALUES (1413,39850.4490456421,39851.6343651134,771,NULL)
INSERT INTO SalesInvoice VALUES (1414,39851.2581607539,39895.7336394114,152,NULL)
INSERT INTO SalesInvoice VALUES (1415,39851.5580473108,39887.7341589844,53,NULL)
INSERT INTO SalesInvoice VALUES (1416,39851.9243814246,39891.5273293153,741,NULL)
INSERT INTO SalesInvoice VALUES (1417,39853.3553670518,39890.3411598755,361,NULL)
INSERT INTO SalesInvoice VALUES (1418,39854.0487103958,39894.2078769436,768,NULL)
INSERT INTO SalesInvoice VALUES (1419,39854.1548609838,39902.2839974476,106,NULL)
INSERT INTO SalesInvoice VALUES (1420,39854.5904250643,39880.4985294101,408,NULL)
INSERT INTO SalesInvoice VALUES (1421,39855.8624321592,39901.4848203979,288,NULL)
INSERT INTO SalesInvoice VALUES (1422,39856.5371107622,39894.5169277349,246,NULL)
INSERT INTO SalesInvoice VALUES (1423,39857.4274858929,39885.483510372,396,NULL)
INSERT INTO SalesInvoice VALUES (1424,39858.3000947506,39864.8012538733,282,NULL)
INSERT INTO SalesInvoice VALUES (1425,39859.1498189858,39872.4025196134,161,NULL)
INSERT INTO SalesInvoice VALUES (1426,39860.0591580422,39888.8818343277,551,NULL)
INSERT INTO SalesInvoice VALUES (1427,39860.5013549206,39894.1336744594,150,NULL)
INSERT INTO SalesInvoice VALUES (1428,39861.4942747132,39884.0125624275,463,NULL)
INSERT INTO SalesInvoice VALUES (1429,39862.1595857599,39898.4705196397,124,NULL)
INSERT INTO SalesInvoice VALUES (1430,39863.0507468055,39904.5980582217,403,NULL)
INSERT INTO SalesInvoice VALUES (1431,39864.1210112135,39903.1160594598,760,NULL)
INSERT INTO SalesInvoice VALUES (1432,39865.5680621522,39893.2375503766,564,NULL)
INSERT INTO SalesInvoice VALUES (1433,39865.8662564324,39885.9843887202,522,NULL)
INSERT INTO SalesInvoice VALUES (1434,39867.0608331513,39887.6556209504,33,NULL)
INSERT INTO SalesInvoice VALUES (1435,39868.4178283863,39887.2963867824,625,NULL)
INSERT INTO SalesInvoice VALUES (1436,39868.9111917069,39901.2672424446,9,NULL)
INSERT INTO SalesInvoice VALUES (1437,39869.7118085448,39903.2860693118,544,NULL)
INSERT INTO SalesInvoice VALUES (1438,39870.2210622457,39872.85573314,400,NULL)
INSERT INTO SalesInvoice VALUES (1439,39871.2456027718,39871.2637254908,194,NULL)
INSERT INTO SalesInvoice VALUES (1440,39872.1270155647,39883.8123252194,114,NULL)
INSERT INTO SalesInvoice VALUES (1441,39873.5103623988,39876.6805492952,600,NULL)
INSERT INTO SalesInvoice VALUES (1442,39873.716534041,39912.5235970382,313,NULL)
INSERT INTO SalesInvoice VALUES (1443,39874.9806776478,39903.9790022356,172,NULL)
INSERT INTO SalesInvoice VALUES (1444,39875.2351240062,39877.290104459,211,NULL)
INSERT INTO SalesInvoice VALUES (1445,39876.4383264447,39879.6373381432,382,NULL)
INSERT INTO SalesInvoice VALUES (1446,39877.5373998254,39926.7522007149,419,NULL)
INSERT INTO SalesInvoice VALUES (1447,39878.1690913684,39901.3792907123,17,NULL)
INSERT INTO SalesInvoice VALUES (1448,39879.3794333612,39913.9065559071,106,NULL)
INSERT INTO SalesInvoice VALUES (1449,39879.8915964654,39881.6833932461,709,NULL)
INSERT INTO SalesInvoice VALUES (1450,39880.8963380488,39908.830160374,235,NULL)
INSERT INTO SalesInvoice VALUES (1451,39881.1922715939,39930.3664545668,33,NULL)
INSERT INTO SalesInvoice VALUES (1452,39882.1049362353,39931.1349824165,681,NULL)
INSERT INTO SalesInvoice VALUES (1453,39882.4579686419,39891.0620471453,509,NULL)
INSERT INTO SalesInvoice VALUES (1454,39883.3579137977,39920.5903474459,633,NULL)
INSERT INTO SalesInvoice VALUES (1455,39883.3650540791,39914.3508142294,86,NULL)
INSERT INTO SalesInvoice VALUES (1456,39883.5070473015,39926.8734380171,600,NULL)
INSERT INTO SalesInvoice VALUES (1457,39884.8987662241,39889.0811061239,433,NULL)
INSERT INTO SalesInvoice VALUES (1458,39885.2804465307,39924.3042628294,42,NULL)
INSERT INTO SalesInvoice VALUES (1459,39886.6658751094,39928.0275499816,490,NULL)
INSERT INTO SalesInvoice VALUES (1460,39887.8370455227,39923.3879543401,84,NULL)
INSERT INTO SalesInvoice VALUES (1461,39887.9329376361,39919.1507500414,14,NULL)
INSERT INTO SalesInvoice VALUES (1462,39889.490738504,39915.8614271385,458,NULL)
INSERT INTO SalesInvoice VALUES (1463,39890.434354982,39894.6279097407,11,NULL)
INSERT INTO SalesInvoice VALUES (1464,39891.1766640418,39917.3004441295,367,NULL)
INSERT INTO SalesInvoice VALUES (1465,39892.2972824134,39918.9915120209,614,NULL)
INSERT INTO SalesInvoice VALUES (1466,39893.6744245939,39917.9761660575,408,NULL)
INSERT INTO SalesInvoice VALUES (1467,39894.7135672325,39911.7974482193,632,NULL)
INSERT INTO SalesInvoice VALUES (1468,39896.1020070484,39932.6734288378,610,NULL)
INSERT INTO SalesInvoice VALUES (1469,39897.2801787604,39911.9216648903,702,NULL)
INSERT INTO SalesInvoice VALUES (1470,39897.492239087,39921.9934673279,179,NULL)
INSERT INTO SalesInvoice VALUES (1471,39899.0137751532,39915.2098839603,209,NULL)
INSERT INTO SalesInvoice VALUES (1472,39899.7594331961,39937.0460245443,468,NULL)
INSERT INTO SalesInvoice VALUES (1473,39900.4748373213,39940.4188405938,747,NULL)
INSERT INTO SalesInvoice VALUES (1474,39901.7696695534,39923.2129719786,281,NULL)
INSERT INTO SalesInvoice VALUES (1475,39903.3427757387,39926.7727214701,186,NULL)
INSERT INTO SalesInvoice VALUES (1476,39904.2403882006,39939.532959644,696,NULL)
INSERT INTO SalesInvoice VALUES (1477,39905.5038258651,39911.9731302027,176,NULL)
INSERT INTO SalesInvoice VALUES (1478,39906.2696338095,39918.9748337826,190,NULL)
INSERT INTO SalesInvoice VALUES (1479,39906.4538967849,39950.8151410263,79,NULL)
INSERT INTO SalesInvoice VALUES (1480,39906.8824494863,39940.5863272961,744,NULL)
INSERT INTO SalesInvoice VALUES (1481,39907.5895454468,39931.6918424492,298,NULL)
INSERT INTO SalesInvoice VALUES (1482,39907.9274949476,39951.0871578605,570,NULL)
INSERT INTO SalesInvoice VALUES (1483,39908.8303562405,39922.3884802705,111,NULL)
INSERT INTO SalesInvoice VALUES (1484,39908.895857302,39949.5889594474,411,NULL)
INSERT INTO SalesInvoice VALUES (1485,39909.0898119285,39950.3015010674,249,NULL)
INSERT INTO SalesInvoice VALUES (1486,39909.2570708918,39913.8518451194,584,NULL)
INSERT INTO SalesInvoice VALUES (1487,39910.1920945051,39914.6031846568,14,NULL)
INSERT INTO SalesInvoice VALUES (1488,39911.5640503077,39937.3617971976,145,NULL)
INSERT INTO SalesInvoice VALUES (1489,39912.4310927597,39950.5411271868,621,NULL)
INSERT INTO SalesInvoice VALUES (1490,39913.4391980796,39934.2298841427,450,NULL)
INSERT INTO SalesInvoice VALUES (1491,39914.5468906756,39953.9059177208,331,NULL)
INSERT INTO SalesInvoice VALUES (1492,39914.9723571527,39927.2999333203,178,NULL)
INSERT INTO SalesInvoice VALUES (1493,39915.0077300748,39918.8721668622,424,NULL)
INSERT INTO SalesInvoice VALUES (1494,39915.9310611445,39962.5760530935,31,NULL)
INSERT INTO SalesInvoice VALUES (1495,39916.5099833552,39921.0230365953,573,NULL)
INSERT INTO SalesInvoice VALUES (1496,39917.123827711,39961.5640860226,464,NULL)
INSERT INTO SalesInvoice VALUES (1497,39918.5776583433,39959.3254252431,393,NULL)
INSERT INTO SalesInvoice VALUES (1498,39919.9748709723,39951.7629530283,704,NULL)
INSERT INTO SalesInvoice VALUES (1499,39921.038147854,39959.8291098353,363,NULL)
INSERT INTO SalesInvoice VALUES (1500,39921.1721321156,39957.4005187688,357,NULL)
INSERT INTO SalesInvoice VALUES (1501,39922.5690576742,39925.2377686564,669,NULL)
INSERT INTO SalesInvoice VALUES (1502,39922.6708479586,39967.5343349139,188,NULL)
INSERT INTO SalesInvoice VALUES (1503,39922.7941710868,39938.6839252485,683,NULL)
INSERT INTO SalesInvoice VALUES (1504,39923.8470146855,39933.1470338192,735,NULL)
INSERT INTO SalesInvoice VALUES (1505,39923.9525441482,39956.1141867751,347,NULL)
INSERT INTO SalesInvoice VALUES (1506,39924.2334276713,39972.3992567884,30,NULL)
INSERT INTO SalesInvoice VALUES (1507,39924.3057295188,39930.4818527335,127,NULL)
INSERT INTO SalesInvoice VALUES (1508,39924.8429457689,39969.0202342117,25,NULL)
INSERT INTO SalesInvoice VALUES (1509,39925.7933820712,39962.3944180113,4,NULL)
INSERT INTO SalesInvoice VALUES (1510,39927.3567788687,39933.558712765,204,NULL)
INSERT INTO SalesInvoice VALUES (1511,39927.5691814712,39960.3840867045,74,NULL)
INSERT INTO SalesInvoice VALUES (1512,39928.7863762618,39969.4140236878,651,NULL)
INSERT INTO SalesInvoice VALUES (1513,39930.3041628216,39931.9145739512,642,NULL)
INSERT INTO SalesInvoice VALUES (1514,39930.740277958,39946.4082858753,682,NULL)
INSERT INTO SalesInvoice VALUES (1515,39931.3286388573,39936.1921063247,763,NULL)
INSERT INTO SalesInvoice VALUES (1516,39931.3765325048,39980.982878118,672,NULL)
INSERT INTO SalesInvoice VALUES (1517,39931.6963005021,39967.7239250772,712,NULL)
INSERT INTO SalesInvoice VALUES (1518,39932.4607972485,39936.1119870808,539,NULL)
INSERT INTO SalesInvoice VALUES (1519,39932.7129234803,39946.6993339877,412,NULL)
INSERT INTO SalesInvoice VALUES (1520,39933.9373144476,39961.0347703026,593,NULL)
INSERT INTO SalesInvoice VALUES (1521,39935.1773917503,39954.0301951309,342,NULL)
INSERT INTO SalesInvoice VALUES (1522,39935.9700408821,39941.4572091011,354,NULL)
INSERT INTO SalesInvoice VALUES (1523,39936.6001074735,39944.7227288468,99,NULL)
INSERT INTO SalesInvoice VALUES (1524,39938.065488765,39948.282632279,612,NULL)
INSERT INTO SalesInvoice VALUES (1525,39938.6498121637,39977.2855260924,32,NULL)
INSERT INTO SalesInvoice VALUES (1526,39940.20341672,39955.115989176,134,NULL)
INSERT INTO SalesInvoice VALUES (1527,39941.5425527755,39953.6679427404,72,NULL)
INSERT INTO SalesInvoice VALUES (1528,39942.4702569899,39968.7046607217,452,NULL)
INSERT INTO SalesInvoice VALUES (1529,39943.697432177,39977.7231407653,501,NULL)
INSERT INTO SalesInvoice VALUES (1530,39944.9889702002,39989.5388080529,656,NULL)
INSERT INTO SalesInvoice VALUES (1531,39945.05449345,39984.2308518572,566,NULL)
INSERT INTO SalesInvoice VALUES (1532,39945.1711948851,39982.3438982821,89,NULL)
INSERT INTO SalesInvoice VALUES (1533,39945.7462005923,39984.0935184854,588,NULL)
INSERT INTO SalesInvoice VALUES (1534,39945.9536806889,39967.149257843,651,NULL)
INSERT INTO SalesInvoice VALUES (1535,39946.8810425765,39948.2725202725,756,NULL)
INSERT INTO SalesInvoice VALUES (1536,39947.9977613436,39995.6146004827,553,NULL)
INSERT INTO SalesInvoice VALUES (1537,39949.2602988876,39953.8964078287,51,NULL)
INSERT INTO SalesInvoice VALUES (1538,39950.7205907367,39953.2684633524,563,NULL)
INSERT INTO SalesInvoice VALUES (1539,39951.6023116552,39981.0447592665,725,NULL)
INSERT INTO SalesInvoice VALUES (1540,39952.1456192397,39980.4222343064,56,NULL)
INSERT INTO SalesInvoice VALUES (1541,39953.3620981928,39996.9130506445,441,NULL)
INSERT INTO SalesInvoice VALUES (1542,39954.7285016781,39988.422917362,349,NULL)
INSERT INTO SalesInvoice VALUES (1543,39955.3064175731,39959.049606495,87,NULL)
INSERT INTO SalesInvoice VALUES (1544,39955.5296985039,40003.1146513283,681,NULL)
INSERT INTO SalesInvoice VALUES (1545,39956.2221212902,39995.327939484,549,NULL)
INSERT INTO SalesInvoice VALUES (1546,39957.7908842559,39967.7552330628,219,NULL)
INSERT INTO SalesInvoice VALUES (1547,39957.865136893,39994.0538926311,403,NULL)
INSERT INTO SalesInvoice VALUES (1548,39958.1511192066,39971.7005248539,663,NULL)
INSERT INTO SalesInvoice VALUES (1549,39958.4405565964,39969.1933290824,548,NULL)
INSERT INTO SalesInvoice VALUES (1550,39959.2655408666,39971.5318833633,632,NULL)
INSERT INTO SalesInvoice VALUES (1551,39960.3685960278,40000.0213804095,505,NULL)
INSERT INTO SalesInvoice VALUES (1552,39960.3689934139,40007.0263937128,388,NULL)
INSERT INTO SalesInvoice VALUES (1553,39960.8030130256,39975.7796778549,139,NULL)
INSERT INTO SalesInvoice VALUES (1554,39962.1473653298,40011.3838503627,765,NULL)
INSERT INTO SalesInvoice VALUES (1555,39962.2114114238,40002.5833258062,744,NULL)
INSERT INTO SalesInvoice VALUES (1556,39962.5403558311,39981.2557378477,443,NULL)
INSERT INTO SalesInvoice VALUES (1557,39963.2393507435,40009.0805405508,388,NULL)
INSERT INTO SalesInvoice VALUES (1558,39964.391485589,39990.9879095292,601,NULL)
INSERT INTO SalesInvoice VALUES (1559,39964.5702476672,39973.4548279144,399,NULL)
INSERT INTO SalesInvoice VALUES (1560,39965.7582580056,40014.067359384,90,NULL)
INSERT INTO SalesInvoice VALUES (1561,39966.3594842625,39969.565006896,704,NULL)
INSERT INTO SalesInvoice VALUES (1562,39967.5002858436,39988.21331382,165,NULL)
INSERT INTO SalesInvoice VALUES (1563,39968.7937028503,40006.7876323439,45,NULL)
INSERT INTO SalesInvoice VALUES (1564,39969.0483008182,40011.1563240825,278,NULL)
INSERT INTO SalesInvoice VALUES (1565,39969.988822228,39970.9281525904,382,NULL)
INSERT INTO SalesInvoice VALUES (1566,39970.8844805832,40004.4948700723,30,NULL)
INSERT INTO SalesInvoice VALUES (1567,39972.2586857694,39981.760804493,711,NULL)
INSERT INTO SalesInvoice VALUES (1568,39972.5650923251,40006.9422965342,213,NULL)
INSERT INTO SalesInvoice VALUES (1569,39972.8447914886,40015.4559933943,665,NULL)
INSERT INTO SalesInvoice VALUES (1570,39974.1155200047,39992.3170306742,413,NULL)
INSERT INTO SalesInvoice VALUES (1571,39974.7948924034,39989.0967407204,149,NULL)
INSERT INTO SalesInvoice VALUES (1572,39976.0560199082,39989.7864004453,418,NULL)
INSERT INTO SalesInvoice VALUES (1573,39976.0639824824,40000.6192044989,525,NULL)
INSERT INTO SalesInvoice VALUES (1574,39976.8854266318,39980.9520700512,546,NULL)
INSERT INTO SalesInvoice VALUES (1575,39976.9737482872,40011.9864427173,103,NULL)
INSERT INTO SalesInvoice VALUES (1576,39977.3756777956,39990.164913095,427,NULL)
INSERT INTO SalesInvoice VALUES (1577,39978.0918225899,39980.7148061261,658,NULL)
INSERT INTO SalesInvoice VALUES (1578,39978.1860990133,40000.924027294,401,NULL)
INSERT INTO SalesInvoice VALUES (1579,39979.3615120943,39989.7388794352,681,NULL)
INSERT INTO SalesInvoice VALUES (1580,39980.8663798268,39992.9303743713,614,NULL)
INSERT INTO SalesInvoice VALUES (1581,39981.9424201824,40002.130345947,770,NULL)
INSERT INTO SalesInvoice VALUES (1582,39982.0204048555,40005.2108331044,8,NULL)
INSERT INTO SalesInvoice VALUES (1583,39983.0480546342,39999.2579524935,392,NULL)
INSERT INTO SalesInvoice VALUES (1584,39983.1251821043,39993.1974157879,238,NULL)
INSERT INTO SalesInvoice VALUES (1585,39984.6132185662,40024.6573403914,726,NULL)
INSERT INTO SalesInvoice VALUES (1586,39985.795320902,40020.9833720935,214,NULL)
INSERT INTO SalesInvoice VALUES (1587,39987.3215347326,40028.0168876186,259,NULL)
INSERT INTO SalesInvoice VALUES (1588,39987.4130961354,40005.1247112363,534,NULL)
INSERT INTO SalesInvoice VALUES (1589,39988.555622832,40037.6359999803,629,NULL)
INSERT INTO SalesInvoice VALUES (1590,39989.3890126028,40006.9933476208,728,NULL)
INSERT INTO SalesInvoice VALUES (1591,39990.6682887385,40005.5821868605,347,NULL)
INSERT INTO SalesInvoice VALUES (1592,39991.211108409,39995.6651452183,317,NULL)
INSERT INTO SalesInvoice VALUES (1593,39991.8623109329,40000.6964117565,442,NULL)
INSERT INTO SalesInvoice VALUES (1594,39993.1060132985,40036.1193872569,458,NULL)
INSERT INTO SalesInvoice VALUES (1595,39994.068937999,40027.0624518722,558,NULL)
INSERT INTO SalesInvoice VALUES (1596,39995.3027084908,40008.0082459969,255,NULL)
INSERT INTO SalesInvoice VALUES (1597,39996.8444526595,40025.4172791935,154,NULL)
INSERT INTO SalesInvoice VALUES (1598,39998.0362362145,40039.1361243141,654,NULL)
INSERT INTO SalesInvoice VALUES (1599,39999.1941345993,40020.2652229798,547,NULL)
INSERT INTO SalesInvoice VALUES (1600,39999.5521142658,40034.2183756021,195,NULL)
INSERT INTO SalesInvoice VALUES (1601,40000.0811846026,40011.2592979446,257,NULL)
INSERT INTO SalesInvoice VALUES (1602,40001.0450209476,40019.8695387742,81,NULL)
INSERT INTO SalesInvoice VALUES (1603,40001.3110652594,40013.4232246112,338,NULL)
INSERT INTO SalesInvoice VALUES (1604,40002.0561817605,40028.0133579184,710,NULL)
INSERT INTO SalesInvoice VALUES (1605,40002.8016041512,40016.810840984,487,NULL)
INSERT INTO SalesInvoice VALUES (1606,40002.8613066714,40037.8533151369,441,NULL)
INSERT INTO SalesInvoice VALUES (1607,40004.2383013201,40005.1155110068,121,NULL)
INSERT INTO SalesInvoice VALUES (1608,40004.9394139322,40010.1084073921,352,NULL)
INSERT INTO SalesInvoice VALUES (1609,40006.3100534991,40020.9432119906,15,NULL)
INSERT INTO SalesInvoice VALUES (1610,40007.7295847792,40009.7746535086,368,NULL)
INSERT INTO SalesInvoice VALUES (1611,40009.2043380358,40040.8947787778,392,NULL)
INSERT INTO SalesInvoice VALUES (1612,40009.554693548,40015.4431069499,601,NULL)
INSERT INTO SalesInvoice VALUES (1613,40009.7519740271,40029.0699665867,298,NULL)
INSERT INTO SalesInvoice VALUES (1614,40010.7292067517,40058.2539324674,87,NULL)
INSERT INTO SalesInvoice VALUES (1615,40010.7614478444,40023.1449324625,289,NULL)
INSERT INTO SalesInvoice VALUES (1616,40011.2206401673,40055.679675274,683,NULL)
INSERT INTO SalesInvoice VALUES (1617,40011.8575408526,40045.4794692402,370,NULL)
INSERT INTO SalesInvoice VALUES (1618,40012.4174828587,40056.0796610831,686,NULL)
INSERT INTO SalesInvoice VALUES (1619,40013.9730020482,40055.7191871815,634,NULL)
INSERT INTO SalesInvoice VALUES (1620,40015.3626374233,40036.8970731836,313,NULL)
INSERT INTO SalesInvoice VALUES (1621,40016.8742192413,40039.6593233769,510,NULL)
INSERT INTO SalesInvoice VALUES (1622,40017.0006320372,40042.8534518163,69,NULL)
INSERT INTO SalesInvoice VALUES (1623,40018.2257584809,40028.3546042985,657,NULL)
INSERT INTO SalesInvoice VALUES (1624,40019.2132645661,40048.3971141382,235,NULL)
INSERT INTO SalesInvoice VALUES (1625,40020.4664589364,40034.2471650245,519,NULL)
INSERT INTO SalesInvoice VALUES (1626,40021.5116462068,40052.6212890379,494,NULL)
INSERT INTO SalesInvoice VALUES (1627,40022.2157569408,40043.1243728044,607,NULL)
INSERT INTO SalesInvoice VALUES (1628,40023.4361776392,40046.5088730045,284,NULL)
INSERT INTO SalesInvoice VALUES (1629,40024.0489965612,40057.8000238586,149,NULL)
INSERT INTO SalesInvoice VALUES (1630,40025.1124228044,40039.8071170927,76,NULL)
INSERT INTO SalesInvoice VALUES (1631,40025.7108865363,40064.5128807221,544,NULL)
INSERT INTO SalesInvoice VALUES (1632,40026.4538353691,40070.4684518868,584,NULL)
INSERT INTO SalesInvoice VALUES (1633,40026.9211019843,40032.0569389855,429,NULL)
INSERT INTO SalesInvoice VALUES (1634,40027.7217355707,40064.7663111388,342,NULL)
INSERT INTO SalesInvoice VALUES (1635,40028.2991154996,40071.7125697722,139,NULL)
INSERT INTO SalesInvoice VALUES (1636,40028.5220834397,40048.9726779231,373,NULL)
INSERT INTO SalesInvoice VALUES (1637,40029.6480568773,40031.9936418737,181,NULL)
INSERT INTO SalesInvoice VALUES (1638,40030.6735616061,40043.3294897467,754,NULL)
INSERT INTO SalesInvoice VALUES (1639,40031.8533404301,40056.7388007921,614,NULL)
INSERT INTO SalesInvoice VALUES (1640,40033.0705306403,40058.3631455204,492,NULL)
INSERT INTO SalesInvoice VALUES (1641,40033.3680402134,40044.035874132,156,NULL)
INSERT INTO SalesInvoice VALUES (1642,40033.4566001668,40058.7942792328,364,NULL)
INSERT INTO SalesInvoice VALUES (1643,40034.1266444239,40083.7039429428,428,NULL)
INSERT INTO SalesInvoice VALUES (1644,40034.3898678259,40054.46665564,400,NULL)
INSERT INTO SalesInvoice VALUES (1645,40035.6920014747,40042.4116545815,729,NULL)
INSERT INTO SalesInvoice VALUES (1646,40036.8071098661,40063.2580163825,636,NULL)
INSERT INTO SalesInvoice VALUES (1647,40037.6517790692,40068.8848180777,710,NULL)
INSERT INTO SalesInvoice VALUES (1648,40038.1669525231,40074.2618183868,322,NULL)
INSERT INTO SalesInvoice VALUES (1649,40039.6915870484,40072.0717549341,118,NULL)
INSERT INTO SalesInvoice VALUES (1650,40039.9838176174,40072.0085015189,771,NULL)
INSERT INTO SalesInvoice VALUES (1651,40040.920768111,40054.8125231244,688,NULL)
INSERT INTO SalesInvoice VALUES (1652,40041.2366218366,40061.1601068674,460,NULL)
INSERT INTO SalesInvoice VALUES (1653,40042.4577026281,40064.7707343578,680,NULL)
INSERT INTO SalesInvoice VALUES (1654,40043.7122244367,40065.1308841699,729,NULL)
INSERT INTO SalesInvoice VALUES (1655,40044.2026613643,40068.807229203,421,NULL)
INSERT INTO SalesInvoice VALUES (1656,40045.1612628649,40065.7267651601,155,NULL)
INSERT INTO SalesInvoice VALUES (1657,40046.1047431278,40064.5306129109,410,NULL)
INSERT INTO SalesInvoice VALUES (1658,40047.0280550456,40049.0501106364,475,NULL)
INSERT INTO SalesInvoice VALUES (1659,40047.218434713,40080.4232067985,741,NULL)
INSERT INTO SalesInvoice VALUES (1660,40048.5657302577,40058.7787974611,187,NULL)
INSERT INTO SalesInvoice VALUES (1661,40048.6161001655,40090.0956409076,532,NULL)
INSERT INTO SalesInvoice VALUES (1662,40049.8485053289,40051.8757819587,524,NULL)
INSERT INTO SalesInvoice VALUES (1663,40050.0358872421,40062.3902442055,382,NULL)
INSERT INTO SalesInvoice VALUES (1664,40050.2917148168,40065.5702700922,198,NULL)
INSERT INTO SalesInvoice VALUES (1665,40051.6652341096,40054.2319926492,331,NULL)
INSERT INTO SalesInvoice VALUES (1666,40053.1306159116,40094.9157245133,116,NULL)
INSERT INTO SalesInvoice VALUES (1667,40053.4207595949,40059.3466070752,715,NULL)
INSERT INTO SalesInvoice VALUES (1668,40054.0154800617,40060.7171569959,163,NULL)
INSERT INTO SalesInvoice VALUES (1669,40055.1016677501,40058.9127415043,579,NULL)
INSERT INTO SalesInvoice VALUES (1670,40056.1530200561,40093.9802038706,245,NULL)
INSERT INTO SalesInvoice VALUES (1671,40056.3514161533,40098.5897850618,558,NULL)
INSERT INTO SalesInvoice VALUES (1672,40056.7499925427,40061.1946694682,180,NULL)
INSERT INTO SalesInvoice VALUES (1673,40058.0666612871,40072.9415480447,625,NULL)
INSERT INTO SalesInvoice VALUES (1674,40059.6456191176,40086.9339858423,701,NULL)
INSERT INTO SalesInvoice VALUES (1675,40060.967630134,40097.6673777903,120,NULL)
INSERT INTO SalesInvoice VALUES (1676,40062.5407653987,40074.7728012614,455,NULL)
INSERT INTO SalesInvoice VALUES (1677,40062.9528562575,40076.3436146112,301,NULL)
INSERT INTO SalesInvoice VALUES (1678,40063.6128798001,40105.1271627493,709,NULL)
INSERT INTO SalesInvoice VALUES (1679,40064.8244771826,40072.2047574974,690,NULL)
INSERT INTO SalesInvoice VALUES (1680,40064.9128825763,40089.9719529642,206,NULL)
INSERT INTO SalesInvoice VALUES (1681,40065.1404312817,40104.9341273165,173,NULL)
INSERT INTO SalesInvoice VALUES (1682,40066.4428069017,40095.641775855,45,NULL)
INSERT INTO SalesInvoice VALUES (1683,40066.7304991912,40080.5079129687,259,NULL)
INSERT INTO SalesInvoice VALUES (1684,40068.2452620547,40102.9702775891,476,NULL)
INSERT INTO SalesInvoice VALUES (1685,40069.0009194629,40080.1318679957,415,NULL)
INSERT INTO SalesInvoice VALUES (1686,40070.2990684916,40076.2711601737,125,NULL)
INSERT INTO SalesInvoice VALUES (1687,40071.6177369454,40086.8792657946,424,NULL)
INSERT INTO SalesInvoice VALUES (1688,40071.7788356125,40110.0611964425,665,NULL)
INSERT INTO SalesInvoice VALUES (1689,40072.0822087378,40084.8790675564,314,NULL)
INSERT INTO SalesInvoice VALUES (1690,40072.3454182358,40119.6840084481,250,NULL)
INSERT INTO SalesInvoice VALUES (1691,40073.4937770263,40098.6024555925,499,NULL)
INSERT INTO SalesInvoice VALUES (1692,40074.8622863096,40117.5239469937,192,NULL)
INSERT INTO SalesInvoice VALUES (1693,40075.8965539869,40125.6739640462,755,NULL)
INSERT INTO SalesInvoice VALUES (1694,40077.3626882699,40114.0067476423,72,NULL)
INSERT INTO SalesInvoice VALUES (1695,40078.3263814732,40103.9291357322,158,NULL)
INSERT INTO SalesInvoice VALUES (1696,40078.9844285687,40098.3454466379,271,NULL)
INSERT INTO SalesInvoice VALUES (1697,40080.0343791165,40112.8322844731,336,NULL)
INSERT INTO SalesInvoice VALUES (1698,40081.2406334459,40097.0010096785,553,NULL)
INSERT INTO SalesInvoice VALUES (1699,40081.4318765231,40100.2018102818,554,NULL)
INSERT INTO SalesInvoice VALUES (1700,40082.4029447606,40083.3692713038,641,NULL)
INSERT INTO SalesInvoice VALUES (1701,40082.9282178134,40129.0741067588,619,NULL)
INSERT INTO SalesInvoice VALUES (1702,40083.3784826448,40132.650099895,338,NULL)
INSERT INTO SalesInvoice VALUES (1703,40084.9246464019,40109.4157322465,159,NULL)
INSERT INTO SalesInvoice VALUES (1704,40085.1578608934,40116.4192644038,468,NULL)
INSERT INTO SalesInvoice VALUES (1705,40085.9444911693,40120.7886393536,529,NULL)
INSERT INTO SalesInvoice VALUES (1706,40086.4481890111,40124.0474165359,546,NULL)
INSERT INTO SalesInvoice VALUES (1707,40086.8641840419,40123.4791236367,156,NULL)
INSERT INTO SalesInvoice VALUES (1708,40088.40161739,40102.5534701044,248,NULL)
INSERT INTO SalesInvoice VALUES (1709,40088.8703954367,40126.2958407364,575,NULL)
INSERT INTO SalesInvoice VALUES (1710,40090.2365533747,40090.9171431711,208,NULL)
INSERT INTO SalesInvoice VALUES (1711,40090.6776615632,40127.329157884,247,NULL)
INSERT INTO SalesInvoice VALUES (1712,40091.0904538761,40136.2943824617,235,NULL)
INSERT INTO SalesInvoice VALUES (1713,40091.3132554845,40103.9049255437,497,NULL)
INSERT INTO SalesInvoice VALUES (1714,40092.022174509,40128.7425289449,27,NULL)
INSERT INTO SalesInvoice VALUES (1715,40092.3207451238,40113.2541769712,582,NULL)
INSERT INTO SalesInvoice VALUES (1716,40093.6997111539,40113.1137972136,126,NULL)
INSERT INTO SalesInvoice VALUES (1717,40094.2151307776,40140.2953976409,500,NULL)
INSERT INTO SalesInvoice VALUES (1718,40095.4451303069,40110.8480490977,259,NULL)
INSERT INTO SalesInvoice VALUES (1719,40096.6217276911,40130.3239401497,248,NULL)
INSERT INTO SalesInvoice VALUES (1720,40097.2799901126,40130.7468514742,526,NULL)
INSERT INTO SalesInvoice VALUES (1721,40098.2254655263,40119.7197699672,235,NULL)
INSERT INTO SalesInvoice VALUES (1722,40098.7719482281,40140.7476758166,448,NULL)
INSERT INTO SalesInvoice VALUES (1723,40099.1300440004,40113.4148703462,51,NULL)
INSERT INTO SalesInvoice VALUES (1724,40100.6375255771,40150.4535803945,249,NULL)
INSERT INTO SalesInvoice VALUES (1725,40102.1980203129,40113.4071933556,289,NULL)
INSERT INTO SalesInvoice VALUES (1726,40103.5751799766,40138.3672664943,147,NULL)
INSERT INTO SalesInvoice VALUES (1727,40104.6116593325,40145.4845306784,34,NULL)
INSERT INTO SalesInvoice VALUES (1728,40105.9728799869,40147.84590327,269,NULL)
INSERT INTO SalesInvoice VALUES (1729,40107.0844209993,40127.2060692523,143,NULL)
INSERT INTO SalesInvoice VALUES (1730,40107.1609203069,40153.165075536,761,NULL)
INSERT INTO SalesInvoice VALUES (1731,40108.7176686589,40149.8858736462,301,NULL)
INSERT INTO SalesInvoice VALUES (1732,40108.8193549074,40124.5583899047,517,NULL)
INSERT INTO SalesInvoice VALUES (1733,40110.3378963935,40121.8890074044,774,NULL)
INSERT INTO SalesInvoice VALUES (1734,40111.217898748,40112.5696199081,378,NULL)
INSERT INTO SalesInvoice VALUES (1735,40112.2754012314,40119.3856299606,31,NULL)
INSERT INTO SalesInvoice VALUES (1736,40113.2318957702,40143.6265449379,252,NULL)
INSERT INTO SalesInvoice VALUES (1737,40114.364339982,40149.5703518416,615,NULL)
INSERT INTO SalesInvoice VALUES (1738,40114.7621185976,40158.9001767872,751,NULL)
INSERT INTO SalesInvoice VALUES (1739,40116.1386764873,40165.2080388578,544,NULL)
INSERT INTO SalesInvoice VALUES (1740,40116.1745117387,40141.0407968291,356,NULL)
INSERT INTO SalesInvoice VALUES (1741,40117.6626408441,40129.8102253554,342,NULL)
INSERT INTO SalesInvoice VALUES (1742,40117.7080495248,40127.074825076,679,NULL)
INSERT INTO SalesInvoice VALUES (1743,40118.3946055246,40158.4582797112,669,NULL)
INSERT INTO SalesInvoice VALUES (1744,40119.0766773221,40161.1102398756,643,NULL)
INSERT INTO SalesInvoice VALUES (1745,40120.6274915077,40161.0909939666,248,NULL)
INSERT INTO SalesInvoice VALUES (1746,40121.5667486147,40154.3826211752,234,NULL)
INSERT INTO SalesInvoice VALUES (1747,40122.3067404186,40170.2474384258,9,NULL)
INSERT INTO SalesInvoice VALUES (1748,40123.548938864,40134.0507486733,512,NULL)
INSERT INTO SalesInvoice VALUES (1749,40123.8616283354,40138.1263191382,709,NULL)
INSERT INTO SalesInvoice VALUES (1750,40123.8759942314,40170.3709532472,98,NULL)
INSERT INTO SalesInvoice VALUES (1751,40124.0972721266,40138.1273438631,48,NULL)
INSERT INTO SalesInvoice VALUES (1752,40125.3778844596,40158.8202657799,641,NULL)
INSERT INTO SalesInvoice VALUES (1753,40126.7420308402,40138.7268348172,713,NULL)
INSERT INTO SalesInvoice VALUES (1754,40128.1704310873,40150.7272237466,742,NULL)
INSERT INTO SalesInvoice VALUES (1755,40128.5447956137,40172.9484406394,133,NULL)
INSERT INTO SalesInvoice VALUES (1756,40128.8543539047,40134.576554357,574,NULL)
INSERT INTO SalesInvoice VALUES (1757,40129.7685330377,40157.5147119492,664,NULL)
INSERT INTO SalesInvoice VALUES (1758,40130.6257105326,40149.391805756,131,NULL)
INSERT INTO SalesInvoice VALUES (1759,40132.1134335881,40139.8370852452,24,NULL)
INSERT INTO SalesInvoice VALUES (1760,40132.1739920081,40136.2286389773,296,NULL)
INSERT INTO SalesInvoice VALUES (1761,40133.1351792454,40168.2308447023,125,NULL)
INSERT INTO SalesInvoice VALUES (1762,40134.1089922991,40139.2107413066,286,NULL)
INSERT INTO SalesInvoice VALUES (1763,40134.2592983944,40139.7825634224,335,NULL)
INSERT INTO SalesInvoice VALUES (1764,40134.8245863788,40144.9947942349,344,NULL)
INSERT INTO SalesInvoice VALUES (1765,40135.3791870846,40172.0528864016,444,NULL)
INSERT INTO SalesInvoice VALUES (1766,40136.9402989966,40142.988187218,422,NULL)
INSERT INTO SalesInvoice VALUES (1767,40138.221624331,40151.1740397249,369,NULL)
INSERT INTO SalesInvoice VALUES (1768,40138.6214067812,40169.2494887554,571,NULL)
INSERT INTO SalesInvoice VALUES (1769,40139.5739783524,40150.5988914834,55,NULL)
INSERT INTO SalesInvoice VALUES (1770,40139.6649651651,40167.2561560361,506,NULL)
INSERT INTO SalesInvoice VALUES (1771,40140.3891618235,40176.0299978244,348,NULL)
INSERT INTO SalesInvoice VALUES (1772,40141.5319738222,40168.5760112235,682,NULL)
INSERT INTO SalesInvoice VALUES (1773,40142.0703189924,40165.5986158372,652,NULL)
INSERT INTO SalesInvoice VALUES (1774,40143.596342095,40147.6209593169,335,NULL)
INSERT INTO SalesInvoice VALUES (1775,40144.4814337713,40193.1807305473,239,NULL)
INSERT INTO SalesInvoice VALUES (1776,40144.9061099086,40186.4457547797,105,NULL)
INSERT INTO SalesInvoice VALUES (1777,40145.7267626949,40189.7952099321,297,NULL)
INSERT INTO SalesInvoice VALUES (1778,40146.7952504913,40174.9769543424,561,NULL)
INSERT INTO SalesInvoice VALUES (1779,40147.5713607576,40184.7245735359,712,NULL)
INSERT INTO SalesInvoice VALUES (1780,40148.4913712177,40192.1041554253,319,NULL)
INSERT INTO SalesInvoice VALUES (1781,40149.4675502306,40195.2892184083,125,NULL)
INSERT INTO SalesInvoice VALUES (1782,40150.3594295678,40188.9346182331,499,NULL)
INSERT INTO SalesInvoice VALUES (1783,40151.9410073215,40160.3341470235,596,NULL)
INSERT INTO SalesInvoice VALUES (1784,40153.0254002327,40184.4282882909,494,NULL)
INSERT INTO SalesInvoice VALUES (1785,40154.3810330817,40167.3639645968,351,NULL)
INSERT INTO SalesInvoice VALUES (1786,40154.8807548866,40161.523338614,599,NULL)
INSERT INTO SalesInvoice VALUES (1787,40155.2531416576,40168.0353308928,223,NULL)
INSERT INTO SalesInvoice VALUES (1788,40155.8697519397,40157.832774119,198,NULL)
INSERT INTO SalesInvoice VALUES (1789,40156.8094462113,40191.5366712853,286,NULL)
INSERT INTO SalesInvoice VALUES (1790,40156.8477651481,40188.7997059114,266,NULL)
INSERT INTO SalesInvoice VALUES (1791,40157.530595225,40178.2993244322,403,NULL)
INSERT INTO SalesInvoice VALUES (1792,40157.8679250679,40162.0528422488,217,NULL)
INSERT INTO SalesInvoice VALUES (1793,40157.893395344,40177.2846019375,745,NULL)
INSERT INTO SalesInvoice VALUES (1794,40159.2356517853,40176.6136761905,346,NULL)
INSERT INTO SalesInvoice VALUES (1795,40160.2918557614,40205.287104856,184,NULL)
INSERT INTO SalesInvoice VALUES (1796,40161.2707347611,40161.7344202305,113,NULL)
INSERT INTO SalesInvoice VALUES (1797,40162.4676528779,40197.7966145018,697,NULL)
INSERT INTO SalesInvoice VALUES (1798,40163.1523868814,40179.552036327,151,NULL)
INSERT INTO SalesInvoice VALUES (1799,40164.3530883946,40172.5766268658,123,NULL)
INSERT INTO SalesInvoice VALUES (1800,40165.904635585,40186.7335210639,40,NULL)
INSERT INTO SalesInvoice VALUES (1801,40167.4495043638,40199.7477501638,396,NULL)
INSERT INTO SalesInvoice VALUES (1802,40168.6038505535,40204.9649155977,747,NULL)
INSERT INTO SalesInvoice VALUES (1803,40169.4537066532,40179.4286651159,452,NULL)
INSERT INTO SalesInvoice VALUES (1804,40170.9764211689,40200.3840433925,490,NULL)
INSERT INTO SalesInvoice VALUES (1805,40171.0973837191,40185.83972248,498,NULL)
INSERT INTO SalesInvoice VALUES (1806,40171.7724804325,40193.2656066084,564,NULL)
INSERT INTO SalesInvoice VALUES (1807,40173.3335405515,40196.3452001284,125,NULL)
INSERT INTO SalesInvoice VALUES (1808,40174.5661089608,40175.3635086429,24,NULL)
INSERT INTO SalesInvoice VALUES (1809,40175.8524183805,40180.8819733276,242,NULL)
INSERT INTO SalesInvoice VALUES (1810,40176.8712661208,40190.3644143382,581,NULL)
INSERT INTO SalesInvoice VALUES (1811,40177.9179918103,40224.5781764542,608,NULL)
INSERT INTO SalesInvoice VALUES (1812,40178.2789268904,40194.8154184878,181,NULL)
INSERT INTO SalesInvoice VALUES (1813,40179.0152543263,40182.8231024416,748,NULL)
INSERT INTO SalesInvoice VALUES (1814,40179.3874777245,40218.7351981894,94,NULL)
INSERT INTO SalesInvoice VALUES (1815,40180.7006775495,40183.7762534474,527,NULL)
INSERT INTO SalesInvoice VALUES (1816,40181.7276481936,40186.2533731021,553,NULL)
INSERT INTO SalesInvoice VALUES (1817,40182.6022874934,40209.4481960194,543,NULL)
INSERT INTO SalesInvoice VALUES (1818,40183.3441885246,40184.0589998776,531,NULL)
INSERT INTO SalesInvoice VALUES (1819,40184.5345066328,40189.803126933,228,NULL)
INSERT INTO SalesInvoice VALUES (1820,40184.7839054102,40214.2498117791,626,NULL)
INSERT INTO SalesInvoice VALUES (1821,40185.5567071194,40207.4539666771,431,NULL)
INSERT INTO SalesInvoice VALUES (1822,40185.7343232169,40212.8015901376,405,NULL)
INSERT INTO SalesInvoice VALUES (1823,40186.3376734394,40231.2886902612,150,NULL)
INSERT INTO SalesInvoice VALUES (1824,40187.823684922,40214.6411101732,602,NULL)
INSERT INTO SalesInvoice VALUES (1825,40189.3210960121,40232.8355347643,456,NULL)
INSERT INTO SalesInvoice VALUES (1826,40190.8875559457,40229.463183035,760,NULL)
INSERT INTO SalesInvoice VALUES (1827,40191.5262353113,40197.5820310149,54,NULL)
INSERT INTO SalesInvoice VALUES (1828,40192.692809432,40208.3686349315,200,NULL)
INSERT INTO SalesInvoice VALUES (1829,40192.9047715689,40241.9635607802,746,NULL)
INSERT INTO SalesInvoice VALUES (1830,40193.9690919548,40215.0371033577,340,NULL)
INSERT INTO SalesInvoice VALUES (1831,40194.0723809589,40194.3362849221,745,NULL)
INSERT INTO SalesInvoice VALUES (1832,40195.2346662969,40198.7432735861,582,NULL)
INSERT INTO SalesInvoice VALUES (1833,40195.6277672261,40225.3103599946,242,NULL)
INSERT INTO SalesInvoice VALUES (1834,40195.9473033895,40206.9249555477,369,NULL)
INSERT INTO SalesInvoice VALUES (1835,40197.3610999225,40215.0022656614,562,NULL)
INSERT INTO SalesInvoice VALUES (1836,40198.5412005264,40199.9713533978,199,NULL)
INSERT INTO SalesInvoice VALUES (1837,40198.8057301132,40206.0744581666,86,NULL)
INSERT INTO SalesInvoice VALUES (1838,40199.66691356,40224.790858832,274,NULL)
INSERT INTO SalesInvoice VALUES (1839,40199.8341392347,40223.7516592766,424,NULL)
INSERT INTO SalesInvoice VALUES (1840,40200.6166792191,40208.8282834622,651,NULL)
INSERT INTO SalesInvoice VALUES (1841,40200.8381821545,40214.4623612958,702,NULL)
INSERT INTO SalesInvoice VALUES (1842,40201.2112748429,40201.6224780468,190,NULL)
INSERT INTO SalesInvoice VALUES (1843,40201.8003008755,40222.4516705428,363,NULL)
INSERT INTO SalesInvoice VALUES (1844,40202.1927369437,40237.2224217077,341,NULL)
INSERT INTO SalesInvoice VALUES (1845,40203.7088709138,40210.9655100204,507,NULL)
INSERT INTO SalesInvoice VALUES (1846,40204.9625863807,40234.2894383775,597,NULL)
INSERT INTO SalesInvoice VALUES (1847,40205.4238569716,40252.3830338142,172,NULL)
INSERT INTO SalesInvoice VALUES (1848,40206.2188558436,40236.6036430701,306,NULL)
INSERT INTO SalesInvoice VALUES (1849,40207.5570066519,40221.7278178653,19,NULL)
INSERT INTO SalesInvoice VALUES (1850,40208.6202876256,40256.146171798,382,NULL)
INSERT INTO SalesInvoice VALUES (1851,40209.5733633136,40220.7076718836,478,NULL)
INSERT INTO SalesInvoice VALUES (1852,40211.0545910536,40257.4707099946,88,NULL)
INSERT INTO SalesInvoice VALUES (1853,40211.1200363464,40252.4802400739,130,NULL)
INSERT INTO SalesInvoice VALUES (1854,40211.5859621437,40244.9234627711,90,NULL)
INSERT INTO SalesInvoice VALUES (1855,40212.6089082197,40254.2123237789,334,NULL)
INSERT INTO SalesInvoice VALUES (1856,40213.2156664057,40262.2477864295,307,NULL)
INSERT INTO SalesInvoice VALUES (1857,40214.5455445896,40264.3254466833,34,NULL)
INSERT INTO SalesInvoice VALUES (1858,40214.9257900899,40260.0419460132,568,NULL)
INSERT INTO SalesInvoice VALUES (1859,40215.7580982365,40254.3004214383,106,NULL)
INSERT INTO SalesInvoice VALUES (1860,40217.3048574801,40236.3025743391,141,NULL)
INSERT INTO SalesInvoice VALUES (1861,40217.3622909074,40255.1431026303,154,NULL)
INSERT INTO SalesInvoice VALUES (1862,40217.9902578901,40243.4214253643,719,NULL)
INSERT INTO SalesInvoice VALUES (1863,40219.4394878183,40246.2804198969,137,NULL)
INSERT INTO SalesInvoice VALUES (1864,40220.702885631,40253.1149564324,451,NULL)
INSERT INTO SalesInvoice VALUES (1865,40221.1849501224,40259.5688682425,348,NULL)
INSERT INTO SalesInvoice VALUES (1866,40222.3938482344,40228.9569456558,673,NULL)
INSERT INTO SalesInvoice VALUES (1867,40223.3720219763,40265.1253126367,722,NULL)
INSERT INTO SalesInvoice VALUES (1868,40224.5843518254,40233.2797285927,83,NULL)
INSERT INTO SalesInvoice VALUES (1869,40224.6534732175,40229.0846260218,588,NULL)
INSERT INTO SalesInvoice VALUES (1870,40225.0337672147,40251.8398576666,461,NULL)
INSERT INTO SalesInvoice VALUES (1871,40226.0646385713,40268.3627562227,627,NULL)
INSERT INTO SalesInvoice VALUES (1872,40227.2648766512,40247.3886340165,49,NULL)
INSERT INTO SalesInvoice VALUES (1873,40227.4730459065,40229.136586299,534,NULL)
INSERT INTO SalesInvoice VALUES (1874,40228.4914435675,40234.1803107545,707,NULL)
INSERT INTO SalesInvoice VALUES (1875,40228.9735179484,40235.780128738,155,NULL)
INSERT INTO SalesInvoice VALUES (1876,40230.4093520302,40264.2849181013,584,NULL)
INSERT INTO SalesInvoice VALUES (1877,40231.2662265975,40264.2139861346,530,NULL)
	     